
Partial Class Maestro

    Private m_MasterConsumablesList() As CaliperLifeSciences.Shared.Consumables.clsConsumable
    Private consumablesAPI As CaliperLifeSciences.Shared.Consumables.clsConsumablesAPI

    Public Enum PlatformType
        ZEPHYR
        SCICLONE
    End Enum

    Private m_DeckLayoutList As System.Collections.ArrayList
    Private m_PlatformType As PlatformType

    Public Sub UpdateConsumableOnDeck()

        Dim targetLayoutName As String
        Dim newConsumable As String
        Dim deckID As String

        Dim plateName As String ' Whatever the new plateName you want to give

        Try
            ' These four variables must be set in Maestro application
            targetLayoutName = Me.GetParameterVariableValue("p_DeckLayout")
            newConsumable = Me.GetParameterVariableValue("p_NewConsumableName")
            plateName = Me.GetParameterVariableValue("p_PlateName")
            deckID = Me.GetParameterVariableValue("p_DeckID")

            If targetLayoutName.Equals(String.Empty) Then
                Exit Sub
            ElseIf plateName.Trim().Equals(String.Empty) Or newConsumable.Trim.Equals(String.Empty) Then
                RemoveDeckMaterialFromLayout(deckID, targetLayoutName)
            Else
                UpdateDeckLayout(targetLayoutName, newConsumable, deckID, plateName)
            End If


        Catch ex As Exception
            Me.RaiseErrorFromVSTA(-9999, "UpdateConsumableOnDeck", "Unable to update deck to use new consumable. Please check to make sure all input parameters are valid.")
        End Try

    End Sub

    ' Call this method if you expect the location to be empty after executing instruction
    Public Sub RemoveDeckMaterialFromLayout()

        Dim targetLayoutName As String
        Dim deckID As String = String.Empty 'A1...C4

        ' May need to get platform variable from Maestro. Right now use SCICLONE as default
        Dim myPlatform As PlatformType = PlatformType.SCICLONE

        ' These three variables must be set in Maestro application
        targetLayoutName = Me.GetParameterVariableValue("p_DeckLayout")
        deckID = Me.GetParameterVariableValue("p_DeckID")

        RemoveDeckMaterialFromLayout(deckID, targetLayoutName)
    End Sub

    Private Sub RemoveDeckMaterialFromLayout(ByVal deckID As String, ByVal targetLayoutName As String)
        Me.RemoveDeckMaterial(deckID, targetLayoutName)
    End Sub

    ' Reset deck layout back to original state.
    ' Should only use this method at the end of Maestro application
    ' It will store information about each deck
    Public Sub ResetDeckLayout()

        Dim deckLoc As String = String.Empty
        Dim layoutName As String = String.Empty
        Dim rowAlpha As String = "A"
        Dim deckVarID As String
        Dim plateName As String = String.Empty

        Dim deckLayout As CDeckLayout
        Dim targetLayoutName As String = String.Empty

        deckLayout = New CDeckLayout()
        Dim deckLayoutDictionary As New System.Collections.Generic.Dictionary(Of String, String)
        Dim kvp As System.Collections.Generic.KeyValuePair(Of String, String)

        plateName = "PlateName"

        targetLayoutName = Me.GetParameterVariableValue("p_DeckLayout")

        Try
            For Each deckLayout In m_DeckLayoutList
                layoutName = deckLayout.DeckLayoutName
                deckLayoutDictionary = deckLayout.DeckLayoutDictionary

                If targetLayoutName.Equals(layoutName) Then
                    For Each kvp In deckLayoutDictionary
                        deckLoc = kvp.Key
                        deckVarID = kvp.Value

                        ' TO DO - CHANGE THIS VALUE TO FIT YOUR CONSUMABLE
                        plateName = "PlateName" & deckLoc

                        UpdateDeckLayout(layoutName, deckVarID, deckLoc, plateName)
                    Next

                    Exit For ' We have found our decklayout so get out of this loop. 
                End If
            Next
        Catch ex As Exception
            Me.RaiseErrorFromVSTA(-9999, "ResetDeckLayout", "Unable to reset deck layout for deck: " & targetLayoutName)
        End Try

    End Sub

    ' This method should be called at the very beginning of each method if you want
    ' to restore the original deck layout when application is finished.
    Public Sub SaveDeckLayout()

        Dim deckLoc As String = String.Empty
        Dim layoutName As String = String.Empty
        Dim rowAlpha As String = "A"
        Dim deckVarID As String

        Dim nRow As Integer = 3 ' Default to Zephyr
        Dim nCol As Integer = 4 ' Default to Zephyr
        Dim i As Integer
        Dim j As Integer

        Dim deckLayout As CDeckLayout
        deckLayout = New CDeckLayout()
        Dim deckLayoutDictionary As New System.Collections.Generic.Dictionary(Of String, String)

        layoutName = Me.GetParameterVariableValue("p_DeckLayout")

        ' Information about the deck is already saved in the collection
        If DeckLayoutExist(layoutName) = False Then
            Exit Sub
        End If

        If m_PlatformType = PlatformType.SCICLONE Then
            nRow = 4
            nCol = 5
        End If

        For i = 1 To nRow
            If i = 1 Then
                rowAlpha = "A"
            ElseIf i = 2 Then
                rowAlpha = "B"
            ElseIf i = 3 Then
                rowAlpha = "C"
            ElseIf i = 4 Then
                rowAlpha = "D"
            End If
            For j = 1 To nCol
                deckLoc = rowAlpha & j
                deckVarID = Me.GetDeckVarIDAtLocation(deckLoc, layoutName)
                If Not deckVarID.Equals(String.Empty) Then
                    deckLayoutDictionary.Add(deckLoc, deckVarID)
                End If

            Next
        Next

        ' Save to dictionary for retrieval later
        deckLayout.DeckLayoutName = layoutName
        deckLayout.DeckLayoutDictionary = deckLayoutDictionary

        If m_DeckLayoutList Is Nothing Then
            m_DeckLayoutList = New System.Collections.ArrayList()
        End If
        m_DeckLayoutList.Add(deckLayout)

    End Sub

    Private Sub UpdateDeckLayout(ByVal targetLayoutName As String, ByVal newConsumable As String, ByVal deckNumber As Integer, ByVal plateName As String)

        Dim myPlatform As PlatformType = PlatformType.SCICLONE

        Dim deckID As String = String.Empty 'A1...C4
        deckID = LookUpDeckNumberID(deckNumber, myPlatform)

        UpdateDeckLayout(targetLayoutName, newConsumable, deckID, plateName)

    End Sub

    Private Sub UpdateDeckLayout(ByVal targetLayoutName As String, ByVal newConsumable As String, ByVal deckID As String, ByVal plateName As String)
        Try

            Dim isErr As Boolean = False

            ' May need to get platform variable from Maestro. Right now use SCICLONE as default
            Dim myPlatform As PlatformType = PlatformType.SCICLONE

            Dim newCons As CaliperLifeSciences.Shared.Consumables.clsConsumable
            Dim macroCons As CaliperLifeSciences.VSTA.MaestroMacroHost.clsConsumable

            If Not targetLayoutName Is String.Empty And Not newConsumable Is String.Empty _
            And Not deckID Is String.Empty Then

                consumablesAPI = New CaliperLifeSciences.Shared.Consumables.clsConsumablesAPI
                consumablesAPI.Get_All_Consumables(m_MasterConsumablesList)

                newCons = GetMasterConsumable(newConsumable)
                If Not newCons Is Nothing Then

                    ' Remove the old consumable first
                    Me.RemoveDeckMaterial(deckID, targetLayoutName)
                    'newCons = GetMasterConsumable(newConsumable)

                    macroCons = Me.NewConsumable()
                    macroCons.ConsumableID = newCons.ConsumableID
                    macroCons.BottomShape = newCons.BottomShape
                    macroCons.Category = newCons.Category
                    macroCons.ColumnOffset = newCons.ColumnOffset
                    macroCons.ColumnPitch = newCons.ColumnPitch
                    macroCons.Columns = newCons.Columns
                    macroCons.ConsumableDescription = newCons.ConsumableDescription
                    macroCons.ConsumableName = newCons.ConsumableName
                    macroCons.Density = newCons.Density
                    ' macroCons.dVdH = masterConsumable.dVdH
                    macroCons.Height = newCons.Height
                    macroCons.InsideLength = newCons.InsideLength
                    macroCons.InsideWidth = newCons.InsideWidth
                    macroCons.Length = newCons.Length
                    macroCons.Manufacturer = newCons.Manufacturer
                    macroCons.RowOffset = newCons.RowOffset
                    macroCons.RowPitch = newCons.RowPitch
                    macroCons.Rows = newCons.Rows
                    macroCons.T2StackingHeight = newCons.T2StackingHeight
                    macroCons.T2StackingOffset = newCons.T2StackingOffset
                    macroCons.TopHeight = newCons.TopHeight
                    macroCons.TopLength = newCons.TopLength
                    macroCons.TopWidth = newCons.TopWidth
                    macroCons.WellsDepth = newCons.WellsDepth
                    macroCons.WellsDiameter = newCons.WellsDiameter
                    macroCons.WellShape = newCons.WellShape
                    macroCons.Width = newCons.Width

                    Dim test As Integer = Convert.ToDecimal(newCons.dVdH)

                    Me.AddDeckMaterialToNamedLayout(deckID, macroCons, plateName, targetLayoutName)

                    ' We need to set this variable so that ConsumableQuery function knows
                    'Me.SetGlobalVariableValue("fend_PlateNameFor" & deckNumberID, newConsumable) '

                    macroCons = Nothing
                End If
            Else
                Me.RaiseErrorFromVSTA(-9998, "UpdateDeckLayout", "Unable to find decklayout = " & targetLayoutName & "AND/OR consumable = " & newConsumable & " . Please check to make sure the consumable exists in the database")
            End If

        Catch ex As Exception
            Me.RaiseErrorFromVSTA(-9999, "UpdateDeckLayout", "Unable to change decklayout. Error Message: " & ex.InnerException.ToString())
        End Try
    End Sub

    Private Function DeckLayoutExist(ByVal layoutName As String) As Boolean

        Dim rt As Boolean = False ' Assume doesn't  exist in the collection

        If m_DeckLayoutList IsNot Nothing Then
            For Each deckLayout As CDeckLayout In m_DeckLayoutList
                If deckLayout.DeckLayoutName = layoutName Then
                    rt = True '
                    Exit For
                End If
            Next
        End If
        Return rt
    End Function

    Public Function GetMasterConsumable(ByRef Name As String) As CaliperLifeSciences.Shared.Consumables.clsConsumable
        Dim nIndex As Short
        Dim cons As CaliperLifeSciences.Shared.Consumables.clsConsumable = Nothing
        For nIndex = 1 To UBound(m_MasterConsumablesList) - 1
            If m_MasterConsumablesList(nIndex).ConsumableName = Name Then
                cons = m_MasterConsumablesList(nIndex)
            End If
        Next nIndex

        Return cons
    End Function
End Class
