Imports System.Collections.Generic

Public Class CDeckPopulation

    Private m_DeckLayoutList As SortedList(Of Integer, CDeck)
    Private m_ApplicationName As String

    Public Sub New()
        m_DeckLayoutList = New SortedList(Of Integer, CDeck)
    End Sub

    Public Sub InitializeMajorSteps(ByVal NumberOfMajorSteps As Integer, ByVal MajorStepNames As String(), ByVal ApplicationName As String)
        'Initialize the data structure for the indicated number of major steps
        Dim i, j As Integer
        m_ApplicationName = ApplicationName
        For i = 1 To NumberOfMajorSteps
            Dim myDeck As New CDeck
            myDeck.MajorStepName = MajorStepNames(i)
            m_DeckLayoutList.Add(i, myDeck)
            For j = 0 To 23 'Initialize all Deck Locations to be Unused
                AddConsumableToMajorStep(i, LookupDeckNumberID(j), Maestro.eLocationType.Unused, String.Empty, String.Empty, 0, False)
            Next
            myDeck = Nothing
        Next
    End Sub

    Public Property ApplicationName() As String
        Get
            ApplicationName = m_ApplicationName
        End Get
        Set(ByVal value As String)
            m_ApplicationName = value
        End Set
    End Property

    Public Property DeckList() As SortedList(Of Integer, CDeck)
        Get
            Return m_DeckLayoutList
        End Get
        Set(ByVal value As SortedList(Of Integer, CDeck))
            m_DeckLayoutList = value
        End Set
    End Property

    Public Sub AddConsumableToMajorStep(ByVal MajorStep As Integer, ByVal DeckPosition As String, ByVal LocationType As Maestro.eLocationType, ByVal ConsumableName As String, ByVal MaestroConsumableName As String, ByVal NumberOfStackedConsumables As Integer, ByVal Lid As Boolean)
        'Add a consumable to a deck position for the indicated Major Step
        Dim myDeckLocation As CDeckLocation

        myDeckLocation = New CDeckLocation
        myDeckLocation.DeckPosition = DeckPosition
        myDeckLocation.LocationType = LocationType
        myDeckLocation.Lid = False
        If ConsumableName <> String.Empty Then
            myDeckLocation.NumberOfStackedConsumables = NumberOfStackedConsumables
        Else
            'We don't have a Consumable, so we want NumberOfStackedConsumables to be 0
            myDeckLocation.NumberOfStackedConsumables = 0
        End If

        'If ConsumableName <> String.Empty And DoesConsumableExist(ConsumableName) Then
        myDeckLocation.ConsumableName = ConsumableName
        myDeckLocation.MaestroConsumableName = MaestroConsumableName
        myDeckLocation.Lid = Lid
        Dim GraphicPath As String = System.IO.Path.Combine(Environment.GetFolderPath(Environment.SpecialFolder.CommonApplicationData), _maestro.GetGlobalVariableValue("g_DeckLayoutFolder"))
        Dim GraphicFile As String = System.IO.Path.Combine(GraphicPath, MajorStep & "\" & DeckPosition)
        'Check first to determine if a png file exists, if it doesn't check if a jpg version exists
        If System.IO.File.Exists(GraphicFile & ".png") Then
            myDeckLocation.ImageFilename = GraphicFile & ".png"
        ElseIf System.IO.File.Exists(GraphicFile & ".jpg") Then
            myDeckLocation.ImageFilename = GraphicFile & ".jpg"
        End If
        Dim CustomDescriptionFile As String = System.IO.Path.Combine(GraphicPath, MajorStep & "\" & DeckPosition & ".txt")
        If System.IO.File.Exists(CustomDescriptionFile) Then
            myDeckLocation.CustomDescription = System.IO.File.ReadAllText(CustomDescriptionFile)
        End If
        'End If
        'Remove item if key already exists
        If m_DeckLayoutList(MajorStep).DeckLayoutList.ContainsKey(myDeckLocation.DeckPositionNumber) Then
            m_DeckLayoutList(MajorStep).DeckLayoutList.Remove(myDeckLocation.DeckPositionNumber)
        End If

        m_DeckLayoutList(MajorStep).DeckLayoutList.Add(myDeckLocation.DeckPositionNumber, myDeckLocation)
        myDeckLocation = Nothing
    End Sub

    Public Sub RemoveConsumableFromMajorStep(ByVal MajorStep As Integer, ByVal DeckLocation As String)

        If m_DeckLayoutList(MajorStep).DeckLayoutList.ContainsKey(LookupDeckNumber(DeckLocation)) Then
            m_DeckLayoutList(MajorStep).DeckLayoutList.Remove(LookupDeckNumber(DeckLocation))
        End If

    End Sub

    Public Function LookupDeckNumberID(ByVal deckNum As Integer) As String
        Dim deckNumberID As String = String.Empty

        If deckNum < 6 Then
            deckNumberID = "A" & Format(deckNum)
        ElseIf deckNum < 12 Then
            deckNumberID = "B" & Format(deckNum) - 6
        ElseIf deckNum < 18 Then
            deckNumberID = "C" & Format(deckNum) - 12
        ElseIf deckNum < 24 Then
            deckNumberID = "D" & Format(deckNum) - 18
        End If

        Return deckNumberID
    End Function

    Public Function LookupDeckNumber(ByVal deckID As String) As Integer
        Dim DeckNumber As Integer

        DeckNumber = Convert.ToInt32(deckID.Substring(1, 1))
        If DeckNumber < 0 Or DeckNumber > 5 Then
            DeckNumber = -1
        Else
            Select Case deckID.Substring(0, 1)
                Case "A"
                    DeckNumber = DeckNumber
                Case "B"
                    DeckNumber = DeckNumber + 6
                Case "C"
                    DeckNumber = DeckNumber + 12
                Case "D"
                    DeckNumber = DeckNumber + 18
                Case Else
                    DeckNumber = -1
            End Select
        End If
        Return DeckNumber
    End Function

    Public Function MajorStepExist(ByVal MajorStep As Integer) As Boolean
        If m_DeckLayoutList.ContainsKey(MajorStep) Then
            MajorStepExist = True
        Else
            MajorStepExist = False
        End If
    End Function

End Class
