Imports System
Imports System.Collections.Generic

Partial Class Maestro

    Public Enum eLocationType
        Unused
        PlateLocator
        RecessedPosition
        CPAC_Microplate96
        CPAC_Microplate384
        CPAC_Reservoir
        CPAC_Deepwell
        CPAC_Unused
        Magnet
        MagnetWithSpacer
        Shaker96
        Shaker384
        ShakerUnused
        TipsWasteChute
    End Enum

    Public Sub PopulateMaestroDeckForStep(ByVal MajorStep As Integer, ByVal LayoutName As String)

        Dim newCons As CaliperLifeSciences.Shared.Consumables.clsConsumable
        Dim macroCons As CaliperLifeSciences.VSTA.MaestroMacroHost.clsConsumable

        If consumablesAPI Is Nothing Then
            consumablesAPI = New CaliperLifeSciences.Shared.Consumables.clsConsumablesAPI
        End If
        If m_MasterConsumablesList Is Nothing Then
            consumablesAPI.Get_All_Consumables(m_MasterConsumablesList)
        End If

        For Each kvp As KeyValuePair(Of Integer, CDeckLocation) In m_DeckPopulation.DeckList(MajorStep).DeckLayoutList
            If kvp.Value.DeckPosition <> "D5" Then 'Don't remove Tips Waste Chute
                Me.RemoveDeckMaterial(kvp.Value.DeckPosition, LayoutName)
            End If
            If kvp.Value.ConsumableName <> String.Empty Then
                newCons = GetMasterConsumable(kvp.Value.ConsumableName)
                If Not newCons Is Nothing Then
                    macroCons = Me.NewConsumable()
                    macroCons.ConsumableID = newCons.ConsumableID
                    macroCons.BottomShape = newCons.BottomShape
                    macroCons.Category = newCons.Category
                    macroCons.ColumnOffset = newCons.ColumnOffset
                    macroCons.ColumnPitch = newCons.ColumnPitch
                    macroCons.Columns = newCons.Columns
                    macroCons.ConsumableDescription = newCons.ConsumableDescription
                    macroCons.ConsumableName = newCons.ConsumableName
                    macroCons.Density = newCons.Density
                    macroCons.Height = newCons.Height
                    macroCons.InsideLength = newCons.InsideLength
                    macroCons.InsideWidth = newCons.InsideWidth
                    macroCons.Length = newCons.Length
                    macroCons.Manufacturer = newCons.Manufacturer
                    macroCons.RowOffset = newCons.RowOffset
                    macroCons.RowPitch = newCons.RowPitch
                    macroCons.Rows = newCons.Rows
                    macroCons.T2StackingHeight = newCons.T2StackingHeight
                    macroCons.T2StackingOffset = newCons.T2StackingOffset
                    macroCons.TopHeight = newCons.TopHeight
                    macroCons.TopLength = newCons.TopLength
                    macroCons.TopWidth = newCons.TopWidth
                    macroCons.WellsDepth = newCons.WellsDepth
                    macroCons.WellsDiameter = newCons.WellsDiameter
                    macroCons.WellShape = newCons.WellShape
                    macroCons.Width = newCons.Width
                    macroCons.Color = newCons.Color
                    macroCons.NotchLL = newCons.NotchLL
                    macroCons.NotchLR = newCons.NotchLR
                    macroCons.NotchUL = newCons.NotchUL
                    macroCons.NotchUR = newCons.NotchUR

                    Me.AddDeckMaterialToNamedLayout(kvp.Value.DeckPosition, macroCons, kvp.Value.MaestroConsumableName, LayoutName, kvp.Value.MaestroConsumableDescription, kvp.Value.NumberOfStackedConsumables)
                Else
                    Throw New Exception("Consumable: " & kvp.Value.ConsumableName & " for deck position " & kvp.Value.DeckPosition & " does not exist in the Consumables Database.  CDeckPopulation.AddConsumableToMajorStep")
                End If
            End If
        Next kvp
    End Sub

    Public Function LookupLocationType(ByVal LocationType As String) As eLocationType
        Select Case LocationType.ToUpper
            Case "Unused".ToUpper
                LookupLocationType = eLocationType.Unused
            Case "PlateLocator".ToUpper
                LookupLocationType = eLocationType.PlateLocator
            Case "RecessedPosition".ToUpper
                LookupLocationType = eLocationType.RecessedPosition
            Case "CPAC_Microplate96".ToUpper
                LookupLocationType = eLocationType.CPAC_Microplate96
            Case "CPAC_Microplate384".ToUpper
                LookupLocationType = eLocationType.CPAC_Microplate384
            Case "CPAC_Reservoir".ToUpper
                LookupLocationType = eLocationType.CPAC_Reservoir
            Case "CPAC_Deepwell".ToUpper
                LookupLocationType = eLocationType.CPAC_Deepwell
            Case "CPAC_Unused".ToUpper
                LookupLocationType = eLocationType.CPAC_Unused
            Case "Magnet".ToUpper
                LookupLocationType = eLocationType.Magnet
            Case "MagnetWithSpacer".ToUpper
                LookupLocationType = eLocationType.MagnetWithSpacer
            Case "Shaker96".ToUpper
                LookupLocationType = eLocationType.Shaker96
            Case "Shaker384".ToUpper
                LookupLocationType = eLocationType.Shaker384
            Case "ShakerUnused".ToUpper
                LookupLocationType = eLocationType.ShakerUnused
            Case "TipsWasteChute".ToUpper
                LookupLocationType = eLocationType.TipsWasteChute
            Case Else
                Me.RaiseErrorFromVSTA(1, "LookupLocationType()", "LocationType '" & LocationType & "' is invalid.")
        End Select
    End Function

    Public Function CheckLocation(ByVal Location As String) As Boolean

        Dim ScicloneLocations As String() = {"A0", "A1", "A2", "A3", "A4", "A5", "B0", "B1", "B2", "B3", "B4", "B5", "C0", "C1", "C2", "C3", "C4", "C5", "D0", "D1", "D2", "D3", "D4", "D5"}
        Dim found As Boolean = False

        For Each x As String In ScicloneLocations
            If x = Location Then
                found = True
            End If
        Next
        CheckLocation = found
    End Function

End Class