Imports System.IO
Imports System.Windows.Forms
Imports System.Collections

Friend Class frmDeckLayout
    Private Const DECKLAYOUT_ERROR_TITLE As String = "Application Deck Layout Error"
    Private _DeckPosition As Integer
    Private _AppGraphicsPath As String
    Private _MajorStep As Integer
    Private _DeckPopulation As CDeckPopulation
    Private _DeckPBlist As New System.Collections.Generic.SortedList(Of Integer, PictureBox)

    Private _ConsumableName As String
    Private _DeckLocation As String
    Private _ConsumableDescription As String
    Private _KeyPressLock As New Object

    Public Sub New()
        ' This call is required by the Windows Form Designer.
        InitializeComponent()

        ' Add any initialization after the InitializeComponent() call.
        _AppGraphicsPath = ""

        'Set up initial conditions for controls
        Me.lblConsumableName.Visible = False
        Me.lblDeckLocation.Visible = False
        Me.txtDeckLocationInfo.Visible = False
        Me.PB_Selected.Visible = False
        Me.lblExplanation.Text = "Please verify that all proper consumables, reagents and tips are present." & Environment.NewLine & Environment.NewLine & "Click on the deck locations to provide additional detail."
        Me.lblExplanation.MaximumSize = New System.Drawing.Size(400, 0)
        Me.lblExplanation.AutoSize = True
        Me.lblExplanation.Visible = True

    End Sub

    Public Property ConsumableName() As String
        Set(ByVal value As String)
            _ConsumableName = value
        End Set
        Get
            ConsumableName = _ConsumableName
        End Get
    End Property

    Public Property DeckLocation() As String
        Set(ByVal value As String)
            _DeckLocation = value
        End Set
        Get
            DeckLocation = _DeckLocation
        End Get
    End Property

    Public WriteOnly Property ConsumableDescription() As String
        Set(ByVal value As String)
            _ConsumableDescription = value
            txtDeckLocationInfo.Text = value
            txtDeckLocationInfo.Visible = True
        End Set
    End Property

    Private Sub PrepDeckLocationInfoAndDisplay(ByVal DeckPosition As Integer)
        Try
            'Set up picture and text for an individual Deck Location

            If _DeckPopulation.DeckList(_MajorStep).DeckLayoutList.ContainsKey(DeckPosition - 1) Then

                lblApplicationName.Visible = False
                lblStepName.Visible = False

                Dim SelectedPB As System.Windows.Forms.PictureBox = Nothing
                _DeckPBlist.TryGetValue(DeckPosition - 1, SelectedPB)

                _DeckPosition = DeckPosition - 1

                Me.lblExplanation.Visible = False
                ' Set vertical scroll bar to true
                txtDeckLocationInfo.ScrollBars = ScrollBars.Vertical

                Dim filePath As String = String.Empty

                If _DeckPopulation.DeckList(_MajorStep).DeckLayoutList(DeckPosition - 1).NumberOfStackedConsumables > 0 Then
                    'This location has a consumable

                    ConsumableName = _DeckPopulation.DeckList(_MajorStep).DeckLayoutList(_DeckPosition).ConsumableName
                    DeckLocation = _DeckPopulation.DeckList(_MajorStep).DeckLayoutList(_DeckPosition).DeckPosition

                    ' display graphic
                    Dim filename As String = String.Empty
                    'Check if Application writer has set an image to use via the VSTA code
                    If _DeckPopulation.DeckList(_MajorStep).DeckLayoutList(_DeckPosition).ImageFilename <> String.Empty Then
                        filename = _DeckPopulation.DeckList(_MajorStep).DeckLayoutList(_DeckPosition).ImageFilename
                        filePath = System.IO.Path.Combine(_AppGraphicsPath, filename)
                    End If
                    'Check if there's a photo in the Method Step Folder
                    If Not System.IO.File.Exists(filePath) Then
                        filename = ConsumableDensityAndCategory(_DeckPopulation.DeckList(_MajorStep).DeckLayoutList(_DeckPosition).ConsumableName) & ".png"
                        filePath = System.IO.Path.Combine(_AppGraphicsPath, filename)
                    End If
                    If System.IO.File.Exists(filePath) Then
                        filePath = System.IO.Path.Combine(_AppGraphicsPath, filename)
                        If System.IO.File.Exists(filePath) Then
                            Me.imgSelectedLocation.Visible = True
                            imgSelectedLocation.Image = DeckUtils.ReplaceImage(Me.imgSelectedLocation.Image, filePath)
                        End If
                    End If

                    'Set the text to be used in the Description field
                    Dim NoSCDescription As String
                    If _DeckPopulation.DeckList(_MajorStep).DeckLayoutList(_DeckPosition).NumberOfStackedConsumables = 1 Then
                        NoSCDescription = "Number of Consumables: "
                    Else
                        NoSCDescription = "Number of Stacked Consumables: "
                    End If

                    Dim DescriptionField As String
                    DescriptionField = Environment.NewLine & _
                    "Consumable Density and Category: " & ConsumableDensityAndCategory(_DeckPopulation.DeckList(_MajorStep).DeckLayoutList(_DeckPosition).ConsumableName) & Environment.NewLine & Environment.NewLine

                    DescriptionField = DescriptionField & NoSCDescription & _DeckPopulation.DeckList(_MajorStep).DeckLayoutList(_DeckPosition).NumberOfStackedConsumables & Environment.NewLine & Environment.NewLine

                    If _DeckPopulation.DeckList(_MajorStep).DeckLayoutList(_DeckPosition).MaestroConsumableDescription <> String.Empty Then
                        DescriptionField = DescriptionField & _
                        _DeckPopulation.DeckList(_MajorStep).DeckLayoutList(_DeckPosition).MaestroConsumableDescription & Environment.NewLine & Environment.NewLine
                    End If

                    If _DeckPopulation.DeckList(_MajorStep).DeckLayoutList(_DeckPosition).CustomDescription <> String.Empty Then
                        DescriptionField = DescriptionField & _
                        _DeckPopulation.DeckList(_MajorStep).DeckLayoutList(_DeckPosition).CustomDescription & Environment.NewLine & Environment.NewLine
                    End If

                    ConsumableDescription = DescriptionField

                    'Set the blue border around the selected picturebox
                    SelectedBorder(SelectedPB)

                    txtDeckLocationInfo.Focus()
                    txtDeckLocationInfo.DeselectAll()

                Else 'We're some type of location without a consumable
                    ConsumableDescription = ""
                    ConsumableName = ""

                    DeckLocation = _DeckPopulation.DeckList(_MajorStep).DeckLayoutList(_DeckPosition).DeckPosition
                    ' display graphic
                    Dim filename As String = String.Empty
                    'Check if Application writer has set an image to use via the VSTA code
                    If _DeckPopulation.DeckList(_MajorStep).DeckLayoutList(_DeckPosition).ImageFilename <> String.Empty Then
                        filename = _DeckPopulation.DeckList(_MajorStep).DeckLayoutList(_DeckPosition).ImageFilename
                        filePath = System.IO.Path.Combine(_AppGraphicsPath, filename)
                    End If
                    'Check if there's a photo in the Method Step Folder
                    If Not System.IO.File.Exists(filePath) Then
                        filename = LocationFilename(_DeckPopulation.DeckList(_MajorStep).DeckLayoutList(_DeckPosition).LocationType)
                        filePath = System.IO.Path.Combine(_AppGraphicsPath, filename)
                    End If
                    If System.IO.File.Exists(filePath) Then
                        filePath = System.IO.Path.Combine(_AppGraphicsPath, filename)
                        If System.IO.File.Exists(filePath) Then
                            Me.imgSelectedLocation.Visible = True
                            imgSelectedLocation.Image = DeckUtils.ReplaceImage(Me.imgSelectedLocation.Image, filePath)
                        End If
                    End If

                    'Set the red border around the selected picturebox
                    SelectedBorder(SelectedPB)

                End If

                If ConsumableName <> String.Empty Then
                    lblConsumableName.Text = DeckLocation & " : " & ConsumableName
                Else
                    lblConsumableName.Text = DeckLocation
                End If
                lblConsumableName.Visible = True

                lblDeckLocation.Text = LocationTypeName(_DeckPopulation.DeckList(_MajorStep).DeckLayoutList(_DeckPosition).LocationType)
                lblDeckLocation.Visible = True
            End If
        Catch ex As Exception
            ' this is necessary because modal window delays appearance of Alarm Manager alarm
            MessageBox.Show(Me, "Error in frmDeckLayout.PrepDeckLocationInfoAndDisplay: " & ex.Message & vbCrLf & vbCrLf & _
                "NOTE: When given the opportunity, Stop Execution of the application and call Caliper Service.", _
                DECKLAYOUT_ERROR_TITLE, MessageBoxButtons.OK, MessageBoxIcon.Exclamation)
            ' since this is an unusual occurence, asking the user to
            ' manually Stop Execution of the application is a reasonable, low-risk solution
            _maestro.RaiseErrorFromVSTA(-9999, "frmDeckLayout.PrepDeckLocationInfoAndDisplay", ex.Message & vbCrLf & _
                "NOTE: Please Stop Execution of the application and call Caliper Service.")
        End Try
    End Sub

    Private Sub SelectedBorder(ByRef SelectedPB As PictureBox)
        PB_Selected.Height = SelectedPB.Height + 6
        PB_Selected.Width = SelectedPB.Width + 6
        PB_Selected.Top = SelectedPB.Top - 3
        PB_Selected.Left = SelectedPB.Left - 3
        PB_Selected.SendToBack()
        PB_Selected.Visible = True
    End Sub

    Private Function LocationFilename(ByVal LocationType As Maestro.eLocationType) As String
        Dim fileName As String = "Empty.png"

        Select Case LocationType
            Case Maestro.eLocationType.CPAC_Deepwell
                fileName = "CPAC Deepwell Insert.png"
            Case Maestro.eLocationType.CPAC_Microplate96
                fileName = "CPAC 96 Microplate Insert.png"
            Case Maestro.eLocationType.CPAC_Microplate384
                fileName = "CPAC 384 Microplate Insert.png"
            Case Maestro.eLocationType.CPAC_Reservoir
                fileName = "CPAC Reservoir Insert.png"
            Case Maestro.eLocationType.CPAC_Unused
                fileName = "CPAC No Insert.png"
            Case Maestro.eLocationType.Magnet
                fileName = "Magnet.png"
            Case Maestro.eLocationType.MagnetWithSpacer
                fileName = "Magnet With Spacer.png"
            Case Maestro.eLocationType.PlateLocator
                fileName = "Plate Locator.png"
            Case Maestro.eLocationType.Unused
                fileName = "Unused.png"
            Case Maestro.eLocationType.RecessedPosition
                fileName = "Recessed Position.png"
            Case Maestro.eLocationType.Shaker96
                fileName = "Shaker 96.png"
            Case Maestro.eLocationType.Shaker384
                fileName = "Shaker 384.png"
            Case Maestro.eLocationType.ShakerUnused
                fileName = "Shaker No Insert.png"
            Case Maestro.eLocationType.TipsWasteChute
                fileName = "Tips Waste Chute.png"
            Case Else
                fileName = "Empty.png"
        End Select
        Return fileName
    End Function

    Private Function LocationTypeName(ByVal LocationType As Maestro.eLocationType) As String
        Dim Name As String = "Unused"

        Select Case LocationType
            Case Maestro.eLocationType.CPAC_Deepwell
                Name = "CPAC with Deepwell Insert"
            Case Maestro.eLocationType.CPAC_Microplate96
                Name = "CPAC with 96 Microplate Insert"
            Case Maestro.eLocationType.CPAC_Microplate384
                Name = "CPAC with 384 Microplate Insert"
            Case Maestro.eLocationType.CPAC_Reservoir
                Name = "CPAC with Reservoir Insert"
            Case Maestro.eLocationType.CPAC_Unused
                Name = "CPAC Unused"
            Case Maestro.eLocationType.Magnet
                Name = "Magnet (no spacer)"
            Case Maestro.eLocationType.MagnetWithSpacer
                Name = "Magnet with Spacer"
            Case Maestro.eLocationType.PlateLocator
                Name = "Plate Locator"
            Case Maestro.eLocationType.Unused
                Name = "Unused"
            Case Maestro.eLocationType.RecessedPosition
                Name = "Recessed Position"
            Case Maestro.eLocationType.Shaker96
                Name = "Shaker with 96 insert"
            Case Maestro.eLocationType.Shaker384
                Name = "Shaker with 384 insert"
            Case Maestro.eLocationType.ShakerUnused
                Name = "Shaker Unused"
            Case Maestro.eLocationType.TipsWasteChute
                Name = "Tips Waste Chute"
            Case Else
                Name = "Unused"
        End Select
        Return Name
    End Function

    Private Sub FillDeckLocation(ByVal deckPosition As Integer, ByRef PictBox As System.Windows.Forms.PictureBox)
        Dim fileName As String
        Dim filePath As String

        'Code for setting the image for the deck location
        If _DeckPopulation.DeckList(_MajorStep).DeckLayoutList.ContainsKey(deckPosition) Then
            If _DeckPopulation.DeckList(_MajorStep).DeckLayoutList(deckPosition).NumberOfStackedConsumables > 0 Then
                fileName = ConsumableDensityAndCategory(_DeckPopulation.DeckList(_MajorStep).DeckLayoutList(deckPosition).ConsumableName) & ".png"
            Else
                fileName = LocationFilename(_DeckPopulation.DeckList(_MajorStep).DeckLayoutList(deckPosition).LocationType)
            End If
        Else
            fileName = "Empty.png"
        End If
        filePath = System.IO.Path.Combine(_AppGraphicsPath, fileName)
        If System.IO.File.Exists(filePath) Then
            PictBox.Image = DeckUtils.ReplaceImage(PictBox.Image, filePath)
            If _DeckPopulation.DeckList(_MajorStep).DeckLayoutList.ContainsKey(deckPosition) Then
                Dim g As System.Drawing.Graphics
                g = System.Drawing.Graphics.FromImage(PictBox.Image)
                Dim mystring As String
                If _DeckPopulation.DeckList(_MajorStep).DeckLayoutList(deckPosition).NumberOfStackedConsumables > 1 Then
                    mystring = _DeckPopulation.DeckList(_MajorStep).DeckLayoutList(deckPosition).NumberOfStackedConsumables
                    g.DrawString(mystring, New System.Drawing.Font("Microsoft Sans Serif", 55, Drawing.FontStyle.Regular), System.Drawing.Brushes.DarkOrange, 43, 8)
                End If
                If _DeckPopulation.DeckList(_MajorStep).DeckLayoutList(deckPosition).Lid Then
                    mystring = "Lid"
                    g.DrawString(mystring, New System.Drawing.Font("Microsoft Sans Serif", 30, Drawing.FontStyle.Bold), System.Drawing.Brushes.DarkOrange, 90, 2)
                End If
            End If
        Else
            'Our image doesn't exist- let's draw a thin border
            PictBox.BorderStyle = BorderStyle.FixedSingle
        End If

        'Position 23 is the Tips Waste Chute
        If deckPosition = 23 Then
            fileName = "Tips Waste Chute.png"
            filePath = System.IO.Path.Combine(_AppGraphicsPath, fileName)
            If System.IO.File.Exists(filePath) Then
                PictBox.Image = DeckUtils.ReplaceImage(PictBox.Image, filePath)
            End If
        End If

    End Sub

    Public Sub PrepareContentForDisplay(ByVal GraphicsPath As String, ByVal MajorStep As Integer, ByRef DeckPopulation As CDeckPopulation)
        Try
            Dim deckPosition As Integer

            _MajorStep = MajorStep
            _DeckPopulation = DeckPopulation
            _AppGraphicsPath = GraphicsPath

            'make the form full-screen and center contents
            Me.Top = 0
            Me.Left = 0

            'add the PB's to the List
            _DeckPBlist.Add(0, Me.imgDeckA0)
            _DeckPBlist.Add(1, Me.imgDeckA1)
            _DeckPBlist.Add(2, Me.imgDeckA2)
            _DeckPBlist.Add(3, Me.imgDeckA3)
            _DeckPBlist.Add(4, Me.imgDeckA4)
            _DeckPBlist.Add(5, Me.imgDeckA5)
            _DeckPBlist.Add(6, Me.imgDeckB0)
            _DeckPBlist.Add(7, Me.imgDeckB1)
            _DeckPBlist.Add(8, Me.imgDeckB2)
            _DeckPBlist.Add(9, Me.imgDeckB3)
            _DeckPBlist.Add(10, Me.imgDeckB4)
            _DeckPBlist.Add(11, Me.imgDeckB5)
            _DeckPBlist.Add(12, Me.imgDeckC0)
            _DeckPBlist.Add(13, Me.imgDeckC1)
            _DeckPBlist.Add(14, Me.imgDeckC2)
            _DeckPBlist.Add(15, Me.imgDeckC3)
            _DeckPBlist.Add(16, Me.imgDeckC4)
            _DeckPBlist.Add(17, Me.imgDeckC5)
            _DeckPBlist.Add(18, Me.imgDeckD0)
            _DeckPBlist.Add(19, Me.imgDeckD1)
            _DeckPBlist.Add(20, Me.imgDeckD2)
            _DeckPBlist.Add(21, Me.imgDeckD3)
            _DeckPBlist.Add(22, Me.imgDeckD4)
            _DeckPBlist.Add(23, Me.imgDeckD5)

            If _DeckPopulation.DeckList.ContainsKey(_MajorStep) Then

                lblApplicationName.Visible = True
                lblStepName.Visible = True

                lblApplicationName.Text = "Application: " & _DeckPopulation.ApplicationName
                If _DeckPopulation.DeckList(MajorStep).MajorStepName <> String.Empty Then
                    lblStepName.Text = _DeckPopulation.DeckList(MajorStep).MajorStepName
                End If

                For deckPosition = 0 To 23
                    Dim pb As System.Windows.Forms.PictureBox = Nothing
                    _DeckPBlist.TryGetValue(deckPosition, pb)
                    If Not pb Is Nothing Then
                        FillDeckLocation(deckPosition, pb)
                    End If
                Next
            Else
                Throw New Exception("MajorStep #" & _MajorStep & " does not exist.  The application developer needs to modify the VSTA code to utilize the correct Method Step." & Environment.NewLine & "frmDeckLayout: PrepareContentForDisplay")
            End If

        Catch ex As Exception
            ' NOTE: Errors will be handled by caller.
            Err.Raise(Err.Number, "frmDeckLayout.PrepareContentForDisplay", Err.Description)
        End Try

    End Sub

    Private Sub cmdOK_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles cmdOK.Click
        Dim DeckConfirmationPhotoForm As frmDeckConfirmationPhoto
        Try
            Me.Hide()

            If DoesEntireDeckPhotoExist = True Then
                DeckConfirmationPhotoForm = New frmDeckConfirmationPhoto
                DeckConfirmationPhotoForm.ShowDialog(_maestro.HostWindow)
                DeckConfirmationPhotoForm.Dispose()
            End If
        Catch ex As Exception
            ' this is necessary because modal window delays appearance of Alarm Manager alarm
            MessageBox.Show(Me, "Error in frmDeckLayout.cmdOK_Click: " & ex.Message & vbCrLf & vbCrLf & _
                "NOTE: When given the opportunity, Stop Execution of the application and call Caliper Service.", _
                DECKLAYOUT_ERROR_TITLE, MessageBoxButtons.OK, MessageBoxIcon.Exclamation)
            ' since this is an unusual occurence, asking the user to
            ' manually Stop Execution of the application is a reasonable, low-risk solution
            _maestro.RaiseErrorFromVSTA(-9999, "frmDeckLayout.cmdOK_Click", ex.Message & vbCrLf & _
                "NOTE: Please Stop Execution of the application and call Caliper Service.")
        Finally
            Close()
        End Try
    End Sub

    Private Sub cmdCancel_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles cmdCancel.Click
        Try
            _maestro.SetGlobalVariableValue("g_ApplicationAborted", True)
        Catch ex As Exception
            _maestro.SetGlobalVariableValue("g_ApplicationAborted", True)
            _maestro.RaiseErrorFromVSTA(-9999, "Deck Layout Form", "Unable to write to global variables:  g_NumberOfColumnsToProcess or g_IsAdapterPreBroadcast")
        Finally
            Me.Hide()
            Close()
        End Try
    End Sub

    Private Sub imgDeckA0_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles imgDeckA0.Click
        PrepDeckLocationInfoAndDisplay(1)
    End Sub

    Private Sub imgDeckA1_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles imgDeckA1.Click
        PrepDeckLocationInfoAndDisplay(2)
    End Sub

    Private Sub imgDeckA2_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles imgDeckA2.Click
        PrepDeckLocationInfoAndDisplay(3)
    End Sub

    Private Sub imgDeckA3_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles imgDeckA3.Click
        PrepDeckLocationInfoAndDisplay(4)
    End Sub

    Private Sub imgDeckA4_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles imgDeckA4.Click
        PrepDeckLocationInfoAndDisplay(5)
    End Sub

    Private Sub imgDeckA5_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles imgDeckA5.Click
        PrepDeckLocationInfoAndDisplay(6)
    End Sub

    Private Sub imgDeckB0_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles imgDeckB0.Click
        PrepDeckLocationInfoAndDisplay(7)
    End Sub

    Private Sub imgDeckB1_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles imgDeckB1.Click
        PrepDeckLocationInfoAndDisplay(8)
    End Sub

    Private Sub imgDeckB2_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles imgDeckB2.Click
        PrepDeckLocationInfoAndDisplay(9)
    End Sub

    Private Sub imgDeckB3_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles imgDeckB3.Click
        PrepDeckLocationInfoAndDisplay(10)
    End Sub

    Private Sub imgDeckB4_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles imgDeckB4.Click
        PrepDeckLocationInfoAndDisplay(11)
    End Sub

    Private Sub imgDeckB5_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles imgDeckB5.Click
        PrepDeckLocationInfoAndDisplay(12)
    End Sub

    Private Sub imgDeckC0_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles imgDeckC0.Click
        PrepDeckLocationInfoAndDisplay(13)
    End Sub

    Private Sub imgDeckC1_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles imgDeckC1.Click
        PrepDeckLocationInfoAndDisplay(14)
    End Sub

    Private Sub imgDeckC2_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles imgDeckC2.Click
        PrepDeckLocationInfoAndDisplay(15)
    End Sub

    Private Sub imgDeckC3_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles imgDeckC3.Click
        PrepDeckLocationInfoAndDisplay(16)
    End Sub

    Private Sub imgDeckC4_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles imgDeckC4.Click
        PrepDeckLocationInfoAndDisplay(17)
    End Sub

    Private Sub imgDeckC5_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles imgDeckC5.Click
        PrepDeckLocationInfoAndDisplay(18)
    End Sub

    Private Sub imgDeckD0_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles imgDeckD0.Click
        PrepDeckLocationInfoAndDisplay(19)
    End Sub

    Private Sub imgDeckD1_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles imgDeckD1.Click
        PrepDeckLocationInfoAndDisplay(20)
    End Sub

    Private Sub imgDeckD2_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles imgDeckD2.Click
        PrepDeckLocationInfoAndDisplay(21)
    End Sub

    Private Sub imgDeckD3_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles imgDeckD3.Click
        PrepDeckLocationInfoAndDisplay(22)
    End Sub

    Private Sub imgDeckD4_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles imgDeckD4.Click
        PrepDeckLocationInfoAndDisplay(23)
    End Sub

    Private Sub imgDeckD5_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles imgDeckD5.Click
        PrepDeckLocationInfoAndDisplay(24)
    End Sub

    Private Enum KeyStrokeDirection
        Up = 0
        Down
        Left
        Right
    End Enum

    Private Sub frmDeckLayout_KeyDown(ByVal sender As Object, ByVal e As System.Windows.Forms.KeyEventArgs) Handles Me.KeyDown
        SyncLock _KeyPressLock
            Select Case e.KeyCode
                Case System.Windows.Forms.Keys.Down
                    PrepNext(KeyStrokeDirection.Down)
                Case System.Windows.Forms.Keys.Up
                    PrepNext(KeyStrokeDirection.Up)
                Case System.Windows.Forms.Keys.Left
                    PrepNext(KeyStrokeDirection.Left)
                Case System.Windows.Forms.Keys.Right
                    PrepNext(KeyStrokeDirection.Right)
            End Select
            '"Swallow" the keystroke- so that the control with the current focus isn't affected
            e.Handled = True
        End SyncLock
    End Sub

    Private Sub PrepNext(ByVal Direction As KeyStrokeDirection)
        Dim NewPosition As Integer
        Try
            NewPosition = GetNext(_DeckPosition, Direction)

            While DetermineIfValidPosition(NewPosition) = False
                NewPosition = GetNext(NewPosition, Direction)
            End While

            PrepDeckLocationInfoAndDisplay(NewPosition + 1)
        Catch ex As Exception
            ' this is necessary because modal window delays appearance of Alarm Manager alarm
            MessageBox.Show(Me, "Error in frmDeckLayout.PrepNext: " & ex.Message & vbCrLf & vbCrLf & _
                "NOTE: When given the opportunity, Stop Execution of the application and call Caliper Service.", _
                DECKLAYOUT_ERROR_TITLE, MessageBoxButtons.OK, MessageBoxIcon.Exclamation)
            ' since this is an unusual occurence, asking the user to
            ' manually Stop Execution of the application is a reasonable, low-risk solution
            _maestro.RaiseErrorFromVSTA(-9999, "frmDeckLayout.PrepNext", ex.Message & vbCrLf & _
                "NOTE: Please Stop Execution of the application and call Caliper Service.")
        End Try
    End Sub

    Private Function DetermineIfValidPosition(ByVal Position As Integer) As Boolean
        Dim ContainsKey As Boolean = False

        ContainsKey = _DeckPopulation.DeckList(_MajorStep).DeckLayoutList.ContainsKey(Position)
        If ContainsKey = True Then
            DetermineIfValidPosition = True
        Else
            DetermineIfValidPosition = False
        End If

    End Function

    Private Function GetNext(ByVal CurrentPostion As Integer, ByVal Direction As KeyStrokeDirection) As Integer
        Dim newPosition As Integer

        Select Case Direction
            Case KeyStrokeDirection.Down
                If CurrentPostion < 18 Then
                    newPosition = CurrentPostion + 6
                Else
                    newPosition = CurrentPostion - 18
                End If
            Case KeyStrokeDirection.Up
                If CurrentPostion > 5 Then
                    newPosition = CurrentPostion - 6
                Else
                    newPosition = CurrentPostion + 18
                End If
            Case KeyStrokeDirection.Left
                If CurrentPostion > 0 Then
                    newPosition = CurrentPostion - 1
                Else
                    newPosition = CurrentPostion + 23
                End If
            Case KeyStrokeDirection.Right
                If CurrentPostion < 23 Then
                    newPosition = CurrentPostion + 1
                Else
                    newPosition = CurrentPostion - 23
                End If
        End Select
        GetNext = newPosition
    End Function

    Private ReadOnly Property DoesEntireDeckPhotoExist() As Boolean
        Get
            Dim EntireDeckPhoto As String
            EntireDeckPhoto = System.IO.Path.Combine(Environment.GetFolderPath(Environment.SpecialFolder.CommonApplicationData), _maestro.GetGlobalVariableValue("g_DeckLayoutFolder"))
            EntireDeckPhoto = System.IO.Path.Combine(EntireDeckPhoto, _maestro.GetGlobalVariableValue("g_MajorStep"))
            EntireDeckPhoto = System.IO.Path.Combine(EntireDeckPhoto, "EntireDeckPhoto")

            If System.IO.File.Exists(EntireDeckPhoto & ".png") Then
                DoesEntireDeckPhotoExist = True
            ElseIf System.IO.File.Exists(EntireDeckPhoto & ".jpg") Then
                DoesEntireDeckPhotoExist = True
            Else
                DoesEntireDeckPhotoExist = False
            End If
        End Get
    End Property

End Class