Public Class CDeckInfo

    Private Const TEACH_BLOCK_HEIGHT As Double = 14.35 ' mm
    Private Const TOLERANCE As Double = 2.0F ' mm
    Private Const TIP_LENGTH As Integer = 52 'mm

    ' Measured by puting two Tip Box on any of the recess deck + Teach Plate. Then measure to deck level
    Private Const RECESS_DECK_OFFSET As Integer = 38 ' mm

    ' This class stores x,y,z and deckID of a deck. In the future we can expand this class 
    Private m_DeckID As String
    Private m_DeckNumber As Integer

    Private m_LeftGripper As CLeftGripper
    Private m_GantryHead As CGantry

    ' The number of stacked consumables for this deck location. 
    ' Initially we get this information from consumable database
    Private m_NumStackedPlates As Integer

    ' The height of the consumable at this deck location.
    ' We get this information from consumable database
    Private m_ConsumableHeight As Double

    ' The taugh position for this deck location.
    ' We get this information from Maestro
    Private m_TaughtCoordinate As CCoordinate

    ' This mininum height safe to pick plate
    Private m_MinSafePickHeight As Double


    ' Empty deck with no stacked plate OR Tip Box
    Private m_IsEmpty As Boolean

    Private m_LayoutName As String

    Private m_IsConflict As Boolean

    Private m_IsSunkenDeckNGS As Boolean

    Public Property DeckID() As String
        Get
            Return m_DeckID
        End Get
        Set(ByVal value As String)
            m_DeckID = value
        End Set
    End Property

    Public Property DeckNumber() As Integer
        Get
            Return m_DeckNumber
        End Get
        Set(ByVal value As Integer)
            m_DeckNumber = value
        End Set
    End Property

    Public Sub New()

        m_GantryHead = New CGantry
        m_LeftGripper = New CLeftGripper

        m_TaughtCoordinate = New CCoordinate
    End Sub


    Public Sub New(ByVal deckID As String)

        Me.New()
        m_DeckID = deckID

    End Sub

    Public Property NumStackedPlates() As Integer
        Get
            Return m_NumStackedPlates
        End Get
        Set(ByVal value As Integer)
            m_NumStackedPlates = value
        End Set

    End Property

    Public Property ConsumableHeight() As Double
        Get
            Return m_ConsumableHeight
        End Get

        Set(ByVal value As Double)
            m_ConsumableHeight = value
        End Set
    End Property


    Public Property TaughtCoordinate() As CCoordinate
        Get
            Return m_TaughtCoordinate
        End Get
        Set(ByVal value As CCoordinate)
            m_TaughtCoordinate = value
        End Set
    End Property

    Public Property LayoutName() As String
        Get
            Return m_LayoutName
        End Get
        Set(ByVal value As String)
            m_LayoutName = value
        End Set
    End Property
    Public Function IsSafeToPick() As Boolean

        Dim isSafe As Boolean = False


        Return isSafe

    End Function

    Public Function PickZHeight() As Double

        Dim tmpConsumableHeight As Double
        Dim zHeight As Double
        Dim gantryHeadAtTeachBlockLevel As Double


        tmpConsumableHeight = m_ConsumableHeight
        ' Note we need to take the negative TIP_LENGTH because of how Maestro interprets coordinate
        gantryHeadAtTeachBlockLevel = m_TaughtCoordinate.ZCoordinate + (-TIP_LENGTH)

        ' Because the taught position is higher for these locations we need an offset.
        ' Based on experiment this value is ~38mm.
        If m_IsSunkenDeckNGS = True Then ' Bring to the deck level

            If (Me.DeckID = "A0" Or Me.DeckID = "B0" Or Me.DeckID = "C0" Or Me.DeckID = "D0") _
            Or (Me.DeckID = "A1" Or Me.DeckID = "B1" Or Me.DeckID = "C1" Or Me.DeckID = "D1") Then
                gantryHeadAtTeachBlockLevel = gantryHeadAtTeachBlockLevel + (-RECESS_DECK_OFFSET)
            End If

        ElseIf m_IsSunkenDeckNGS = False Then ' Bring to the deck level
            gantryHeadAtTeachBlockLevel = gantryHeadAtTeachBlockLevel + (-TEACH_BLOCK_HEIGHT)
            tmpConsumableHeight = 0
        End If

        zHeight = gantryHeadAtTeachBlockLevel + (tmpConsumableHeight)

        Return zHeight

    End Function

    Public ReadOnly Property IsEmpty() As Boolean

        Get
            Return m_IsEmpty
        End Get

    End Property

    Public Property IsConflict() As Boolean
        Get
            Return m_IsConflict
        End Get
        Set(ByVal value As Boolean)
            m_IsConflict = value
        End Set
    End Property


#Region "Left Gripper Coordinate"
    Public Property LeftGripperX() As Single
        Get
            Return m_LeftGripper.XCoordinate
        End Get
        Set(ByVal value As Single)
            m_LeftGripper.XCoordinate = value
        End Set
    End Property

    Public Property LeftGripperY() As Single
        Get
            Return m_LeftGripper.YCoordinate
        End Get
        Set(ByVal value As Single)
            m_LeftGripper.YCoordinate = value
        End Set
    End Property

    Public Property LeftGripperZ() As Single
        Get
            Return m_LeftGripper.ZCoordinate
        End Get
        Set(ByVal value As Single)
            m_LeftGripper.ZCoordinate = value
        End Set
    End Property
#End Region

#Region "Gantry Head Coordinate"
    Public Property GantryHeadX() As Single
        Get
            Return m_GantryHead.XCoordinate
        End Get
        Set(ByVal value As Single)
            m_GantryHead.XCoordinate = value
        End Set
    End Property

    Public Property GantryHeadY() As Single
        Get
            Return m_GantryHead.YCoordinate
        End Get
        Set(ByVal value As Single)
            m_GantryHead.YCoordinate = value
        End Set
    End Property

    Public Property GantryHeadZ() As Single
        Get
            Return m_GantryHead.ZCoordinate
        End Get
        Set(ByVal value As Single)
            m_GantryHead.ZCoordinate = value
        End Set
    End Property
#End Region


    Public Property IsSunkenDeckNGS() As Boolean
        Get
            Return m_IsSunkenDeckNGS
        End Get
        Set(ByVal value As Boolean)
            m_IsSunkenDeckNGS = value
        End Set
    End Property

End Class
