Imports System.Collections.Generic

Public Class CIndex

    Private m_SampleInfo As List(Of CSample)

    ' The largest barcode index
    Private m_MaxBarcodeIndex As Integer

    ' The smallest barcode index
    Private m_MinBarcodeIndex As Integer

    ' If one-to-one mapping then we load all tips at once
    Private m_IsOneToOneMap As Boolean

    ' The starting dispense column
    Private m_DestinationWellColumnStart As Integer

    ' The starting aspirate column
    Private m_SourceWellColumnStart As Integer

    Private m_MaxIndexID As Integer
    Private m_MinIndexID As Integer

    Public Sub AddSample(ByVal sample As CSample)

        If sample Is Nothing Then
            Exit Sub
        End If
        If m_SampleInfo Is Nothing Then
            m_SampleInfo = New List(Of CSample)
        End If
        m_SampleInfo.Add(sample)

    End Sub

    Public Sub RemoveSample(ByVal sample As CSample)
        If m_SampleInfo Is Nothing Then
            Exit Sub
        End If

        If m_SampleInfo.Count <= 0 Or sample Is Nothing Then
            Exit Sub
        End If

        For Each s As CSample In m_SampleInfo
            If s.SampleID = sample.SampleID Then
                m_SampleInfo.Remove(sample)
                Exit Sub
            End If
        Next
    End Sub

    Public Function GetSampleBasedOnSampleID(ByVal sampleID As String) As CSample
        Return GetSample(sampleID)
    End Function

    Public Function GetListOFSamplePoolID(ByVal poolID As Integer) As List(Of CSample)
        Return GetSampleList(poolID)
    End Function

    Public Function GetListOfSampleBarcodeID(ByVal barcodeID As Integer) As List(Of CSample)
        Return GetSampleList(barcodeID)
    End Function

    Public ReadOnly Property NumberOfSamples() As Integer
        Get
            Return m_SampleInfo.Count
        End Get
    End Property

    Private Function GetSampleList(ByVal srchID As Integer) As List(Of CSample)
        Dim sampleList As List(Of CSample) = New List(Of CSample)

        For Each s As CSample In m_SampleInfo
            If s.IndexID = srchID Then
                sampleList.Add(s)
            End If
        Next

        Return sampleList
    End Function

    Private Function GetSample(ByVal srchString As String) As CSample
        Dim sample As CSample = Nothing
        For Each s As CSample In m_SampleInfo
            If s.SourceDeckID.DeckID = srchString Then
                sample = s
                Exit For
            End If
        Next
        Return sample
    End Function

    Public ReadOnly Property SampleList() As List(Of CSample)
        Get
            Return m_SampleInfo
        End Get
    End Property

    Public Property MaxBarcodeIndex() As Integer
        Get
            Return m_MaxBarcodeIndex
        End Get
        Set(ByVal value As Integer)
            m_MaxBarcodeIndex = value
        End Set
    End Property

    Public Property MinBarcodeIndex() As Integer
        Get
            Return m_MinBarcodeIndex
        End Get
        Set(ByVal value As Integer)
            m_MinBarcodeIndex = value
        End Set
    End Property

    Public Property IsOneToOneMap() As Boolean
        Get
            Return m_IsOneToOneMap
        End Get
        Set(ByVal value As Boolean)
            m_IsOneToOneMap = value
        End Set
    End Property

    Public Property DestinationWellColumnStart() As Integer
        Get
            Return m_DestinationWellColumnStart
        End Get
        Set(ByVal value As Integer)
            m_DestinationWellColumnStart = value
        End Set
    End Property

    Public Property SourceWellColumnStart() As Integer
        Get
            Return m_SourceWellColumnStart
        End Get
        Set(ByVal value As Integer)
            m_SourceWellColumnStart = value
        End Set
    End Property

    Public Sub New()
        m_SampleInfo = Nothing
    End Sub

End Class
