Imports System.Collections
Imports System.Collections.Generic

Public Class CPlates

    'Private m_ExcelObj As Microsoft.Office.Interop.Excel.Application
    Private m_Workbook As String
    Private m_Worksheet As String
    Private m_NumberOfColumns As Integer
    Private m_NumberOfRows As Integer
    Private m_NumberofSamples As Integer
    Private m_PlateList As List(Of CPlateLayout)
    ' Holds mapping of plate name (key) and  Sciclone deck (value)
    Private m_PlateMapList As Hashtable
    Private m_frmPlates As frmPlates
    'Private m_frmSimplePlatesInfo As frmSimplePlateInfo
    Private m_IsCancel As Boolean
    Private m_HostWindow As CaliperLifeSciences.VSTA.MaestroMacroHost.WindowWrapper
    Private m_frmSampleInformation As frmSampleInformation

    Public Sub ReadPlateLayoutSpreadSheet(ByVal workBookName As String, ByVal sheetName As String, ByVal isShowSimplePlateInfo As Boolean)

        ' ---------------------------------------------------------------------------
        '                                RULES
        '
        ' Sample Information:
        ' ------------------
        ' 1. Number of Columns = (2,4) <--- Row 2, Col 4
        ' 2. Number of Rows    = (3,4)
        ' 3. Total Number of Samples = (4,4)
        '
        '
        ' 1st Plate Layout Information:
        ' ----------------------------
        ' 1. Plate Name = (6,1)
        ' 2. Reagent Name = (8,2) ---> (8,13) <---- 12 columns
        ' 3. Layout = (10,2) ---> (10,13) start
        '             (17,2) ---> (17,13) end
        '
        '
        ' ALL subsequents plates must follow equations below:
        ' 
        ' 1. Platename offset = 6 + 15*i
        ' 2. ReagentName offset = 8 + 15*i
        ' 3. Layout offset (start) 10 + 15*i
        '                  (end)   17 + 15*i
        ' Where i = 1, 2,....n
        ' ---------------------------------------------------------------------------
        If Not System.IO.File.Exists(workBookName) Then
            System.Windows.Forms.MessageBox.Show("Unable to locate Excel file.")
            Exit Sub
        End If

        Dim COLUMN_START_CONST As Integer = 2
        Dim COLUMN_END_CONST As Integer = 13
        Dim PLATE_ROW_START_CONST As Integer = 6
        Dim REAGENT_ROW_START_CONST As Integer = 8
        Dim LAYOUT_ROW_START_CONST As Integer = 10
        Dim LAYOUT_ROW_END_CONST As Integer = 17

        Dim rowIndex As Integer
        Dim rowStart As Integer = 10
        Dim rowEnd As Integer = 17

        Dim colIndex As Integer

        Dim constantVal As Integer = 15
        Dim deltaPlate As Integer
        Dim deltaReagent As Integer
        Dim deltaLayoutStart As Integer
        Dim deltaLayoutEnd As Integer

        Dim plateName As String
        Dim reagentName As String
        Dim wellList As New List(Of Double)

        ' Read the sheet
        Dim i As Integer
        Dim k As Integer
        Dim j As Integer = 0 ' Must be initialized to 0.

        Dim lastExcelRow As Integer
        Dim numPlates As Integer

        Dim tmpVar As String

        Dim plateLayout As CPlateLayout
        Dim reagent As CReagent

        Dim priorReagentName As String = Nothing
        Dim priorReagent As CReagent = Nothing
        Dim plateNameToMaestroDeckID As String = String.Empty

        Dim excelObj As Object = CreateObject("Excel.Application")
        Dim objWorkBook As Object
        Dim objWorkSheet As Object
        Dim sheet As Object

        'IH130614 opening this results in an exception 'Old file type ' 
        'IH130618 solved: 
        'see http://support.microsoft.com/kb/320369/en-us
        ' (solusion as in LEXAddOn.vb: OpenWorkbook())

        Dim oldCI As System.Globalization.CultureInfo = _
                System.Threading.Thread.CurrentThread.CurrentCulture
        System.Threading.Thread.CurrentThread.CurrentCulture = _
            New System.Globalization.CultureInfo("en-US")

        objWorkBook = excelObj.Workbooks.Open(workBookName)

        'System.Threading.Thread.CurrentThread.CurrentCulture = oldCI

        objWorkBook.Activate()
        objWorkSheet = objWorkBook.Sheets()

        Dim fileDate As String = String.Empty

        ' Make sure to clear all plates from this list before adding
        m_PlateList.Clear()

        For i = 1 To objWorkSheet.Count
            sheet = objWorkSheet(i)

            If sheet.Name.ToUpper().Equals(sheetName.ToUpper()) Then

                fileDate = sheet.Cells(1, 2).Value

                ' 1. Determine the number of reagent plates
                lastExcelRow = sheet.UsedRange.Rows.Count ' sheet.Cells.SpecialCells(Microsoft.Office.Interop.Excel.XlCellType.xlCellTypeLastCell).Row()
                deltaPlate = PLATE_ROW_START_CONST + constantVal * j
                For k = 1 To lastExcelRow

                    If k Mod deltaPlate = 0 And sheet.Cells(deltaPlate, 1).Value IsNot Nothing Then
                        numPlates = numPlates + 1
                        j = j + 1
                        deltaPlate = PLATE_ROW_START_CONST + constantVal * j
                    End If
                Next

                ' 3. For each plate get the layout
                j = 0 ' Reset j index to 0
                For j = 0 To numPlates - 1

                    plateLayout = New CPlateLayout
                    ' 2. Get Sample Information - see spreadsheet or note above as to why
                    ' we get these numbers
                    plateLayout.NumberOfColumns = sheet.Cells(2, 4).Value
                    m_NumberOfColumns = plateLayout.NumberOfColumns

                    plateLayout.NumberOfRows = sheet.Cells(3, 4).Value
                    m_NumberOfRows = plateLayout.NumberOfRows

                    plateLayout.NumberOfSamples = sheet.Cells(4, 4).Value
                    m_NumberofSamples = plateLayout.NumberOfSamples

                    deltaPlate = PLATE_ROW_START_CONST + constantVal * j
                    deltaReagent = REAGENT_ROW_START_CONST + constantVal * j
                    deltaLayoutStart = LAYOUT_ROW_START_CONST + constantVal * j
                    deltaLayoutEnd = LAYOUT_ROW_END_CONST + constantVal * j

                    plateName = sheet.Cells(deltaPlate, 1).Value
                    plateLayout.PlateName = plateName
                    Try
                        plateNameToMaestroDeckID = sheet.Cells(deltaPlate + 1, 1).value.Split(":")(1).Trim()
                        plateLayout.PlateDeckID = plateNameToMaestroDeckID
                    Catch ex As Exception
                        ' if the spread sheet does not have sciclone deck defined then
                        ' force to show detailed screen
                        isShowSimplePlateInfo = False
                    End Try

                    ' Get Plate Layout Information
                    For colIndex = COLUMN_START_CONST To COLUMN_END_CONST

                        reagentName = sheet.Cells(deltaReagent, colIndex).Value

                        reagent = New CReagent()
                        ' Fill in reagent info for this column
                        reagent.ReagentName = reagentName
                        reagent.StartColumn = colIndex - 1

                        ' Does this reagent span multiple columns
                        If priorReagentName = reagentName Then
                            reagent.IsSpanMultiColumns = True
                            priorReagent.IsSpanMultiColumns = True
                        End If
                        priorReagentName = reagentName
                        priorReagent = reagent

                        ' Read the cells value
                        For rowIndex = deltaLayoutStart To deltaLayoutEnd
                            tmpVar = sheet.Cells(rowIndex, colIndex).Value
                            If tmpVar IsNot Nothing Then
                                If tmpVar.ToString().Trim().Equals(String.Empty) Or tmpVar.ToString().Trim().Equals("0") Then
                                    ' Empty string or '0'
                                    tmpVar = -9999
                                Else
                                    ' Split string using ' ' as the delimiter then remove the label "UL"
                                    ' to get the volume
                                    tmpVar = tmpVar.ToUpper().Split(" ")(0)
                                    If tmpVar.Contains("UL") Then
                                        tmpVar = tmpVar.Replace("UL", String.Empty).Trim()
                                    End If

                                End If

                            Else ' "Nothing"
                                tmpVar = -9999
                            End If
                            wellList.Add(tmpVar)

                        Next ' Go to next row

                        ' Save reagent volume information
                        reagent.WellList = wellList
                        reagent.InitializeReagentVolume()

                        If j = 0 Then ' First Plate
                            ' Now we are finished with 1st column of the layout...Add the list to frmReagentControl
                            m_frmPlates.BuildPlateInfo(PLATE_NUMBER.PLATE1, plateName, reagentName, colIndex, wellList)
                        ElseIf j = 1 Then ' Second Plate
                            m_frmPlates.BuildPlateInfo(PLATE_NUMBER.PLATE2, plateName, reagentName, colIndex, wellList)
                        ElseIf j = 2 Then ' Third Plate
                            m_frmPlates.BuildPlateInfo(PLATE_NUMBER.PLATE3, plateName, reagentName, colIndex, wellList)
                        ElseIf j = 3 Then ' Fourth Plate
                            m_frmPlates.BuildPlateInfo(PLATE_NUMBER.PLATE4, plateName, reagentName, colIndex, wellList)
                        ElseIf j = 4 Then ' Fifth Plate
                            m_frmPlates.BuildPlateInfo(PLATE_NUMBER.PLATE5, plateName, reagentName, colIndex, wellList)
                        Else

                        End If

                        ' Add this reagent to plate
                        plateLayout.AddReagent(reagent)

                        ' Clear this list for the next column
                        wellList.Clear()

                    Next ' Go to next  column

                    ' Add this plate to the list
                    m_PlateList.Add(plateLayout)

                    ' Mapping of plate to Maestro Deck
                    m_PlateMapList.Add(plateName, plateNameToMaestroDeckID)
                Next ' Go to next plate

                ' We have found our sheet. Now get out of this loop
                Exit For
            End If
        Next

        objWorkBook.Close()
        excelObj.Quit()
        excelObj = Nothing
        objWorkBook = Nothing


        Dim dlgResult As System.Windows.Forms.DialogResult = Windows.Forms.DialogResult.None
        m_IsCancel = False
        If isShowSimplePlateInfo Then
            'm_frmSimplePlatesInfo = New frmSimplePlateInfo(m_NumberOfRows, m_NumberOfColumns, m_NumberofSamples, m_PlateMapList)
            'dlgResult = m_frmSimplePlatesInfo.ShowDialog(m_HostWindow)

            m_frmSampleInformation = New frmSampleInformation(m_NumberOfRows, m_NumberOfColumns, m_NumberofSamples, m_Workbook, fileDate)

            dlgResult = m_frmSampleInformation.ShowDialog(m_HostWindow)
            If dlgResult = Windows.Forms.DialogResult.Cancel Then
                m_IsCancel = True
            End If
        Else
            'If numPlates = 1 Then
            '    m_frmPlates.TabCtrlPlateLayouts.TabPages.Remove(m_frmPlates.TabPage2)
            'End If

            excelObj = Nothing
            'objWorkBook = Nothing

            m_frmPlates.MapList = m_PlateMapList

            m_frmPlates.NumberOfColumns = m_NumberOfColumns
            m_frmPlates.NumberOfRows = m_NumberOfRows
            m_frmPlates.NumberOfSamples = m_NumberofSamples

            While (Not dlgResult = System.Windows.Forms.DialogResult.OK)
                dlgResult = m_frmPlates.ShowDialog(m_HostWindow)
            End While

            If dlgResult = Windows.Forms.DialogResult.OK Then
                m_IsCancel = m_frmPlates.Cancel
                If m_IsCancel = False Then
                    ' Get back mapping between Plate name and Sciclone deck
                    m_PlateMapList = m_frmPlates.MapList

                    ' Now map deck ID from m_PlateMapList to m_PlateList
                    MapDeckIDFromPlateMapListToPlateList()
                End If
            End If
        End If

    End Sub

    ' This function is basically the same as the one above.
    ' The different is that it requires the user add Excel reference to the project. 
    ' The advantage of using this function is it will not close an open workbook.
    'Public Sub ReadPlateLayoutSpreadSheet(ByVal workBookName As String, ByVal sheetName As String, ByVal isShowSimplePlateInfo As Boolean)

    '    ' ---------------------------------------------------------------------------
    '    '                                RULES
    '    '
    '    ' Sample Information:
    '    ' ------------------
    '    ' 1. Number of Columns = (2,4) <--- Row 2, Col 4
    '    ' 2. Number of Rows    = (3,4)
    '    ' 3. Total Number of Samples = (4,4)
    '    '
    '    '
    '    ' 1st Plate Layout Information:
    '    ' ----------------------------
    '    ' 1. Plate Name = (6,1)
    '    ' 2. Reagent Name = (8,2) ---> (8,13) <---- 12 columns
    '    ' 3. Layout = (10,2) ---> (10,13) start
    '    '             (17,2) ---> (17,13) end
    '    '
    '    '
    '    ' ALL subsequents plates must follow equations below:
    '    ' 
    '    ' 1. Platename offset = 6 + 15*i
    '    ' 2. ReagentName offset = 8 + 15*i
    '    ' 3. Layout offset (start) 10 + 15*i
    '    '                  (end)   17 + 15*i
    '    ' Where i = 1, 2,....n
    '    ' ---------------------------------------------------------------------------
    '    If Not System.IO.File.Exists(workBookName) Then
    '        System.Windows.Forms.MessageBox.Show("Unable to locate Excel file.")
    '        Exit Sub
    '    End If

    '    Dim workBook As Microsoft.Office.Interop.Excel.Workbook
    '    Dim workSheet As Microsoft.Office.Interop.Excel.Sheets
    '    Dim sheet As Microsoft.Office.Interop.Excel.Worksheet

    '    Dim COLUMN_START_CONST As Integer = 2
    '    Dim COLUMN_END_CONST As Integer = 13
    '    Dim PLATE_ROW_START_CONST As Integer = 6
    '    Dim REAGENT_ROW_START_CONST As Integer = 8
    '    Dim LAYOUT_ROW_START_CONST As Integer = 10
    '    Dim LAYOUT_ROW_END_CONST As Integer = 17

    '    Dim rowIndex As Integer
    '    Dim rowStart As Integer = 10
    '    Dim rowEnd As Integer = 17

    '    Dim colIndex As Integer

    '    Dim constantVal As Integer = 15
    '    Dim deltaPlate As Integer
    '    Dim deltaReagent As Integer
    '    Dim deltaLayoutStart As Integer
    '    Dim deltaLayoutEnd As Integer

    '    Dim plateName As String
    '    Dim reagentName As String
    '    Dim wellList As New List(Of Double)

    '    ' Read the sheet
    '    Dim i As Integer
    '    Dim k As Integer
    '    Dim j As Integer = 0 ' Must be initialized to 0.

    '    Dim lastExcelRow As Integer
    '    Dim numPlates As Integer

    '    Dim tmpVar As String

    '    Dim plateLayout As CPlateLayout
    '    Dim reagent As CReagent

    '    Dim priorReagentName As String = Nothing
    '    Dim priorReagent As CReagent = Nothing
    '    Dim plateNameToMaestroDeckID As String = String.Empty

    '    m_ExcelObj = New Microsoft.Office.Interop.Excel.Application

    '    workBook = m_ExcelObj.Workbooks.Open(workBookName)
    '    workSheet = workBook.Sheets()

    '    ' Make sure to clear all plates from this list before adding
    '    m_PlateList.Clear()

    '    For i = 1 To workSheet.Count
    '        sheet = workSheet(i)

    '        If sheet.Name.ToUpper().Equals(sheetName.ToUpper()) Then

    '            ' 1. Determine the number of reagent plates
    '            lastExcelRow = sheet.Cells.SpecialCells(Microsoft.Office.Interop.Excel.XlCellType.xlCellTypeLastCell).Row()
    '            deltaPlate = PLATE_ROW_START_CONST + constantVal * j
    '            For k = 1 To lastExcelRow

    '                If k Mod deltaPlate = 0 And sheet.Cells(deltaPlate, 1).Value IsNot Nothing Then
    '                    numPlates = numPlates + 1
    '                    j = j + 1
    '                    deltaPlate = PLATE_ROW_START_CONST + constantVal * j
    '                End If
    '            Next

    '            ' 3. For each plate get the layout
    '            j = 0 ' Reset j index to 0
    '            For j = 0 To numPlates - 1

    '                plateLayout = New CPlateLayout
    '                ' 2. Get Sample Information - see spreadsheet or note above as to why
    '                ' we get these numbers
    '                plateLayout.NumberOfColumns = sheet.Cells(2, 4).Value
    '                m_NumberOfColumns = plateLayout.NumberOfColumns

    '                plateLayout.NumberOfRows = sheet.Cells(3, 4).Value
    '                m_NumberOfRows = plateLayout.NumberOfRows

    '                plateLayout.NumberOfSamples = sheet.Cells(4, 4).Value
    '                m_NumberofSamples = plateLayout.NumberOfSamples

    '                deltaPlate = PLATE_ROW_START_CONST + constantVal * j
    '                deltaReagent = REAGENT_ROW_START_CONST + constantVal * j
    '                deltaLayoutStart = LAYOUT_ROW_START_CONST + constantVal * j
    '                deltaLayoutEnd = LAYOUT_ROW_END_CONST + constantVal * j

    '                plateName = sheet.Cells(deltaPlate, 1).Value
    '                plateLayout.PlateName = plateName
    '                Try
    '                    plateNameToMaestroDeckID = sheet.Cells(deltaPlate + 1, 1).value.Split(":")(1).Trim()
    '                    plateLayout.PlateDeckID = plateNameToMaestroDeckID
    '                Catch ex As Exception
    '                    ' if the spread sheet does not have sciclone deck defined then
    '                    ' force to show detailed screen
    '                    isShowSimplePlateInfo = False
    '                End Try

    '                ' Get Plate Layout Information
    '                For colIndex = COLUMN_START_CONST To COLUMN_END_CONST

    '                    reagentName = sheet.Cells(deltaReagent, colIndex).Value

    '                    reagent = New CReagent()
    '                    ' Fill in reagent info for this column
    '                    reagent.ReagentName = reagentName
    '                    reagent.StartColumn = colIndex - 1

    '                    ' Does this reagent span multiple columns
    '                    If priorReagentName = reagentName Then
    '                        reagent.IsSpanMultiColumns = True
    '                        priorReagent.IsSpanMultiColumns = True
    '                    End If
    '                    priorReagentName = reagentName
    '                    priorReagent = reagent

    '' Read the cells value
    '                    For rowIndex = deltaLayoutStart To deltaLayoutEnd
    '                        tmpVar = sheet.Cells(rowIndex, colIndex).Value
    '                        If tmpVar IsNot Nothing Then
    '                            If tmpVar.ToString().Trim().Equals(String.Empty) Or tmpVar.ToString().Trim().Equals("0") Then
    '' Empty string or '0'
    '                                tmpVar = -9999
    '                            Else
    '' Split string using ' ' as the delimiter then remove the label "UL"
    '' to get the volume
    '                                tmpVar = tmpVar.ToUpper().Split(" ")(0)
    '                                If tmpVar.Contains("UL") Then
    '                                    tmpVar = tmpVar.Replace("UL", String.Empty).Trim()
    '                                End If

    '                            End If

    '                        Else ' "Nothing"
    '                                tmpVar = -9999
    '                        End If
    '                        wellList.Add(tmpVar)

    '                    Next ' Go to next row

    '                    ' Save reagent volume information
    '                    reagent.WellList = wellList
    '                    reagent.InitializeReagentVolume()

    '                    If j = 0 Then ' First Plate
    '                        ' Now we are finished with 1st column of the layout...Add the list to frmReagentControl
    '                        m_frmPlates.BuildPlateInfo(PLATE_NUMBER.PLATE1, plateName, reagentName, colIndex, wellList)
    '                    ElseIf j = 1 Then ' Second Plate
    '                        m_frmPlates.BuildPlateInfo(PLATE_NUMBER.PLATE2, plateName, reagentName, colIndex, wellList)
    '                    ElseIf j = 2 Then ' Third Plate
    '                        m_frmPlates.BuildPlateInfo(PLATE_NUMBER.PLATE3, plateName, reagentName, colIndex, wellList)
    '                    ElseIf j = 3 Then ' Fourth Plate
    '                        m_frmPlates.BuildPlateInfo(PLATE_NUMBER.PLATE4, plateName, reagentName, colIndex, wellList)
    '                    ElseIf j = 4 Then ' Fifth Plate
    '                        m_frmPlates.BuildPlateInfo(PLATE_NUMBER.PLATE5, plateName, reagentName, colIndex, wellList)
    '                    Else

    '                    End If

    '                    ' Add this reagent to plate
    '                    plateLayout.AddReagent(reagent)

    '                    ' Clear this list for the next column
    '                    wellList.Clear()

    '                Next ' Go to next  column

    '                ' Add this plate to the list
    '                m_PlateList.Add(plateLayout)

    '                ' Mapping of plate to Maestro Deck
    '                m_PlateMapList.Add(plateName, plateNameToMaestroDeckID)
    '            Next ' Go to next plate

    '            ' We have found our sheet. Now get out of this loop
    '            Exit For
    '        End If
    '    Next

    '    workBook.Close(False, Nothing, Nothing)
    '    m_ExcelObj.Quit()
    '    m_ExcelObj = Nothing
    '    Dim dlgResult As System.Windows.Forms.DialogResult = Windows.Forms.DialogResult.None
    '    m_IsCancel = False
    '    If isShowSimplePlateInfo Then
    '        m_frmSimplePlatesInfo = New frmSimplePlateInfo(m_NumberOfRows, m_NumberOfColumns, m_NumberofSamples, m_PlateMapList)

    '        dlgResult = m_frmSimplePlatesInfo.ShowDialog(m_HostWindow)
    '        If dlgResult = Windows.Forms.DialogResult.Cancel Then
    '            m_IsCancel = True
    '        End If
    '    Else
    '        'If numPlates = 1 Then
    '        '    m_frmPlates.TabCtrlPlateLayouts.TabPages.Remove(m_frmPlates.TabPage2)
    '        'End If

    '        m_ExcelObj = Nothing
    '        m_frmPlates.MapList = m_PlateMapList

    '        m_frmPlates.NumberOfColumns = m_NumberOfColumns
    '        m_frmPlates.NumberOfRows = m_NumberOfRows
    '        m_frmPlates.NumberOfSamples = m_NumberofSamples

    '        While (Not dlgResult = System.Windows.Forms.DialogResult.OK)
    '            dlgResult = m_frmPlates.ShowDialog(m_HostWindow)
    '        End While

    '        If dlgResult = Windows.Forms.DialogResult.OK Then
    '            m_IsCancel = m_frmPlates.Cancel
    '            If m_IsCancel = False Then
    '                ' Get back mapping between Plate name and Sciclone deck
    '                m_PlateMapList = m_frmPlates.MapList

    '                ' Now map deck ID from m_PlateMapList to m_PlateList
    '                MapDeckIDFromPlateMapListToPlateList()
    '            End If
    '        End If
    '    End If

    'End Sub

    Private Sub MapDeckIDFromPlateMapListToPlateList()

        For Each item As DictionaryEntry In m_PlateMapList
            For Each plate As CPlateLayout In m_PlateList
                If item.Key.ToString.Equals(plate.PlateName) Then
                    plate.PlateDeckID = item.Value.ToString
                    Exit For
                End If
            Next
        Next
    End Sub

    Private Function FindPlateBasedOnDeckID(ByRef deckID As String) As String

        Dim plateNameFound As String = String.Empty
        If m_PlateMapList Is Nothing Then
            Return plateNameFound
        End If
        For Each item As DictionaryEntry In m_PlateMapList
            If item.Value.ToString.ToUpper.Trim.Equals(deckID.ToUpper.Trim) Then
                plateNameFound = item.Key.ToString()
            End If
        Next

        Return plateNameFound
    End Function

    Public ReadOnly Property PlateList() As List(Of CPlateLayout)
        Get
            Return m_PlateList
        End Get

    End Property

    Public Property ExcelWorkBookName() As String
        Get
            Return m_Workbook
        End Get
        Set(ByVal value As String)
            m_Workbook = value
        End Set
    End Property

    Public Property ExcelWorkSheet() As String
        Get
            Return m_Worksheet
        End Get
        Set(ByVal value As String)
            m_Worksheet = value
        End Set
    End Property

    Public ReadOnly Property NumberOfColumns() As Integer
        Get
            Return m_NumberOfColumns
        End Get

    End Property

    Public ReadOnly Property IsCancel() As Boolean
        Get
            Return m_IsCancel
        End Get
    End Property
    Public Sub New()
        m_frmPlates = New frmPlates
        m_PlateList = New List(Of CPlateLayout)
        m_PlateMapList = New Hashtable
    End Sub

    Public WriteOnly Property HostWindow() As CaliperLifeSciences.VSTA.MaestroMacroHost.WindowWrapper
        Set(ByVal value As CaliperLifeSciences.VSTA.MaestroMacroHost.WindowWrapper)
            m_HostWindow = value
        End Set
    End Property
End Class
