Imports System.Collections.Generic
Imports System.Data.SqlClient
Imports System.Windows.Forms
Imports System.Drawing

Public Class frmDeckSetup

    Private Const MAX_DECK As Integer = 20
    Private Const DEFAULT_DESC As String = "< Type your instruction here. Then load image below >"
    Private m_ImagePageList As List(Of CImagePage)

    Private m_ImageFileName As String
    Private m_ApplicationName As String
    Private m_DeckID As String
    Private m_Description As String
    Private m_DBID As String ' As GUID
    Private m_PageImage As Drawing.Image
    Private m_IsAbort As Boolean

    Private m_CAppDBData As CAppDBData

    ' Determines whether this application has associated images 
    ' and steps description in the database
    'Private m_IsAppDBExists As Boolean = False

    Private m_Image As System.Drawing.Image

    Private Sub btnNext_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles btnNext.Click

        Dim cImage As CImagePage = Nothing
        Dim pageNo As Integer
        ErrorProvider1.SetError(picBox, String.Empty)

        If Not txtDesc.Text.Equals(String.Empty) And picBox.Image IsNot Nothing Then

            ' Save current page
            pageNo = lblPage.Text
            SaveCurrentPage(pageNo)

            ' Now move to the next page
            lblPage.Text = pageNo + 1
            pageNo = pageNo + 1

            PopulateImageScreen(pageNo)

            SetBtnPreviousState(True)
        End If
    End Sub

    Private Sub btnLoadImage_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles btnLoadImage.Click

        Dim dlgRes As DialogResult
        dlgRes = OpenFileDialog1.ShowDialog(Me)
        If dlgRes = Windows.Forms.DialogResult.OK Then
            m_ImageFileName = OpenFileDialog1.FileName
            picBox.Image = System.Drawing.Image.FromFile(m_ImageFileName)
            picBox.ImageLocation = m_ImageFileName

        End If

    End Sub

    Sub New(ByVal appName As String)

        ' This call is required by the Windows Form Designer.
        InitializeComponent()

        ' Add any initialization after the InitializeComponent() call.
        m_ApplicationName = appName
        m_ImagePageList = New List(Of CImagePage)

        m_CAppDBData = New CAppDBData

    End Sub

    Private Sub btnAbort_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles btnAbort.Click

        Dim dlgBx As DialogResult
        dlgBx = MessageBox.Show("Are you sure you want to stop?", "Abort", MessageBoxButtons.YesNo)
        If dlgBx = Windows.Forms.DialogResult.Yes Then
            m_IsAbort = True
            Me.Close()
        End If
    End Sub

    Private Sub btnFinish_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles btnFinish.Click
        Dim dlgBx As DialogResult
        dlgBx = MessageBox.Show("Ready to Start Run?", "Finished", MessageBoxButtons.YesNo)
        If dlgBx = Windows.Forms.DialogResult.Yes Then

            ' Push objects in m_ImageDescList to database
            m_CAppDBData.PushImagePageObjectsToDB(m_ImagePageList)

            m_IsAbort = False
            Me.Close()
        End If
    End Sub

    Public Property IsAbort() As Boolean
        Get
            Return m_IsAbort
        End Get
        Set(ByVal value As Boolean)
            m_IsAbort = value
        End Set
    End Property

    Private Sub btnPrevious_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles btnPrevious.Click

        Dim cImage As CImagePage = Nothing
        ' Pull object from m_ImageDescList
        Dim pageNo As Integer = Convert.ToInt32(lblPage.Text) - 1

        If pageNo = 0 Then
            SetBtnPreviousState(False)
        Else
            SetBtnPreviousState(True)
            PopulateImageScreen(pageNo)
        End If

    End Sub

    ' Check if image page is already in the collection
    Private Function ImagePageExists(ByVal pageNo As Integer) As CImagePage

        If m_ImagePageList.Count <= 0 Or pageNo <= 0 Then
            Return Nothing
        End If

        For Each imgPage As CImagePage In m_ImagePageList
            If imgPage.ImagePageNumber = pageNo Then
                Return imgPage
            End If
        Next
        Return Nothing
    End Function

    ' Reset screen 
    Private Sub ResetImageScreen()
        m_ImageFileName = String.Empty
        txtDesc.Text = DEFAULT_DESC
        picBox.Image = Nothing
        picBox.ImageLocation = String.Empty


        m_DBID = String.Empty
        m_DeckID = String.Empty
        m_ImageFileName = String.Empty
        m_Description = String.Empty
        m_Image = Nothing
    End Sub

    ' Add image page to collection
    Private Function AddImagePageToCollection(ByVal pageNo As Integer, ByVal isDBData As Boolean) As Boolean

        Dim isAdded As Boolean = False
        Try
            Dim imageObj As CImagePage = Nothing
            imageObj = ImagePageExists(pageNo)

            If imageObj Is Nothing Then
                m_ImagePageList.Add(CreateImageClass(pageNo, isDBData))
            Else ' Update current image object


                If Not txtDesc.Text.Equals(imageObj.Description) Then
                    imageObj.Description = txtDesc.Text
                    imageObj.ImagePageChanged = True
                End If
                If Not cboDeckID.SelectedItem.Equals(imageObj.DeckID) Then
                    imageObj.DeckID = cboDeckID.SelectedItem
                    imageObj.ImagePageChanged = True
                End If
                If Not picBox.ImageLocation.Equals(imageObj.ImageFile) Then

                    Dim imageByteArray() As Byte = Nothing
                    Dim curImage As Image = Nothing
                    UtilMod.ImageConversion(picBox.ImageLocation, imageByteArray)
                    UtilMod.ByteToImage(curImage, imageByteArray)
                    imageObj.DeckImage = curImage
                    imageObj.ImagePageChanged = True

                End If

                If imageObj.ImagePageChanged Then
                    RemoveImageFromList(pageNo)
                    m_ImagePageList.Add(imageObj)
                End If

            End If
            isAdded = True
        Catch ex As Exception
            'TO DO - Add exception message
        End Try

        Return isAdded
    End Function

    Private Sub frmDeckSetup_Load(ByVal sender As Object, ByVal e As System.EventArgs) Handles Me.Load
        lblPage.Text = 1
        lblAppName.Text = m_ApplicationName

        ' Try to retrieve information about this application from the database
        m_ImagePageList.Clear()
        LoadAppInfofFromDB()

    End Sub

    Private Sub SetBtnPreviousState(ByVal state As Boolean)
        If state Then
            btnPrevious.Enabled = True
        Else
            btnPrevious.Enabled = False
        End If
    End Sub

    Private Function CreateImageClass(ByVal pageNo As Integer, ByVal isDBData As Boolean) As CImagePage
        Dim cImage As CImagePage = Nothing

        cImage = New CImagePage
        If m_DBID Is Nothing Or m_DBID.Equals(String.Empty) Then
            m_DBID = Guid.NewGuid().ToString
        End If
        cImage = m_CAppDBData.CreateImageClass(m_DBID, m_DeckID, m_ImageFileName, m_Description, pageNo, m_ApplicationName, m_Image)
        If isDBData Then
            cImage.ImagePageChanged = False
        End If
        Return cImage
    End Function

    Private Sub LoadAppInfofFromDB()
        Dim cImage As CImagePage
        Dim appObj As CConnectDB = New CConnectDB
        Dim oDBReader As SqlDataReader
        Dim err As String = Nothing
        Dim i As Integer = 1
        Dim pageNo As Integer
        Try
            oDBReader = appObj.GetDBImageData(m_ApplicationName)

            If Not oDBReader Is Nothing Then

                If oDBReader.HasRows Then

                    ' Loop through the DataReader
                    While oDBReader.Read
                        cImage = New CImagePage
                        m_DBID = oDBReader.GetGuid(0).ToString()
                        m_ImageFileName = oDBReader.GetString(1)
                        'oDBReader.GetString(2)
                        m_ApplicationName = oDBReader.GetString(2)
                        m_Description = oDBReader.GetString(3)

                        m_DeckID = oDBReader.GetString(4)
                        pageNo = oDBReader.GetInt32(5)

                        Dim byteArray() As Byte
                        byteArray = CType(oDBReader.GetValue(6), Byte())
                        UtilMod.ByteToImage(m_Image, byteArray)

                        AddImagePageToCollection(pageNo, True)

                    End While
                End If
                oDBReader.Close()
                appObj.CloseSQLDBConn(err)

                'Display the first image page
                PopulateImageScreen(1)

                If err Is Nothing Then
                    Exit Sub
                ElseIf Not err.Equals(String.Empty) Then
                    MessageBox.Show("FrmDeckSetup::LoadAppInfofFromDB(): " & err)
                End If
            End If

        Catch ex As Exception
            MessageBox.Show("FrmDeckSetup::LoadAppInfofFromDB(): Unable to retrieve application information from database.", "DB Connection Error", MessageBoxButtons.OK)
        End Try

    End Sub

    Private Sub SaveCurrentPage(ByVal pageNo As Integer)

        m_Description = txtDesc.Text
        m_DeckID = cboDeckID.SelectedItem
        m_PageImage = picBox.Image
        ' Don't proceed to next page if current page is empty
        If Not m_Description.Equals(String.Empty) And m_PageImage IsNot Nothing Then
            ' Add this page to collection
            If AddImagePageToCollection(pageNo, False) Then
                ' Reset fields to empty for the next page
                ResetImageScreen()
            End If
        Else
            ErrorProvider1.SetError(picBox, "Can not proceed to next page unles image AND description are specified.")
        End If
    End Sub

    Private Sub PopulateImageScreen(ByVal pageNo As Integer)
        Dim isImageFound As Boolean
        For Each obj As CImagePage In m_ImagePageList
            If obj.ImagePageNumber = pageNo Then
                txtDesc.Text = obj.Description
                cboDeckID.SelectedItem = obj.DeckID
                picBox.Image = obj.DeckImage
                'picBox.ImageLocation = obj.ImageFile
                lblPage.Text = obj.ImagePageNumber
                picBox.Tag = obj.DBID
                isImageFound = True
                Exit For
            End If
        Next

        If isImageFound = False Then
            ResetImageScreen()
        End If
    End Sub

    Private Sub RemoveImageFromList(ByVal pageNo As Integer)

        For Each obj As CImagePage In m_ImagePageList
            If obj.ImagePageNumber = pageNo Then
                m_ImagePageList.Remove(obj)
                Exit For
            End If
        Next
    End Sub
End Class