Imports System.Collections
Imports System.Collections.Generic
'Imports CaliperLifeSciences.NGS

Partial Class Maestro
    Private Sub Maestro_Startup(ByVal sender As Object, ByVal e As System.EventArgs) Handles Me.Startup
    End Sub

    Private Sub Maestro_Shutdown(ByVal sender As Object, ByVal e As System.EventArgs) Handles Me.Shutdown
    End Sub

    Private m_PlateList As List(Of CPlateLayout)
    Private m_Plates As CPlates
    Private m_TimerThread As Threading.Thread
    Private m_CAppDBObj As CAppDBData
    Private m_WaitTimer As Boolean
    Private m_IsSunkenDeckNGS As Boolean
    Private m_NormalizationList As List(Of CNorm)

    Public Enum HEAD_POSITION_TYPE

        RGRIPPER ' Right gripper
        LGRIPPER ' Left gripper
        GANTRY   ' Gantry

    End Enum


#Region "Reagent Distribution"

    Public Sub ReadExcelSpreadSheet()

        Dim excelFile As String
        Dim excelSheet As String
        Dim isShowSimplePlateInfo As Boolean

        Try
            m_Plates = New CPlates()
            excelFile = Me.GetGlobalVariableValue("g_ExcelWorkBook")
            excelSheet = Me.GetGlobalVariableValue("g_ExcelWorkSheet")
            m_Plates.ExcelWorkBookName = excelFile
            m_Plates.HostWindow = Me.HostWindow
            isShowSimplePlateInfo = Convert.ToBoolean(Me.GetGlobalVariableValue("g_IsShowSimplePlateInfo"))

            If System.IO.File.Exists(excelFile) Then
                m_Plates.ReadPlateLayoutSpreadSheet(excelFile, excelSheet, isShowSimplePlateInfo)
                ' Get back the list of all the plates.

                If m_Plates.IsCancel = False Then
                    m_PlateList = m_Plates.PlateList()
                End If

                Try
                    Me.SetGlobalVariableValue("g_ApplicationAborted", m_Plates.IsCancel)
                    Me.SetGlobalVariableValue("g_NumberOfColumnsToProcess", m_Plates.NumberOfColumns)
                Catch ex As Exception
                    Me.SetGlobalVariableValue("g_ApplicationAborted", True)
                    Me.RaiseErrorFromVSTA(-9999, "Maestro::ReadExcelSpreadSheet", " Unable to write to global variables.")
                End Try

            End If
        Catch ex As Exception
            Me.SetGlobalVariableValue("g_ApplicationAborted", True)
            Me.RaiseErrorFromVSTA(-9999, "Maestro::ReadExcelSpreadSheet", " Unable to read global variables from excel.")
        End Try
    End Sub

    Public Sub GetColAndRowByReagentColumn()

        Dim reagentColumn As Integer = 0
        Dim aspirateVol As Decimal
        Dim errMessage As String = String.Empty

        Dim col As Integer
        Dim row As Integer
        Dim aspirateColStart As Integer

        Dim sourceDeckID As String = String.Empty

        ' If we haven't read the excel spreadsheet then read it now
        If m_PlateList Is Nothing Then
            ReadExcelSpreadSheet()

            If m_PlateList Is Nothing Then
                Me.SetGlobalVariableValue("g_ApplicationAborted", True)
                Me.RaiseErrorFromVSTA(-9999, "Maestro Class in VSTA::GetColAndRowByReagentColumn(): ", "Unable to read Excel spreadsheet. Make sure the file exists.")
                Exit Sub
            End If
        End If

        If m_PlateList.Count <= 0 Then
            Me.SetLocalVariableValue("g_ApplicationAborted", "true")
            Me.RaiseErrorFromVSTA(-9999, "Maestro Class in VSTA::GetColAndRowByReagentColumn(): ", "No plat layout defined.")
            Exit Sub
        End If

        sourceDeckID = Me.GetParameterVariableValue("p_AspirateDeckID")
        If sourceDeckID.Equals(String.Empty) Then
            Me.SetLocalVariableValue("g_ApplicationAborted", "true")
            Me.RaiseErrorFromVSTA(-9999, "Maestro Class in VSTA::GetColAndRowByReagentColumn(): ", "Unable to determine reagent deck ID. Please check your input parameter - p_PlateName.")
            Exit Sub
        End If

        For Each plate As CPlateLayout In m_PlateList

            If plate.PlateDeckID.ToUpper.Trim().Equals(sourceDeckID.ToUpper.Trim) Then
                reagentColumn = Convert.ToInt32(Me.GetParameterVariableValue("p_ReagentColumn"))
                If reagentColumn <= 0 Then
                    Me.RaiseErrorFromVSTA(-9999, "Maestro Class in VSTA::GetColAndRowByReagentColumn(): ", "Unable to determine reagent name. Please check your input parameter - p_ReagentName.")
                    Exit Sub
                End If

                Try
                    aspirateVol = Convert.ToDecimal(Me.GetParameterVariableValue("p_AspirateVolume"))
                Catch ex As Exception
                    Me.SetLocalVariableValue("g_ApplicationAborted", "true")
                    Me.RaiseErrorFromVSTA(-9999, "Maestro Class in VSTA::GetColAndRowByReagentColumn(): ", "Invalid input volumne. Please check your input parameter - p_AspirateVolume.")
                    Exit Sub
                End Try

                plate.GetColAndRowByReagentColumn(reagentColumn, aspirateVol, row, col, aspirateColStart, errMessage)

                If errMessage.Equals(String.Empty) Then
                    'Set local variable for number of tips to load...Row and Col
                    Me.SetLocalVariableValue("l_Row", row)
                    Me.SetLocalVariableValue("l_Col", col)
                    Me.SetLocalVariableValue("l_AspirateColStart", aspirateColStart)
                Else
                    Me.SetLocalVariableValue("g_ApplicationAborted", "true")
                    Me.RaiseErrorFromVSTA(-9999, "Maestro Class in VSTA::GetColAndRowByReagentColumn(): ", errMessage)
                End If

            End If
        Next

    End Sub

#End Region

#Region "Deck Calibration Code"

    Private m_DeckDictionary As List(Of CDeckInfo)

    Public Sub SetGantryHeadCoordinate()
        Dim x As Double
        Dim y As Double
        Dim z As Double
        Dim deckID As String
        Dim deckNum As Integer
        Try
            deckID = Me.GetLocalVariableValue("l_DeckID")
            deckNum = Convert.ToInt32(Me.GetLocalVariableValue("l_DeckNumber"))

            'Me.GetTaughtPosition(deckID, "MainArray", x, y, z)

            x = Convert.ToSingle(Me.GetLocalVariableValue("l_HeadX"))
            y = Convert.ToSingle(Me.GetLocalVariableValue("l_HeadY"))
            z = Convert.ToSingle(Me.GetLocalVariableValue("l_HeadZ"))

            SetXYZCoordinate(HEAD_POSITION_TYPE.GANTRY, deckID, deckNum, x, y, z)

        Catch ex As Exception
            Me.SetGlobalVariableValue("g_ApplicationAborted", True)
            Me.RaiseErrorFromVSTA(-9999, "Maestro::SetGantryHeadCoordinate", " Unable to read lobal variables l_DeckID, l_HeadX, l_Heady or l_Headz.")
        End Try
    End Sub

    Public Sub GetGantryHeadCoordinate()
        Dim deckID As String
        Dim x As Single
        Dim y As Single
        Dim z As Single

        Try

            deckID = Me.GetLocalVariableValue("l_DeckID")
            GetXYZCoordinate(HEAD_POSITION_TYPE.GANTRY, deckID, x, y, z)

            Me.SetLocalVariableValue("l_HeadX", x)
            Me.SetLocalVariableValue("l_HeadY", y)
            Me.SetLocalVariableValue("l_HeadZ", z)

        Catch ex As Exception
            Me.SetGlobalVariableValue("g_ApplicationAborted", True)
            Me.RaiseErrorFromVSTA(-9999, "Maestro::GetGantryHeadCoordinate", " Unable to read/write local variables.")

        End Try
    End Sub

    Public Sub SetLeftGripperHeadCoordinate()

        Dim x As Double
        Dim y As Double
        Dim z As Double
        Dim deckID As String
        Dim deckNum As Integer
        Try
            deckID = Me.GetLocalVariableValue("l_DeckID")
            x = Convert.ToSingle(Me.GetLocalVariableValue("l_GripperX"))
            y = Convert.ToSingle(Me.GetLocalVariableValue("l_GripperY"))
            z = Convert.ToSingle(Me.GetLocalVariableValue("l_GripperZ"))
            deckNum = Convert.ToInt32(Me.GetLocalVariableValue("l_DeckNumber"))

            'Me.GetTaughtPosition(deckID, "Left-Gripper:Center", x, y, z)

            SetXYZCoordinate(HEAD_POSITION_TYPE.LGRIPPER, deckID, deckNum, x, y, z)

        Catch ex As Exception
            Me.SetGlobalVariableValue("g_ApplicationAborted", True)
            Me.RaiseErrorFromVSTA(-9999, "Maestro::SetLeftGripperXYZCoordinate", " Unable to read lobal variables l_DeckID, l_Gripperx, l_Grippery or l_Gripperz.")
        End Try
    End Sub

    Public Sub GetLeftGripperHeadCoordinate()
        Dim deckID As String
        Dim x As Single
        Dim y As Single
        Dim z As Single

        Try

            deckID = Me.GetLocalVariableValue("l_DeckID")
            GetXYZCoordinate(HEAD_POSITION_TYPE.LGRIPPER, deckID, x, y, z)

            Me.SetLocalVariableValue("l_GripperX", x)
            Me.SetLocalVariableValue("l_GripperY", y)
            Me.SetLocalVariableValue("l_GripperZ", z)

        Catch ex As Exception
            Me.SetGlobalVariableValue("g_ApplicationAborted", True)
            Me.RaiseErrorFromVSTA(-9999, "Maestro::GetLeftGripperHeadCoordinate", " Unable to read/write local variables.")

        End Try
    End Sub

    Public Sub IsCalibrateNextDeckLocation()
        Dim deckNum As Integer
        Dim r As Integer
        Try

            deckNum = Convert.ToInt32(Me.GetLocalVariableValue("l_DeckNumber"))
            r = Convert.ToInt32(Me.GetLocalVariableValue("l_R"))

            If (deckNum Mod 5) = r Then
                Me.SetLocalVariableValue("l_IsCalibrateNextDeckLocation", "true")
            Else
                Me.SetLocalVariableValue("l_IsCalibrateNextDeckLocation", "false")
            End If
        Catch ex As Exception
            Me.SetGlobalVariableValue("g_ApplicationAborted", True)
            Me.RaiseErrorFromVSTA(-9999, "Maestro::IsCalibrateNextDeckLocation", " Unable to read local variable from Maestro.")
        End Try

    End Sub

    Private Sub GetXYZCoordinate(ByVal HeadPositionType As HEAD_POSITION_TYPE, ByVal deckID As String, ByRef x As Single, ByRef y As Single, ByRef z As Single)

        If m_DeckDictionary Is Nothing Then
            Exit Sub
        End If

        For Each cDeck As CDeckInfo In m_DeckDictionary
            If cDeck.DeckID.Equals(deckID.Trim) Then

                If HeadPositionType = HEAD_POSITION_TYPE.LGRIPPER Then
                    x = cDeck.LeftGripperX
                    y = cDeck.LeftGripperY
                    z = cDeck.LeftGripperZ

                ElseIf HeadPositionType = HEAD_POSITION_TYPE.GANTRY Then
                    x = cDeck.GantryHeadX
                    y = cDeck.GantryHeadY
                    z = cDeck.GantryHeadZ
                End If

                Exit For
            End If
        Next

    End Sub

    Private Sub SetXYZCoordinate(ByVal HeadPositionType As HEAD_POSITION_TYPE, ByVal deckID As String, ByVal deckNum As Integer, ByVal x As Single, ByVal y As Single, ByVal z As Single)

        Dim deckInfo As CDeckInfo = Nothing
        deckInfo = New CDeckInfo(deckID, deckNum)
        If HeadPositionType = HEAD_POSITION_TYPE.LGRIPPER Then
            deckInfo.LeftGripperX = x
            deckInfo.LeftGripperY = y
            deckInfo.LeftGripperZ = z

        ElseIf HeadPositionType = HEAD_POSITION_TYPE.GANTRY Then
            deckInfo.GantryHeadX = x
            deckInfo.GantryHeadY = y
            deckInfo.GantryHeadZ = z
        End If

        If m_DeckDictionary Is Nothing Then
            m_DeckDictionary = New List(Of CDeckInfo)
        End If

        ' Check if deck already exists in the collection
        ' if so then only replace the coordinate. 
        UpdateDeckDictionaryList(deckInfo, HeadPositionType)
    End Sub

    Private Sub UpdateDeckDictionaryList(ByVal deckInfo As CDeckInfo, ByVal HeadPositionType As HEAD_POSITION_TYPE)
        Dim isUpdate As Boolean = False
        If m_DeckDictionary Is Nothing Then
            Exit Sub
        End If
        Try
            For Each cDeck As CDeckInfo In m_DeckDictionary
                If cDeck.DeckID.Equals(deckInfo.DeckID) Then

                    If HeadPositionType = HEAD_POSITION_TYPE.LGRIPPER Then
                        cDeck.LeftGripperX = deckInfo.LeftGripperX
                        cDeck.LeftGripperY = deckInfo.LeftGripperY
                        cDeck.LeftGripperZ = deckInfo.LeftGripperZ

                    ElseIf HeadPositionType = HEAD_POSITION_TYPE.GANTRY Then
                        cDeck.GantryHeadX = deckInfo.GantryHeadX
                        cDeck.GantryHeadY = deckInfo.GantryHeadY
                        cDeck.GantryHeadZ = deckInfo.GantryHeadZ
                    End If

                    cDeck.DeckNumber = deckInfo.DeckNumber
                    isUpdate = True
                    Exit For
                End If
            Next
            If isUpdate = False Then
                m_DeckDictionary.Add(deckInfo)
            End If
        Catch ex As Exception
            Me.SetGlobalVariableValue("g_ApplicationAborted", True)
            Me.RaiseErrorFromVSTA(-9999, "Maestro::UpdateDeckDictionaryList", " Unable to update deck information dictionary lost.")
        End Try

    End Sub

#End Region

#Region "Barcode Indexing"
    Private m_BarcodeIndex As CIndex
    Private m_SampleListForThisIndex As List(Of CSample)
    Private m_SourceWellID As String

    Public Sub ReadDNABarcodeIndex()

        If m_BarcodeIndex Is Nothing Then
            Dim indexWorkbook As CReadIndexWorkbook = New CReadIndexWorkbook
            Dim workBookName As String
            Dim sheetName As String

            'Dim cReadBarcode As CReadBarcode = New CReadBarcode
            'Dim xmlFile As String

            Try
                workBookName = Me.GetGlobalVariableValue("g_ExcelWorkBook")
                sheetName = Me.GetGlobalVariableValue("g_IndexWorkSheet")
                m_BarcodeIndex = indexWorkbook.ReadIndexWorkbook(workBookName, sheetName)

                ' NOTHING IS WRONG WITH THIS CODE. It reads from xml file instead of spreadsheet
                'xmlFile = Me.GetGlobalVariableValue("g_BarcodeXMLFile")
                'm_BarcodeIndex = cReadBarcode.ReadBarcode(xmlFile)

                If m_BarcodeIndex.MaxBarcodeIndex = 0 Then
                    Me.SetGlobalVariableValue("g_ApplicationAborted", True)
                    Me.RaiseErrorFromVSTA(-9999, "Maestro::ReadDNABarcodeIndex", " Unable to read barcode index spreadsheet. Please verify that the index sheet specified by the variable g_IndexWorkSheet matches the one in the workbook.")
                End If

                If m_BarcodeIndex Is Nothing Then
                    Me.SetGlobalVariableValue("g_ApplicationAborted", True)
                    Me.RaiseErrorFromVSTA(-9999, "Maestro::ReadDNABarcodeIndex", " Unable to read barcode index spreadsheet. Please verify that the index sheet specified by the variable g_IndexWorkSheet matches the one in the workbook.")
                End If


                Threading.Thread.Sleep(2000)
            Catch ex As Exception
                Me.SetGlobalVariableValue("g_ApplicationAborted", True)
                Me.RaiseErrorFromVSTA(-9999, "Maestro::ReadDNABarcodeIndex", " Unable to read barcode index spreadsheet. Please verify that the index sheet specified by the variable g_IndexWorkSheet matches the one in the workbook.")

            End Try
        End If
    End Sub
    Public Sub GetNumberOfDNABarcodeSamples()

        Try
            If m_BarcodeIndex Is Nothing Then
                ReadDNABarcodeIndex()
            End If

            Dim numSamples As Integer = m_BarcodeIndex.NumberOfSamples()
            Me.SetGlobalVariableValue("g_NumberOfSamples", numSamples)

        Catch ex As Exception
            Me.SetGlobalVariableValue("g_ApplicationAborted", True)
            Me.RaiseErrorFromVSTA(-9999, "Maestro::GetNumberOfDNABarcodeSamples", " Unable to get barcode sample list.")

        End Try

    End Sub

    ' Get the max barcode index
    Public Sub GetMaxBarcodeIndex()

        Try
            If m_BarcodeIndex Is Nothing Then
                ReadDNABarcodeIndex()
            End If

            Dim totalBarcodeIndex As Integer = m_BarcodeIndex.MaxBarcodeIndex
            Me.SetLocalVariableValue("l_MaxBarcodeIndex", totalBarcodeIndex)

        Catch ex As Exception

            Me.SetGlobalVariableValue("g_ApplicationAborted", True)
            Me.RaiseErrorFromVSTA(-9999, "Maestro::GetMaxBarcodeIndex", " Unable to get max barcode index.")

        End Try
    End Sub

    ' For each barcode index counter we query for IndexID that we got from the spread sheet
    Public Sub GetNextBarcodeIndexID()

        Try
            Dim indexCounter As Integer
            Dim row As Integer = 0
            Dim col As Integer = 0

            m_SourceWellID = String.Empty
            If m_SampleListForThisIndex Is Nothing Then
                m_SampleListForThisIndex = New List(Of CSample)
            Else
                m_SampleListForThisIndex.Clear()
            End If

            ' Get a list of all the samples then query for IndexID
            Dim numSamples As List(Of CSample) = m_BarcodeIndex.SampleList()

            If numSamples IsNot Nothing Then
                indexCounter = Convert.ToInt32(Me.GetLocalVariableValue("l_BarcodeIndexCounter"))

                ' Get number of samples with this index from sample list
                For Each s As CSample In numSamples
                    If s.IndexID = indexCounter Then
                        m_SampleListForThisIndex.Add(s)

                        ' Source barcode wellID should be the same if multiple samples
                        ' have the same Index ID
                        m_SourceWellID = s.SourceWellID
                    End If
                Next

                If m_SampleListForThisIndex.Count > 0 Then
                    Me.SetLocalVariableValue("l_NumIndex", m_SampleListForThisIndex.Count)
                    ' Now get row and col for this well ID
                    UtilMod.GetRowColBasedOnWellID(m_SourceWellID, row, col)
                End If

                Me.SetLocalVariableValue("l_SourceWellID", m_SourceWellID)

                ' Now get row and col for this well ID
                Me.SetLocalVariableValue("l_AspirateCol", col)
                Me.SetLocalVariableValue("l_AspirateRow", row)

            End If

        Catch ex As Exception
            Me.SetGlobalVariableValue("g_ApplicationAborted", True)
            Me.RaiseErrorFromVSTA(-9999, "Maestro::GetNextBarcodeIndexID", " Unable to retrieve barcode index ID.")

        End Try

    End Sub

    Public Sub GetSampleInfoForIndexLoopCounter()

        Try
            Dim row As Integer = 0
            Dim col As Integer = 0

            Dim subIndex As Integer = Convert.ToInt32(Me.GetLocalVariableValue("l_SubIndex"))
            Dim sample As CSample = m_SampleListForThisIndex(subIndex)

            ' Protect against number larger than current list count
            If subIndex < m_SampleListForThisIndex.Count Then

                Me.SetLocalVariableValue("l_IndexID", sample.IndexID)
                Me.SetLocalVariableValue("l_PoolID", sample.PoolID)
                Me.SetLocalVariableValue("l_SampleID", sample.SampleID)

                ' Now get row and col for this well ID
                UtilMod.GetRowColBasedOnWellID(m_SourceWellID, row, col)
                Me.SetLocalVariableValue("l_AspirateCol", col)
                Me.SetLocalVariableValue("l_AspirateRow", row)
                Me.SetLocalVariableValue("l_SourceWellID", sample.SourceWellID)

                ' Now get row and col for this well ID
                UtilMod.GetRowColBasedOnWellID(sample.DestinationWellID, row, col)

                ' This is a 96-well map so we can ignore section below
                ' This is for 384-well plate
                'Select Case row
                '    Case 1
                '        row = row     'A
                '    Case 2
                '        row = row + 1 'B
                '    Case 3
                '        row = row + 2 'C
                '    Case 4
                '        row = row + 3 'D
                '    Case 5
                '        row = row + 4 'E
                '    Case 6
                '        row = row + 5 'F
                '    Case 7
                '        row = row + 6 'G
                '    Case 8
                '        row = row + 7 'H
                'End Select

                'Select Case col
                '    Case 1
                '        col = col     '1
                '    Case 2
                '        col = col + 1 '2
                '    Case 3
                '        col = col + 2 '3
                '    Case 4
                '        col = col + 3 '4
                '    Case 5
                '        col = col + 4 '5
                '    Case 6
                '        col = col + 5 '6
                '    Case 7
                '        col = col + 6 '7
                '    Case 8
                '        col = col + 7 '8
                '    Case 9
                '        col = col + 8 '9
                '    Case 10
                '        col = col + 9 '10
                '    Case 11
                '        col = col + 10 '11
                '    Case 12
                '        col = col + 11 '12
                'End Select

                Me.SetLocalVariableValue("l_DispenseCol", col)
                Me.SetLocalVariableValue("l_DispenseRow", row)
                Me.SetLocalVariableValue("l_DestinationWellID", sample.DestinationWellID)
            End If


        Catch ex As Exception
            Me.SetGlobalVariableValue("g_ApplicationAborted", True)
            Me.RaiseErrorFromVSTA(-9999, "Maestro::GetSampleInfoForIndexLoopCounter", " Unable to to read/write Maestro local variable.")
        End Try

    End Sub

    Public Sub IsOneToOneMapping()
        Me.SetLocalVariableValue("l_IsOneToOneMapping", m_BarcodeIndex.IsOneToOneMap)
    End Sub

    ' Only use this method if IsOneToOneMapping is true
    Public Sub SetAspirateDispenseColumn()
        Me.SetLocalVariableValue("l_DispenseCol", m_BarcodeIndex.DestinationWellColumnStart)
        Me.SetLocalVariableValue("l_DispenseRow", 1)

        Me.SetLocalVariableValue("l_AspirateCol", m_BarcodeIndex.SourceWellColumnStart)
        Me.SetLocalVariableValue("l_AspirateRow", 1)

    End Sub

#End Region

#Region "Appplication Image Setup"
    Public Sub InitialApplicatiomImageSetup()

        Dim isAbort As Boolean
        Dim appName As String = Nothing

        Try
            appName = Me.CurrentApplicationName
            Me.SetGlobalVariableValue("g_ApplicationName", appName)
            Dim frm As frmDeckSetup = New frmDeckSetup(appName)

            frm.ShowDialog(Me.HostWindow)
            isAbort = frm.IsAbort
            If isAbort Then

                Me.SetGlobalVariableValue("g_ApplicationAborted", True)
                Me.RaiseErrorFromVSTA("-9999", "Startup Screen", "User has Aborted")
            End If
        Catch ex As Exception
            Me.RaiseErrorFromVSTA(-9999, "Maestro::InitialApplicatiomImageSetup", " Unable to read/write application name.")
        End Try

    End Sub
#End Region

#Region "Generic"

    Public Sub GetDeckIDByDeckNumber()

        Dim deckID As String
        Dim deckNum As Integer

        Try
            deckNum = Convert.ToInt32(Me.GetLocalVariableValue("l_DeckNumber"))
            deckID = LookUpDeckNumberID(deckNum, PlatformType.SCICLONE)
            Me.SetLocalVariableValue("l_DeckID", deckID)

        Catch ex As Exception
            Me.SetGlobalVariableValue("g_ApplicationAborted", True)
            Me.RaiseErrorFromVSTA(-9999, "Maestro::GetDeckIDByDeckNumber", " Unable to read lobal variables l_DeckNumber or l_DeckID.")
        End Try
    End Sub

    ' Use in UTIL_BarcodeIndexing method, SPRI Clean up
    Public Sub ComputeTransferCycles()

        Dim MAX_TIP_VOLUME As Single = 150 ' default
        Dim totalAspVol As Single
        Dim tmp As Single
        Dim numCycle As Integer

        Try
            totalAspVol = Convert.ToInt32(Me.GetLocalVariableValue("l_TotalAspirateVol"))
            tmp = totalAspVol / MAX_TIP_VOLUME '150 is the max tips volume
            numCycle = Math.Floor(tmp) + 1

            Me.SetLocalVariableValue("l_TransCycles", numCycle)

            Try
                Dim transferVolPerCycle As Integer
                transferVolPerCycle = totalAspVol / numCycle

                Me.SetLocalVariableValue("l_TransVolPerCycle", transferVolPerCycle)
            Catch ex As Exception
                ' Suppress this error. Some methods that call this may not care about the transfer vol per cycle
            End Try
        Catch ex As Exception
            Me.RaiseErrorFromVSTA(-9999, "Maestro::ComputeTransferCycles", " Unable to compute number of transfer cycles. Make sure the following variables are defined: l_TotalAspirateVol,l_TransCycles, and l_TransVolPerCycle ")
        End Try
    End Sub

#End Region

#Region "Consumable Stack"


#End Region


    Private m_TitleString As String
#Region "Application Setup"

    Public Sub ShowSampleSetupForm()

        Try
            Dim dlgSample As DlgSampleSetup

            Dim appName As String

            appName = Me.GetGlobalVariableValue("g_ApplicationName")
            dlgSample = New DlgSampleSetup()
            If dlgSample.ShowDialog(Me.HostWindow) = Windows.Forms.DialogResult.OK Then
                Me.SetGlobalVariableValue("g_NumberOfColumnsToProcess", dlgSample.SampleColumn)
                Me.SetGlobalVariableValue("g_SampleVolume", dlgSample.SampleVolume)
                Me.SetGlobalVariableValue("g_BeadVolume", dlgSample.BeadVolume)
                Me.SetGlobalVariableValue("g_ElutionVolume", dlgSample.ElutionVolume)
                'Me.SetGlobalVariableValue("g_IsAdapterPreBroadcast", dlgSample.IsPreBroadcastAdapters)

            Else
                Me.SetGlobalVariableValue("g_ApplicationAborted", True)
                Me.RaiseErrorFromVSTA(-9999, "ShowSampleSetupForm", "User Aborted")
            End If
        Catch ex As Exception
            Me.SetGlobalVariableValue("g_ApplicationAborted", True)
            Me.RaiseErrorFromVSTA(-9999, "ShowSampleSetupForm", "Unable to write to global variables:  g_NumberOfColumnsToProcess or g_IsAdapterPreBroadcast")
        End Try

    End Sub

    Public Sub ShowTime()
        m_WaitTimer = Convert.ToBoolean(Me.GetGlobalVariableValue("g_TimerWait"))
        StartThread("Incubation Time Remaining")
    End Sub

    Public Sub ShowBeadCaptureTime()
        m_WaitTimer = Convert.ToBoolean(Me.GetGlobalVariableValue("g_TimerWait"))
        StartThread("Bead Capture Time Remaining")
    End Sub

    Private Sub StartThread(ByVal screenTitle As String)
        m_TitleString = screenTitle
        m_TimerThread = New Threading.Thread(AddressOf StartIncubationTime)
        m_TimerThread.IsBackground = True
        m_TimerThread.Start()
        'm_TimerThread.Join() ' should be set by SetTimerWait function below

    End Sub

    Private Sub StartIncubationTime()
        Dim delayFrm As DlgDelayTime = New DlgDelayTime

        Try
            delayFrm.TimeRemain = Me.GetGlobalVariableValue("g_IncubationTimer")
            delayFrm.DelayTimeTitle = m_TitleString

            delayFrm.ShowDialog(Me.HostWindow)

            If delayFrm.StopDelay Then
                ' Continue executing. Don't show alarm
                'Me.SetGlobalVariableValue("g_ApplicationAborted", True)
                'Me.RaiseErrorFromVSTA(-9999, "Maestro::ShowTime", " User has aborted the incubation time.")

            End If
        Catch ex As Exception
            Me.SetGlobalVariableValue("g_ApplicationAborted", True)
            Me.RaiseErrorFromVSTA(-9999, "Maestro::ShowTime", " Unable to retrieve global variable: g_IncubationTimer.")

        End Try
    End Sub

    Public Sub ShowUserScreen()

        ' ----------------------------------------------------------------------------------------
        ' This section of code is for case where we don't connect to database. 
        '-----------------------------------------------------------------------------------------
        Dim appFolder As String
        Dim appName As String
        Dim screenName As String

        Try

            appFolder = Me.GetGlobalVariableValue("g_ThisApplicationScreensPath")
            appName = Me.GetGlobalVariableValue("g_ApplicationName")
            screenName = Me.GetGlobalVariableValue("g_ScreenName")

            Dim deckSetupFrm As frmDeckSetupScreen = New frmDeckSetupScreen

            deckSetupFrm.SetUserScreen(screenName, appFolder, appName)

            'Minimize Maestro ICP
            Me.HideHostGUI()

            'Open the Form
            deckSetupFrm.ShowDialog(Me.HostWindow)

            If deckSetupFrm.IsAbort Then
                Me.SetGlobalVariableValue("g_ApplicationAborted", True)
                Me.RaiseErrorFromVSTA(-9999, "Maestro::ShowUserScreen", " User Aborted.")
            End If
            'Show Maestro ICP
            Me.ShowHostGUI()

            'If Not m_IsSunkenDeckNGS Then
            '    Dim deckSetupFrm As frmDeckSetupScreen = New frmDeckSetupScreen

            '    deckSetupFrm.SetUserScreen(screenName, appFolder, appName)

            '    'Minimize Maestro ICP
            '    Me.HideHostGUI()

            '    'Open the Form
            '    deckSetupFrm.ShowDialog(Me.HostWindow)

            '    If deckSetupFrm.IsAbort Then
            '        Me.SetGlobalVariableValue("g_ApplicationAborted", True)
            '        Me.RaiseErrorFromVSTA(-9999, "Maestro::ShowUserScreen", " User Aborted.")
            '    End If
            '    'Show Maestro ICP
            '    Me.ShowHostGUI()
            'Else
            '    ' ----------------------------------------------------------------------------------------
            '    ' This section of code is for connecting to database. Should uncomment this section
            '    ' when running with database. However, be sure to uncomment section above
            '    '-----------------------------------------------------------------------------------------

            '    ' Check if we should upload screens to DB
            '    UploadAppScreenToDB(appFolder, appName, screenName)

            '    Dim deckSetupFrm As frmDeckSetup
            '    deckSetupFrm = New frmDeckSetup(appName)

            '    'Minimize Maestro ICP
            '    Me.HideHostGUI()

            '    deckSetupFrm.ShowDialog(Me.HostWindow)
            '    'Show Maestro ICP
            '    Me.ShowHostGUI()

            'End If
        Catch ex As Exception
            Me.SetGlobalVariableValue("g_ApplicationAborted", True)
            Me.RaiseErrorFromVSTA(-9999, "Maestro::ShowUserScreen", " Unable to retrieve global variable g_ScreenName, g_ThisApplicationScreensPath, or g_ApplicationName.")
        End Try
    End Sub

    Private Sub UploadAppScreenToDB(ByVal appFolder As String, ByVal appName As String, ByVal screenName As String)

        ' File extensions
        Dim TXT As String = ".TXT"
        Dim IMG As String = ".JPG"

        Dim path As String

        ' Object for setting/retrieving image list to/from DB
        If m_CAppDBObj Is Nothing Then
            m_CAppDBObj = New CAppDBData
        End If

        ' Check if images for this application is in DB by looking up app name.
        If Not m_CAppDBObj.CheckImageByAppname(appName) Then

            Dim imageList As List(Of System.Drawing.Image) = New List(Of System.Drawing.Image)

            Dim imageFileList As Dictionary(Of Integer, String) = New Dictionary(Of Integer, String)
            Dim txtFileList As Dictionary(Of Integer, String) = New Dictionary(Of Integer, String)

            path = appFolder & screenName
            ' Get all the files from the image directory. Note: may contain text file
            Dim fileList() As String = System.IO.Directory.GetFiles(path)

            Dim fileInfo As System.IO.FileInfo
            Dim key As Integer
            Dim min As Integer
            Dim max As Integer

            Dim i As Integer
            Dim s As String
            For i = 0 To fileList.Length - 1
                s = fileList(i)
                fileInfo = New System.IO.FileInfo(s)

                ' Separate text files from image files
                If fileInfo.Extension.ToUpper.Equals(TXT) Then
                    key = Convert.ToInt32(fileInfo.Name.ToLower.Substring(4).Replace(TXT.ToLower, String.Empty))
                    txtFileList.Add(key, s)
                ElseIf fileInfo.Extension.ToUpper.Equals(IMG) Then
                    key = Convert.ToInt32(fileInfo.Name.ToLower.Substring(7).Replace(IMG.ToLower, String.Empty))

                    imageFileList.Add(key, s)

                    If i = 0 Then
                        min = key
                    End If
                    min = Math.Min(min, key)
                    max = Math.Max(max, key)
                End If
            Next

            ' Create Image class from list
            ' Image file name should match text file name
            Dim imgPage As CImagePage
            Dim imgPath As String
            Dim imgDesc As String
            Dim imgPageList As List(Of CImagePage) = New List(Of CImagePage)
            For i = min To max
                If imageFileList.ContainsKey(i) Then
                    imgPath = imageFileList.Item(i)

                    ' Open the text file and read content
                    imgDesc = System.IO.File.ReadAllText(txtFileList.Item(i))

                    imgPage = New CImagePage
                    imgPage.DBID = Guid.NewGuid().ToString
                    imgPage.AppName = appName
                    imgPage.Description = imgDesc
                    imgPage.ImageFile = imgPath
                    imgPage.ImagePageNumber = i
                    imgPage.DeckImage = System.Drawing.Image.FromFile(imgPath)
                    imgPage.DeckID = String.Empty
                    imgPage.ImagePageChanged = True
                    imgPageList.Add(imgPage)

                End If
            Next i

            m_CAppDBObj.PushImagePageObjectsToDB(imgPageList)
        End If
    End Sub

    Public Sub SetWaitTimer()
        Try
            If m_TimerThread Is Nothing Then
                Exit Sub
            End If

            m_WaitTimer = Convert.ToBoolean(Me.GetGlobalVariableValue("g_TimerWait"))
            If m_WaitTimer Then
                m_TimerThread.Join()
            End If
        Catch ex As Exception
            Me.SetGlobalVariableValue("g_ApplicationAborted", True)
            Me.RaiseErrorFromVSTA(-9999, "Maestro::ShowUserScreen", " Unable to retrieve global variable g_TimerWait.")
        End Try
    End Sub

#End Region

#Region "Normalization"
    Public Sub ReadNormalizationWorkBook()

        Dim excelFile As String
        Dim excelSheet As String
        Try
            excelFile = Me.GetGlobalVariableValue("g_ExcelWorkBook")
            excelSheet = Me.GetGlobalVariableValue("g_ExcelWorkSheet")
            Dim normFrm As frmNorm = New frmNorm(excelFile, excelSheet)
            normFrm.ShowDialog(Me.HostWindow)

            If normFrm.IsCancel Then
                Me.SetGlobalVariableValue("g_ApplicationAborted", True)
                Me.RaiseErrorFromVSTA(-9999, "ReadNormalizationWorkBook", "User has aborted the application")
            Else
                m_NormalizationList = normFrm.NormalizationList
                Me.SetGlobalVariableValue("g_NumberOfColumnsToProcess", normFrm.NumberOfSampleColumns)

                Me.SetGlobalVariableValue("g_TotalVolume", normFrm.TotalVolume)
            End If
        Catch ex As Exception
            Me.SetGlobalVariableValue("g_ApplicationAborted", True)
            Me.RaiseErrorFromVSTA(-9999, "ReadNormalizationWorkBook", "Unable to load normalization file.")
        End Try

    End Sub

    Public Sub GetTotalNormalizationSamples()
        Try
            If m_NormalizationList Is Nothing Then
                ReadNormalizationWorkBook()
            End If
            ' This is the actual number of samples read from the spreadsheet
            Me.SetLocalVariableValue("l_NumberOfSamples", m_NormalizationList.Count)
        Catch ex As Exception
            Me.RaiseErrorFromVSTA(-9999, "Maestro::GetTotalNumberOfNormalizationSample", "Unable to set local variable l_NumberOfSamples")
        End Try
    End Sub
    Public Sub GetNextNormalizationSample()

        Dim col As Integer = 0
        Dim row As Integer = 0
        Dim maestroIndex As Integer
        Dim normClass As CNorm
        Dim concentration As Single = 0
        maestroIndex = Me.GetLocalVariableValue("l_CurrentSampleIndex")

        Try

            If m_NormalizationList Is Nothing Then
                GetTotalNormalizationSamples()
            End If
            normClass = m_NormalizationList(maestroIndex - 1)

            If Not normClass Is Nothing Then
                concentration = normClass.Sample.ConcentrationVolume
                UtilMod.GetRowColBasedOnWellID(normClass.SourceWellID, row, col)
            End If

            ' Maestro must check if values for these variables are 0 or less
            Me.SetLocalVariableValue("l_SampleRowNumber", row)
            Me.SetLocalVariableValue("l_SampleColumnNumber", col)
            Me.SetLocalVariableValue("l_SampleAspirateVolume", concentration)

        Catch ex As Exception
            Me.RaiseErrorFromVSTA(-9999, "Maestro::GetNextNormalizationSample", "Unable to set local variable l_SampleRowNumber, l_SampleColumnNumber, or l_SampleAspirateVolume")

        End Try

    End Sub

#End Region

#Region "Validate Reagent and Indexing sheets"

    ' Note: this method should only be called at the beginning of Maestro application (inside UTIL_Display_User Interface)
    ' It will load the reagent and indexing sheets as specified by the global variable g_ExcelWorkSheet and g_IndexWorkSheet
    ' This sheet must exists in the same workbook
    Public Sub ValidateWorkBook()
        Try
            ReadExcelSpreadSheet()
            ReadDNABarcodeIndex()
        Catch ex As Exception
            Me.RaiseErrorFromVSTA(-9999, "VSTA::ValidateWorkBook", "Unable to validate reagent sheet. Please check to make sure the sheet(s) are in correct format")
        End Try

    End Sub
#End Region

    Public Sub ConvertEntryPointToBoolean()
        Dim entryPointStr As String
        Dim isEntryPoint As Boolean
        Try
            entryPointStr = Me.GetParameterVariableValue("p_NGSMain_IsEntryPoint").ToString()
        Catch ex As Exception
            entryPointStr = String.Empty
        End Try

        If entryPointStr.Trim().ToUpper().Equals("TRUE") Then
            isEntryPoint = True
        Else
            isEntryPoint = False
        End If

        Me.SetLocalVariableValue("l_IsEntryPoint", isEntryPoint)
    End Sub

    Public Sub NGS_TellAlarmManagerToUsePopups()
        Me.AlarmManagerMode = eAlarmManagerMode.ePopup
    End Sub

End Class

