Imports System.Collections
Imports System.Collections.Generic
Imports System.IO
Imports CaliperLifeSciences.NGS
Imports DeckConfirmation

Partial Class Maestro
    Private Sub Maestro_Startup(ByVal sender As Object, ByVal e As System.EventArgs) Handles Me.Startup        
    End Sub

    Private Sub Maestro_Shutdown(ByVal sender As Object, ByVal e As System.EventArgs) Handles Me.Shutdown
    End Sub

    Private m_PlateList As List(Of CPlateLayout)
    Private m_Plates As CPlates
    Private m_TimerThread As Threading.Thread
    Private m_CAppDBObj As CAppDBData
    Private m_WaitTimer As Boolean
    Private m_IsSunkenDeckNGS As Boolean
    Private m_NormalizationList As List(Of CNorm)
    Private m_DeckPopulation As CDeckPopulation

    ' Vision data
    Dim _vExec As CVisionExecutor
    Dim _loc As String() = New String(4) {}

    Private _currentLayoutName As String
    Private _currentDeckLocation As String
    Private _currentExpectedNotch As String
    Private _currentExpectedColor As String
    Private _currentExpectedStackHeight As Integer

    Private _allDeckConsumablesDict As Generic.Dictionary(Of String, CConsumable)
    Private _currentConsumable As CConsumable
    Private _currentConsumableName As String
    Private _currentConsumableCategory As String
    Private _consumablesManager As CConsumablesManager

    Private _visionFeatureResults As Collection
    Private _brightness As Integer
    Private m_TitleString As String

    Public Enum HEAD_POSITION_TYPE

        RGRIPPER ' Right gripper
        LGRIPPER ' Left gripper
        GANTRY   ' Gantry

    End Enum

#Region "Reagent Distribution"

    Public Sub ReadExcelSpreadSheet()

        Dim excelFile As String
        Dim excelSheet As String
        Dim isShowSimplePlateInfo As Boolean

        Try
            m_Plates = New CPlates()
            excelFile = Path.Combine(Environment.GetFolderPath(Environment.SpecialFolder.CommonApplicationData), Me.GetGlobalVariableValue("g_ExcelWorkBook"))
            excelSheet = Me.GetGlobalVariableValue("g_ExcelWorkSheet")
            m_Plates.HostWindow = Me.HostWindow
            m_Plates.ExcelWorkBookName = excelFile

            isShowSimplePlateInfo = Convert.ToBoolean(Me.GetGlobalVariableValue("g_IsShowSimplePlateInfo"))

            If System.IO.File.Exists(excelFile) Then
                m_Plates.ReadPlateLayoutSpreadSheet(excelFile, excelSheet, isShowSimplePlateInfo)
                ' Get back the list of all the plates.

                If m_Plates.IsCancel = False Then
                    m_PlateList = m_Plates.PlateList()
                End If

                Try
                    Me.SetGlobalVariableValue("g_ApplicationAborted", m_Plates.IsCancel)
                    Me.SetGlobalVariableValue("g_NumberOfColumnsToProcess", m_Plates.NumberOfColumns)
                Catch ex As Exception
                    Me.SetGlobalVariableValue("g_ApplicationAborted", True)
                    Me.RaiseErrorFromVSTA(-9999, "Maestro::ReadExcelSpreadSheet", " Unable to write to global variables.")
                End Try

            End If
        Catch ex As Exception
            Me.SetGlobalVariableValue("g_ApplicationAborted", True)
            Me.RaiseErrorFromVSTA(-9999, "Maestro::ReadExcelSpreadSheet", " Unable to read global variables from excel.")
        End Try
    End Sub

    Public Sub GetColAndRowByReagentColumn()

        Dim reagentColumn As Integer = 0
        Dim aspirateVol As Decimal
        Dim errMessage As String = String.Empty

        Dim col As Integer
        Dim row As Integer
        Dim aspirateColStart As Integer

        Dim sourceDeckID As String = String.Empty

        ' If we haven't read the excel spreadsheet then read it now
        If m_PlateList Is Nothing Then
            ReadExcelSpreadSheet()

            If m_PlateList Is Nothing Then
                Me.SetGlobalVariableValue("g_ApplicationAborted", True)
                Me.RaiseErrorFromVSTA(-9999, "Maestro Class in VSTA::GetColAndRowByReagentColumn(): ", "Unable to read Excel spreadsheet. Make sure the file exists.")
                Exit Sub
            End If
        End If

        If m_PlateList.Count <= 0 Then
            Me.SetLocalVariableValue("g_ApplicationAborted", "true")
            Me.RaiseErrorFromVSTA(-9999, "Maestro Class in VSTA::GetColAndRowByReagentColumn(): ", "No plat layout defined.")
            Exit Sub
        End If

        sourceDeckID = Me.GetParameterVariableValue("p_AspirateDeckID")
        If sourceDeckID.Equals(String.Empty) Then
            Me.SetLocalVariableValue("g_ApplicationAborted", "true")
            Me.RaiseErrorFromVSTA(-9999, "Maestro Class in VSTA::GetColAndRowByReagentColumn(): ", "Unable to determine reagent deck ID. Please check your input parameter - p_PlateName.")
            Exit Sub
        End If

        For Each plate As CPlateLayout In m_PlateList

            If plate.PlateDeckID.ToUpper.Trim().Equals(sourceDeckID.ToUpper.Trim) Then
                reagentColumn = Convert.ToInt32(Me.GetParameterVariableValue("p_ReagentColumn"))
                If reagentColumn <= 0 Then
                    Me.RaiseErrorFromVSTA(-9999, "Maestro Class in VSTA::GetColAndRowByReagentColumn(): ", "Unable to determine reagent name. Please check your input parameter - p_ReagentName.")
                    Exit Sub
                End If

                Try
                    aspirateVol = Convert.ToDecimal(Me.GetParameterVariableValue("p_AspirateVolume"))
                Catch ex As Exception
                    Me.SetLocalVariableValue("g_ApplicationAborted", "true")
                    Me.RaiseErrorFromVSTA(-9999, "Maestro Class in VSTA::GetColAndRowByReagentColumn(): ", "Invalid input volumne. Please check your input parameter - p_AspirateVolume.")
                    Exit Sub
                End Try

                plate.GetColAndRowByReagentColumn(reagentColumn, aspirateVol, row, col, aspirateColStart, errMessage)

                If errMessage.Equals(String.Empty) Then
                    'Set local variable for number of tips to load...Row and Col
                    Me.SetLocalVariableValue("l_Row", row)
                    Me.SetLocalVariableValue("l_Col", col)
                    Me.SetLocalVariableValue("l_AspirateColStart", aspirateColStart)
                Else
                    Me.SetLocalVariableValue("g_ApplicationAborted", "true")
                    Me.RaiseErrorFromVSTA(-9999, "Maestro Class in VSTA::GetColAndRowByReagentColumn(): ", errMessage)
                End If

            End If
        Next

    End Sub

#End Region

#Region "Barcode Indexing"
    Private m_BarcodeIndex As CIndex
    Private m_SampleListForThisIndex As List(Of CSample)
    Private m_SourceWellID As String

    Public Sub ReadDNABarcodeIndex()

        m_BarcodeIndex = Nothing
        'If m_BarcodeIndex Is Nothing Then
        Dim indexWorkbook As CReadIndexWorkbook = New CReadIndexWorkbook
        Dim workBookName As String
        Dim sheetName As String

        'Dim cReadBarcode As CReadBarcode = New CReadBarcode
        'Dim xmlFile As String

        Try
            workBookName = System.IO.Path.Combine(Environment.GetFolderPath(Environment.SpecialFolder.CommonApplicationData), Me.GetGlobalVariableValue("g_ExcelWorkBook"))
            sheetName = Me.GetGlobalVariableValue("g_IndexWorkSheet")
            m_BarcodeIndex = indexWorkbook.ReadIndexWorkbook(workBookName, sheetName)

            ' NOTHING IS WRONG WITH THIS CODE. It reads from xml file instead of spreadsheet
            'xmlFile = Me.GetGlobalVariableValue("g_BarcodeXMLFile")
            'm_BarcodeIndex = cReadBarcode.ReadBarcode(xmlFile)


            If m_BarcodeIndex.MaxBarcodeIndex = 0 Then
                Me.SetGlobalVariableValue("g_ApplicationAborted", True)
                Me.RaiseErrorFromVSTA(-9999, "Maestro::ReadDNABarcodeIndex", " Unable to read barcode index spreadsheet. Please verify that the index sheet specified by the variable g_IndexWorkSheet matches the one in the workbook.")
            End If

            If m_BarcodeIndex Is Nothing Then
                Me.SetGlobalVariableValue("g_ApplicationAborted", True)
                Me.RaiseErrorFromVSTA(-9999, "Maestro::ReadDNABarcodeIndex", " Unable to read barcode index spreadsheet. Please verify that the index sheet specified by the variable g_IndexWorkSheet matches the one in the workbook.")
            End If

            Threading.Thread.Sleep(2000)
        Catch ex As Exception
            Me.SetGlobalVariableValue("g_ApplicationAborted", True)
            Me.RaiseErrorFromVSTA(-9999, "Maestro::ReadDNABarcodeIndex", " Unable to read barcode index spreadsheet.")

        End Try
        'End If
    End Sub
    Public Sub GetNumberOfDNABarcodeSamples()

        Try
            If m_BarcodeIndex Is Nothing Then
                ReadDNABarcodeIndex()
            End If

            Dim numSamples As Integer = m_BarcodeIndex.NumberOfSamples()
            Me.SetGlobalVariableValue("g_NumberOfSamples", numSamples)

        Catch ex As Exception
            Me.SetGlobalVariableValue("g_ApplicationAborted", True)
            Me.RaiseErrorFromVSTA(-9999, "Maestro::GetNumberOfDNABarcodeSamples", " Unable to get barcode sample list.")

        End Try

    End Sub

    ' Get the max barcode index
    Public Sub GetMaxBarcodeIndex()

        Try
            If m_BarcodeIndex Is Nothing Then
                ReadDNABarcodeIndex()
            End If

            Dim totalBarcodeIndex As Integer = m_BarcodeIndex.MaxBarcodeIndex
            Me.SetLocalVariableValue("l_MaxBarcodeIndex", totalBarcodeIndex)

        Catch ex As Exception

            Me.SetGlobalVariableValue("g_ApplicationAborted", True)
            Me.RaiseErrorFromVSTA(-9999, "Maestro::GetMaxBarcodeIndex", " Unable to get max barcode index.")

        End Try
    End Sub

    ' For each barcode index counter we query for IndexID that we got from the spread sheet
    Public Sub GetNextBarcodeIndexID()

        Try
            Dim indexCounter As Integer
            Dim row As Integer = 0
            Dim col As Integer = 0

            m_SourceWellID = String.Empty
            If m_SampleListForThisIndex Is Nothing Then
                m_SampleListForThisIndex = New List(Of CSample)
            Else
                m_SampleListForThisIndex.Clear()
            End If

            ' Get a list of all the samples then query for IndexID
            Dim numSamples As List(Of CSample) = m_BarcodeIndex.SampleList()

            If numSamples IsNot Nothing Then
                indexCounter = Convert.ToInt32(Me.GetLocalVariableValue("l_BarcodeIndexCounter"))

                ' Get number of samples with this index from sample list
                For Each s As CSample In numSamples
                    If s.IndexID = indexCounter Then
                        m_SampleListForThisIndex.Add(s)

                        ' Source barcode wellID should be the same if multiple samples
                        ' have the same Index ID
                        m_SourceWellID = s.SourceWellID
                    End If
                Next

                If m_SampleListForThisIndex.Count > 0 Then
                    Me.SetLocalVariableValue("l_NumIndex", m_SampleListForThisIndex.Count)
                    ' Now get row and col for this well ID
                    UtilMod.GetRowColBasedOnWellID(m_SourceWellID, row, col)
                End If

                Me.SetLocalVariableValue("l_SourceWellID", m_SourceWellID)

                ' Now get row and col for this well ID
                Me.SetLocalVariableValue("l_AspirateCol", col)
                Me.SetLocalVariableValue("l_AspirateRow", row)

            End If

        Catch ex As Exception
            Me.SetGlobalVariableValue("g_ApplicationAborted", True)
            Me.RaiseErrorFromVSTA(-9999, "Maestro::GetNextBarcodeIndexID", " Unable to retrieve barcode index ID.")

        End Try

    End Sub

    Public Sub GetSampleInfoForIndexLoopCounter()

        Try
            Dim row As Integer = 0
            Dim col As Integer = 0

            Dim subIndex As Integer = Convert.ToInt32(Me.GetLocalVariableValue("l_SubIndex"))
            Dim sample As CSample = m_SampleListForThisIndex(subIndex)

            ' Protect against number larger than current list count
            If subIndex < m_SampleListForThisIndex.Count Then

                Me.SetLocalVariableValue("l_IndexID", sample.IndexID)
                Me.SetLocalVariableValue("l_PoolID", sample.PoolID)
                Me.SetLocalVariableValue("l_SampleID", sample.SampleID)

                ' Now get row and col for this well ID
                UtilMod.GetRowColBasedOnWellID(m_SourceWellID, row, col)
                Me.SetLocalVariableValue("l_AspirateCol", col)
                Me.SetLocalVariableValue("l_AspirateRow", row)
                Me.SetLocalVariableValue("l_SourceWellID", sample.SourceWellID)

                ' Now get row and col for this well ID
                UtilMod.GetRowColBasedOnWellID(sample.DestinationWellID, row, col)

                ' This is for 384-well plate
                Select Case row
                    Case 1
                        row = row     'A
                    Case 2
                        row = row + 1 'B
                    Case 3
                        row = row + 2 'C
                    Case 4
                        row = row + 3 'D
                    Case 5
                        row = row + 4 'E
                    Case 6
                        row = row + 5 'F
                    Case 7
                        row = row + 6 'G
                    Case 8
                        row = row + 7 'H
                End Select

                Select Case col
                    Case 1
                        col = col     '1
                    Case 2
                        col = col + 1 '2
                    Case 3
                        col = col + 2 '3
                    Case 4
                        col = col + 3 '4
                    Case 5
                        col = col + 4 '5
                    Case 6
                        col = col + 5 '6
                    Case 7
                        col = col + 6 '7
                    Case 8
                        col = col + 7 '8
                    Case 9
                        col = col + 8 '9
                    Case 10
                        col = col + 9 '10
                    Case 11
                        col = col + 10 '11
                    Case 12
                        col = col + 11 '12
                End Select

                Me.SetLocalVariableValue("l_DispenseCol", col)
                Me.SetLocalVariableValue("l_DispenseRow", row)
                Me.SetLocalVariableValue("l_DestinationWellID", sample.DestinationWellID)
            End If


        Catch ex As Exception
            Me.SetGlobalVariableValue("g_ApplicationAborted", True)
            Me.RaiseErrorFromVSTA(-9999, "Maestro::GetSampleInfoForIndexLoopCounter", " Unable to to read/write Maestro local variable.")
        End Try

    End Sub

    Public Sub IsOneToOneMapping()
        Me.SetLocalVariableValue("l_IsOneToOneMapping", m_BarcodeIndex.IsOneToOneMap)
    End Sub

    ' Only use this method if IsOneToOneMapping is true
    Public Sub SetAspirateDispenseColumn()
        Me.SetLocalVariableValue("l_DispenseCol", m_BarcodeIndex.DestinationWellColumnStart)
        Me.SetLocalVariableValue("l_DispenseRow", 1)

        Me.SetLocalVariableValue("l_AspirateCol", m_BarcodeIndex.SourceWellColumnStart)
        Me.SetLocalVariableValue("l_AspirateRow", 1)

    End Sub
    Private Sub Compute384WellRowColOffSet(ByRef row As Integer, ByRef col As Integer)

        ' This is for 384-well plate
        Select Case row
            Case 1
                row = row     'A
            Case 2
                row = row + 1 'B
            Case 3
                row = row + 2 'C
            Case 4
                row = row + 3 'D
            Case 5
                row = row + 4 'E
            Case 6
                row = row + 5 'F
            Case 7
                row = row + 6 'G
            Case 8
                row = row + 7 'H
        End Select

        Select Case col
            Case 1
                col = col     '1
            Case 2
                col = col + 1 '2
            Case 3
                col = col + 2 '3
            Case 4
                col = col + 3 '4
            Case 5
                col = col + 4 '5
            Case 6
                col = col + 5 '6
            Case 7
                col = col + 6 '7
            Case 8
                col = col + 7 '8
            Case 9
                col = col + 8 '9
            Case 10
                col = col + 9 '10
            Case 11
                col = col + 10 '11
            Case 12
                col = col + 11 '12
        End Select

    End Sub

    Public Sub GetRowColOffsetFor384WellPlate()
        Dim row As Integer
        Dim col As Integer

        Try
            col = Me.GetLocalVariableValue("l_Column")
            'row = Me.GetLocalVariableValue("l_Row")

            Compute384WellRowColOffSet(row, col)

            Me.SetLocalVariableValue("l_Column", col)
            'Me.SetLocalVariableValue("l_Row", row)
        Catch ex As Exception
            Me.RaiseErrorFromVSTA(-9999, "GetRowColOffsetFor384WellPlate", "Unable to get local variables")
        End Try
    End Sub

#End Region

#Region "Appplication Image Setup"
    Public Sub InitialApplicatiomImageSetup()

        Dim isAbort As Boolean
        Dim appName As String = Nothing

        Try
            appName = Me.CurrentApplicationName
            Me.SetGlobalVariableValue("g_ApplicationName", appName)
            Dim frm As frmDeckSetup = New frmDeckSetup(appName)

            frm.ShowDialog(Me.HostWindow)
            isAbort = frm.IsAbort
            If isAbort Then

                Me.SetGlobalVariableValue("g_ApplicationAborted", True)
                Me.RaiseErrorFromVSTA("-9999", "Startup Screen", "User has Aborted")
            End If
        Catch ex As Exception
            Me.RaiseErrorFromVSTA(-9999, "Maestro::InitialApplicatiomImageSetup", " Unable to read/write application name.")
        End Try

    End Sub
#End Region

#Region "Generic"

    Public Sub GetDeckIDByDeckNumber()

        Dim deckID As String
        Dim deckNum As Integer

        Try
            deckNum = Convert.ToInt32(Me.GetLocalVariableValue("l_DeckNumber"))
            deckID = LookUpDeckNumberID(deckNum, PlatformType.SCICLONE)
            Me.SetLocalVariableValue("l_DeckID", deckID)

        Catch ex As Exception
            Me.SetGlobalVariableValue("g_ApplicationAborted", True)
            Me.RaiseErrorFromVSTA(-9999, "Maestro::GetDeckIDByDeckNumber", " Unable to read lobal variables l_DeckNumber or l_DeckID.")
        End Try
    End Sub

    ' Use in UTIL_BarcodeIndexing method, SPRI Clean up
    Public Sub ComputeTransferCycles()

        Dim MAX_TIP_VOLUME As Single = 150 ' default
        Dim totalAspVol As Single
        Dim tmp As Single
        Dim numCycle As Integer

        Try
            totalAspVol = Convert.ToInt32(Me.GetLocalVariableValue("l_TotalAspirateVol"))
            tmp = totalAspVol / MAX_TIP_VOLUME '150 is the max tips volume
            numCycle = Math.Floor(tmp) + 1

            Me.SetLocalVariableValue("l_TransCycles", numCycle)

            Try
                Dim transferVolPerCycle As Integer
                transferVolPerCycle = totalAspVol / numCycle

                Me.SetLocalVariableValue("l_TransVolPerCycle", transferVolPerCycle)
            Catch ex As Exception
                ' Suppress this error. Some methods that call this may not care about the transfer vol per cycle
            End Try
        Catch ex As Exception
            Me.RaiseErrorFromVSTA(-9999, "Maestro::ComputeTransferCycles", " Unable to compute number of transfer cycles. Make sure the following variables are defined: l_TotalAspirateVol,l_TransCycles, and l_TransVolPerCycle ")
        End Try
    End Sub

    Public Sub NGS_TellAlarmManagerToUsePopups()
        Me.AlarmManagerMode = eAlarmManagerMode.ePopup
    End Sub

    Public Sub ConvertEntryPointToBoolean()
        Dim entryPointStr As String
        Dim isEntryPoint As Boolean
        Try
            entryPointStr = Me.GetParameterVariableValue("p_NGSMain_IsEntryPoint").ToString()
        Catch ex As Exception
            entryPointStr = String.Empty
        End Try

        If entryPointStr.Trim().ToUpper().Equals("TRUE") Then
            isEntryPoint = True
        Else
            isEntryPoint = False
        End If

        Try
            Me.SetLocalVariableValue("l_IsEntryPoint", isEntryPoint)

        Catch ex As Exception
            Me.RaiseErrorFromVSTA(-9999, "VSTA::ConvertEntryPointToBoolean", "Local variable 'l_IsEntryPoint' in main method is not defined. Please define this variable and try again.")
        End Try
    End Sub

#End Region

#Region "Consumable Stack"


#End Region

#Region "Application Setup"

    Public Sub ShowSampleSetupForm()        

        Try
            Dim dlgSample As DlgSampleSetup

            Dim appName As String

            appName = Me.GetGlobalVariableValue("g_ApplicationName")
            dlgSample = New DlgSampleSetup()
            If dlgSample.ShowDialog(Me.HostWindow) = Windows.Forms.DialogResult.OK Then
                Me.SetGlobalVariableValue("g_NumberOfColumnsToProcess", dlgSample.SampleColumn)
                Me.SetGlobalVariableValue("g_IsAdapterPreBroadcast", dlgSample.IsPreBroadcastAdapters)

            Else
                Me.SetGlobalVariableValue("g_ApplicationAborted", True)
                Me.RaiseErrorFromVSTA(-9999, "ShowSampleSetupForm", "User Aborted")
            End If
        Catch ex As Exception
            Me.SetGlobalVariableValue("g_ApplicationAborted", True)
            Me.RaiseErrorFromVSTA(-9999, "ShowSampleSetupForm", "Unable to write to global variables:  g_NumberOfColumnsToProcess or g_IsAdapterPreBroadcast")
        End Try

    End Sub

    Public Sub ShowTime()
        m_WaitTimer = Convert.ToBoolean(Me.GetGlobalVariableValue("g_TimerWait"))
        'IH131022 label changed to a generally acceptable wording
        'StartThread("Incubation Time Remaining")
        StartThread("Time Remaining")
    End Sub

    Public Sub ShowBeadCaptureTime()
        m_WaitTimer = Convert.ToBoolean(Me.GetGlobalVariableValue("g_TimerWait"))
        StartThread("Bead Capture Time Remaining")
    End Sub

    Private Sub StartThread(ByVal screenTitle As String)
        m_TitleString = screenTitle
        m_TimerThread = New Threading.Thread(AddressOf StartIncubationTime)
        m_TimerThread.IsBackground = True
        m_TimerThread.Start()
        'm_TimerThread.Join() ' should be set by SetTimerWait function below

    End Sub

    Private Sub StartIncubationTime()
        'Dim delayFrm As DlgDelayTime = New DlgDelayTime
        'IH131010 Dialog in Lexogen look
        Dim delayFrm As LEXdlg_Delay_Time_dialog = New LEXdlg_Delay_Time_dialog

        Try
            delayFrm.TimeRemain = Me.GetGlobalVariableValue("g_IncubationTimer")
            delayFrm.DelayTimeTitle = m_TitleString

            delayFrm.ShowDialog(Me.HostWindow)

            If delayFrm.StopDelay Then
                ' Continue executing. Don't show alarm
                'Me.SetGlobalVariableValue("g_ApplicationAborted", True)
                'Me.RaiseErrorFromVSTA(-9999, "Maestro::ShowTime", " User has aborted the incubation time.")

            End If
        Catch ex As Exception
            Me.SetGlobalVariableValue("g_ApplicationAborted", True)
            Me.RaiseErrorFromVSTA(-9999, "Maestro::ShowTime", " Unable to retrieve global variable: g_IncubationTimer.")

        End Try
    End Sub

    Public Sub ShowUserScreen()

        ' ----------------------------------------------------------------------------------------
        ' This section of code is for case where we don't connect to database. 
        '-----------------------------------------------------------------------------------------
        Dim appFolder As String
        Dim appName As String
        Dim screenName As String

        Try
            m_IsSunkenDeckNGS = Convert.ToBoolean(Me.GetGlobalVariableValue("g_IsSunkenDeckNGS"))

            appFolder = Path.Combine(Environment.GetFolderPath(Environment.SpecialFolder.CommonApplicationData), Me.GetGlobalVariableValue("g_ThisApplicationScreensPath"))
            appName = Me.GetGlobalVariableValue("g_ApplicationName")
            screenName = Me.GetGlobalVariableValue("g_ScreenName")

            Dim deckSetupFrm As frmDeckSetupScreen = New frmDeckSetupScreen

            deckSetupFrm.SetUserScreen(screenName, appFolder, appName)

            'Minimize Maestro ICP
            'Me.HideHostGUI()

            'Open the Form
            deckSetupFrm.ShowDialog(Me.HostWindow)

            If deckSetupFrm.IsAbort Then
                Me.SetGlobalVariableValue("g_ApplicationAborted", True)
                'Me.RaiseErrorFromVSTA(-9999, "Maestro::ShowUserScreen", " User Aborted.")
            End If
            'Show Maestro ICP
            'Me.ShowHostGUI()

            'If Not m_IsSunkenDeckNGS Then
            '    Dim deckSetupFrm As frmDeckSetupScreen = New frmDeckSetupScreen

            '    deckSetupFrm.SetUserScreen(screenName, appFolder, appName)

            '    'Minimize Maestro ICP
            '    Me.HideHostGUI()

            '    'Open the Form
            '    deckSetupFrm.ShowDialog(Me.HostWindow)

            '    If deckSetupFrm.IsAbort Then
            '        Me.SetGlobalVariableValue("g_ApplicationAborted", True)
            '        Me.RaiseErrorFromVSTA(-9999, "Maestro::ShowUserScreen", " User Aborted.")
            '    End If
            '    'Show Maestro ICP
            '    Me.ShowHostGUI()
            'Else
            '    ' ----------------------------------------------------------------------------------------
            '    ' This section of code is for connecting to database. Should uncomment this section
            '    ' when running with database. However, be sure to uncomment section above
            '    '-----------------------------------------------------------------------------------------

            '    ' Check if we should upload screens to DB
            '    UploadAppScreenToDB(appFolder, appName, screenName)

            '    Dim deckSetupFrm As frmDeckSetup
            '    deckSetupFrm = New frmDeckSetup(appName)

            '    'Minimize Maestro ICP
            '    Me.HideHostGUI()

            '    deckSetupFrm.ShowDialog(Me.HostWindow)
            '    'Show Maestro ICP
            '    Me.ShowHostGUI()

            'End If
        Catch ex As Exception
            Me.SetGlobalVariableValue("g_ApplicationAborted", True)
            Me.RaiseErrorFromVSTA(-9999, "Maestro::ShowUserScreen", " Unable to retrieve global variable g_ScreenName, g_ThisApplicationScreensPath, or g_ApplicationName.")
        End Try
    End Sub

    Private Sub UploadAppScreenToDB(ByVal appFolder As String, ByVal appName As String, ByVal screenName As String)

        ' File extensions
        Dim TXT As String = ".TXT"
        Dim IMG As String = ".JPG"

        Dim path As String

        ' Object for setting/retrieving image list to/from DB
        If m_CAppDBObj Is Nothing Then
            m_CAppDBObj = New CAppDBData
        End If

        ' Check if images for this application is in DB by looking up app name.
        If Not m_CAppDBObj.CheckImageByAppname(appName) Then

            Dim imageList As List(Of System.Drawing.Image) = New List(Of System.Drawing.Image)

            Dim imageFileList As Dictionary(Of Integer, String) = New Dictionary(Of Integer, String)
            Dim txtFileList As Dictionary(Of Integer, String) = New Dictionary(Of Integer, String)

            path = appFolder & screenName
            ' Get all the files from the image directory. Note: may contain text file
            Dim fileList() As String = System.IO.Directory.GetFiles(path)

            Dim fileInfo As System.IO.FileInfo
            Dim key As Integer
            Dim min As Integer
            Dim max As Integer

            Dim i As Integer
            Dim s As String
            For i = 0 To fileList.Length - 1
                s = fileList(i)
                fileInfo = New System.IO.FileInfo(s)

                ' Separate text files from image files
                If fileInfo.Extension.ToUpper.Equals(TXT) Then
                    key = Convert.ToInt32(fileInfo.Name.ToLower.Substring(4).Replace(TXT.ToLower, String.Empty))
                    txtFileList.Add(key, s)
                ElseIf fileInfo.Extension.ToUpper.Equals(IMG) Then
                    key = Convert.ToInt32(fileInfo.Name.ToLower.Substring(7).Replace(IMG.ToLower, String.Empty))

                    imageFileList.Add(key, s)

                    If i = 0 Then
                        min = key
                    End If
                    min = Math.Min(min, key)
                    max = Math.Max(max, key)
                End If
            Next

            ' Create Image class from list
            ' Image file name should match text file name
            Dim imgPage As CImagePage
            Dim imgPath As String
            Dim imgDesc As String
            Dim imgPageList As List(Of CImagePage) = New List(Of CImagePage)
            For i = min To max
                If imageFileList.ContainsKey(i) Then
                    imgPath = imageFileList.Item(i)

                    ' Open the text file and read content
                    imgDesc = System.IO.File.ReadAllText(txtFileList.Item(i))

                    imgPage = New CImagePage
                    imgPage.DBID = Guid.NewGuid().ToString
                    imgPage.AppName = appName
                    imgPage.Description = imgDesc
                    imgPage.ImageFile = imgPath
                    imgPage.ImagePageNumber = i
                    imgPage.DeckImage = System.Drawing.Image.FromFile(imgPath)
                    imgPage.DeckID = String.Empty
                    imgPage.ImagePageChanged = True
                    imgPageList.Add(imgPage)

                End If
            Next i

            m_CAppDBObj.PushImagePageObjectsToDB(imgPageList)
        End If
    End Sub

    Public Sub SetWaitTimer()
        Try
            If m_TimerThread Is Nothing Then
                Exit Sub
            End If

            m_WaitTimer = Convert.ToBoolean(Me.GetGlobalVariableValue("g_TimerWait"))
            If m_WaitTimer Then
                m_TimerThread.Join()
            End If
        Catch ex As Exception
            Me.SetGlobalVariableValue("g_ApplicationAborted", True)
            Me.RaiseErrorFromVSTA(-9999, "Maestro::ShowUserScreen", " Unable to retrieve global variable g_TimerWait.")
        End Try
    End Sub

#End Region

#Region "Normalization"
    Public Sub ReadNormalizationWorkBook()

        Dim excelFile As String
        Dim excelSheet As String
        Try
            excelFile = System.IO.Path.Combine(Environment.GetFolderPath(Environment.SpecialFolder.CommonApplicationData), Me.GetGlobalVariableValue("g_ExcelWorkBook"))
            excelSheet = Me.GetGlobalVariableValue("g_ExcelWorkSheet")
            Dim normFrm As frmNorm = New frmNorm(excelFile, excelSheet)
            normFrm.ShowDialog(Me.HostWindow)

            If normFrm.IsCancel Then
                Me.SetGlobalVariableValue("g_ApplicationAborted", True)
                Me.RaiseErrorFromVSTA(-9999, "ReadNormalizationWorkBook", "User has aborted the application")
            Else
                m_NormalizationList = normFrm.NormalizationList
                Me.SetGlobalVariableValue("g_NumberOfColumnsToProcess", normFrm.NumberOfSampleColumns)

                Me.SetGlobalVariableValue("g_TotalVolume", normFrm.TotalVolume)
            End If
        Catch ex As Exception
            Me.SetGlobalVariableValue("g_ApplicationAborted", True)
            Me.RaiseErrorFromVSTA(-9999, "ReadNormalizationWorkBook", "Unable to load normalization file.")
        End Try

    End Sub

    Public Sub GetTotalNormalizationSamples()
        Try
            If m_NormalizationList Is Nothing Then
                ReadNormalizationWorkBook()
            End If
            ' This is the actual number of samples read from the spreadsheet
            Me.SetLocalVariableValue("l_NumberOfSamples", m_NormalizationList.Count)
        Catch ex As Exception
            Me.RaiseErrorFromVSTA(-9999, "Maestro::GetTotalNumberOfNormalizationSample", "Unable to set local variable l_NumberOfSamples")
        End Try
    End Sub
    Public Sub GetNextNormalizationSample()

        Dim col As Integer = 0
        Dim row As Integer = 0
        Dim maestroIndex As Integer
        Dim normClass As CNorm
        Dim concentration As Single = 0
        maestroIndex = Me.GetLocalVariableValue("l_CurrentSampleIndex")

        Try

            If m_NormalizationList Is Nothing Then
                GetTotalNormalizationSamples()
            End If
            normClass = m_NormalizationList(maestroIndex - 1)

            If Not normClass Is Nothing Then
                concentration = normClass.Sample.ConcentrationVolume
                UtilMod.GetRowColBasedOnWellID(normClass.SourceWellID, row, col)
            End If

            ' Maestro must check if values for these variables are 0 or less
            Me.SetLocalVariableValue("l_SampleRowNumber", row)
            Me.SetLocalVariableValue("l_SampleColumnNumber", col)
            Me.SetLocalVariableValue("l_SampleAspirateVolume", concentration)

        Catch ex As Exception
            Me.RaiseErrorFromVSTA(-9999, "Maestro::GetNextNormalizationSample", "Unable to set local variable l_SampleRowNumber, l_SampleColumnNumber, or l_SampleAspirateVolume")

        End Try

    End Sub

#End Region

#Region "Validate Reagent and Indexing sheets"

    ' Note: this method should only be called at the beginning of Maestro application (inside UTIL_Display_User Interface)
    ' It will load the reagent and indexing sheets as specified by the global variable g_ExcelWorkSheet and g_IndexWorkSheet
    ' This sheet must exists in the same workbook
    Public Sub ValidateWorkBook()
        Try            
            ReadExcelSpreadSheet()
            ReadDNABarcodeIndex()
        Catch ex As Exception
            Me.RaiseErrorFromVSTA(-9999, "VSTA::ValidateWorkBook", "Unable to validate reagent sheet. Please check to make sure the sheet(s) are in correct format")
        End Try

    End Sub
#End Region


#Region "DeckGUIConfirmation"

    Public Sub InitializeDeckData()
        'Public VSTA call to do the initial populating of the Deck Data
        _maestro = Me

        m_DeckPopulation = New CDeckPopulation

        Dim NumberOfMajorSteps As Integer
        NumberOfMajorSteps = Me.GetGlobalVariableValue("g_NumberOfMajorSteps")

        Dim MajorStepNames(NumberOfMajorSteps + 1) As String
        Dim i As Integer
        For i = 1 To NumberOfMajorSteps
            MajorStepNames(i) = Me.GetGlobalVariableValue("g_StepName" & i)
        Next

        Dim AppName As String
        AppName = Me.GetGlobalVariableValue("g_ApplicationName")
        m_DeckPopulation.InitializeMajorSteps(NumberOfMajorSteps, MajorStepNames, AppName)

    End Sub

    Public Sub PopulateDeckLocation()

        Dim Location As String
        Dim Name As String
        Dim LocationType As String
        Dim NumberOfStackedConsumables As Integer
        Dim Lid As Boolean

        Dim Inputs() As String = Split(Me.MacroParameterInputs, ",")

        Location = Inputs(0).ToUpper
        Name = Inputs(1).Trim
        LocationType = Inputs(2).Trim
        NumberOfStackedConsumables = Inputs(3)
        Lid = Inputs(4)

        If m_DeckPopulation Is Nothing Then
            Me.RaiseErrorFromVSTA(1, "PopulateDeckLocation()", "Please call the VSTA Macro InitializeDeckData prior to calling PopulateDeckLocation.")
        End If

        If Location = String.Empty Then
            Me.RaiseErrorFromVSTA(1, "Error in PopulateDeckLocation()", "PopulateDeckLocation requires a string Input Parameter 1 stipulating the Location.")
        ElseIf CheckLocation(Location) = False Then
            Me.RaiseErrorFromVSTA(1, "Error in PopulateDeckLocation()", "Location: " & Location & " is not a valid Deck Location.")
        End If

        If LocationType = String.Empty Then
            Me.RaiseErrorFromVSTA(1, "Error in PopulateDeckLocation()", "PopulateDeckLocation requires a string Input Parameter 3 stipulating the Location Type.")
        End If

        Dim MajorStep As Integer
        MajorStep = Me.GetGlobalVariableValue("g_MajorStep")

        Dim myLocationType As eLocationType
        myLocationType = LookupLocationType(LocationType)

        Dim MaestroMaterialName As String = String.Empty
        If Name <> String.Empty Then
            If DoesConsumableExist(Name) Then
                MaestroMaterialName = Location & " Consumable"
            Else
                Me.RaiseErrorFromVSTA(1, "Error in PopulateDeckLocation()", "Consumable '" & Name & "' does not exist in the Consumables Database.")
            End If
        End If

        m_DeckPopulation.AddConsumableToMajorStep( _
            MajorStep, _
            Location, _
            myLocationType, _
            Name, _
            MaestroMaterialName, _
            NumberOfStackedConsumables, _
            Lid)

    End Sub

    Public Sub PopulateMaestroDeckForStep()
        'Public VSTA call to populate the Maestro deck for a particular Major Step
        Dim LayoutName As String = "MainLayout"
        Dim MajorStep As Integer
        MajorStep = Me.GetGlobalVariableValue("g_MajorStep")

        If m_DeckPopulation Is Nothing Then
            Me.RaiseErrorFromVSTA(1, "PopulateMaestroDeckForStep()", "Please call the VSTA Macro InitializeDeckData prior to calling PopulateMaestroDeckForSetup.")
        End If

        'We default to "MainLayout" unless the layout has been specified as a VSTA parameter
        Dim stringArray() As String = Split(Me.MacroParameterInputs, ",")
        If stringArray(0) <> String.Empty Then
            LayoutName = stringArray(0)
        End If

        PopulateMaestroDeckForStep(MajorStep, LayoutName)

    End Sub

    Public Sub DisplayDeckConfirmationScreen()
        'Public VSTA call to display the Deck Confirmation screens
        Dim DeckLayout As frmDeckLayout
        Dim DeckLayoutFolder As String

        _maestro = Me

        DeckLayoutFolder = Path.Combine(Environment.GetFolderPath(Environment.SpecialFolder.CommonApplicationData), Me.GetGlobalVariableValue("g_DeckLayoutFolder"))

        Dim MajorStep As Integer
        MajorStep = Me.GetGlobalVariableValue("g_MajorStep")

        If m_DeckPopulation Is Nothing Then
            Me.RaiseErrorFromVSTA(1, "DisplayDeckConfirmationScreen()", "Please call the VSTA Macro InitializeDeckData prior to calling DisplayDeckConfirmationScreen.")
        End If

        DeckLayout = New frmDeckLayout()
        DeckLayout.PrepareContentForDisplay(DeckLayoutFolder, MajorStep, m_DeckPopulation)
        DeckLayout.ShowDialog(Me.HostWindow)
        DeckLayout.Dispose()

    End Sub

#End Region


#Region "Vision Behavior"

    'initialize camera capture
    Public Sub VisionInit()
        Try
            _vExec = New CVisionExecutor
            If Me.QueryMaestroProperty("SimulationMode", String.Empty) = "True" Then
                _vExec.init(True, Me)
            Else
                _vExec.init(False, Me)
            End If

            _currentLayoutName = String.Empty
            _currentDeckLocation = String.Empty
            _currentExpectedNotch = String.Empty
            _currentExpectedColor = String.Empty

            _allDeckConsumablesDict = New Generic.Dictionary(Of String, CConsumable)

            _consumablesManager = New CConsumablesManager()
            _consumablesManager.LoadConsumables(String.Empty)

            _currentConsumable = Nothing
            _currentConsumableName = String.Empty
            _currentConsumableCategory = String.Empty

            _visionFeatureResults = New Collection
            _brightness = 5
        Catch ex As Exception
            Me.RaiseErrorFromVSTA(1, "Error in VisionInit()", ex.Message)
        End Try
    End Sub
    'recognize the four locations starting from A2
    Public Sub VisionRecA2()
        Try
            ' ?? a3 = _allDeckConsumablesDict("A3") ' "2 * Generic SBS Plate 96"
            'fake the stack number "2 * " because it can't be retrieved from maestro yet  'Me.GetDeckVarIDAtLocation("A3", "MainLayout")
            _loc(0) = "A2"
            _loc(1) = "B2"
            _loc(2) = "A3"
            _loc(3) = "B3"

            'indicate if any of the locations has a riser
            'Dim hasRiser As Boolean() = New Boolean() {False, False, True, False}

            _vExec.RecF( _
                GetConsumableFromDict(_loc(0)), _
                GetConsumableFromDict(_loc(1)), _
                GetConsumableFromDict(_loc(2)), _
                GetConsumableFromDict(_loc(3)), _
                GetDeckLocationTypes(), _
                GetDeckLocationsNumStackedConsumables(), _
                GetDeckLocationsWithLidPresent())
        Catch ex As Exception
            Me.RaiseErrorFromVSTA(2, "Error in VisionRecA2()", ex.Message)
        End Try
    End Sub
    'recognize the four locations starting from C2
    Public Sub VisionRecC2()
        Try
            _loc(0) = "C2"
            _loc(1) = "D2"
            _loc(2) = "C3"
            _loc(3) = "D3"

            _vExec.RecF( _
                GetConsumableFromDict(_loc(0)), _
                GetConsumableFromDict(_loc(1)), _
                GetConsumableFromDict(_loc(2)), _
                GetConsumableFromDict(_loc(3)), _
                GetDeckLocationTypes(), _
                GetDeckLocationsNumStackedConsumables(), _
                GetDeckLocationsWithLidPresent())
        Catch ex As Exception
            Me.RaiseErrorFromVSTA(3, "Error in VisionRecC2()", ex.Message)
        End Try
    End Sub
    'recognize the four locations starting from C4
    Public Sub VisionRecC4()
        Try
            _loc(0) = "C4"
            _loc(1) = "D4"
            _loc(2) = "C5"
            _loc(3) = "D5"

            _vExec.RecF( _
                GetConsumableFromDict(_loc(0)), _
                GetConsumableFromDict(_loc(1)), _
                GetConsumableFromDict(_loc(2)), _
                GetConsumableFromDict(_loc(3)), _
                GetDeckLocationTypes(), _
                GetDeckLocationsNumStackedConsumables(), _
                GetDeckLocationsWithLidPresent())
        Catch ex As Exception
            Me.RaiseErrorFromVSTA(4, "Error in VisionRecC4()", ex.Message)
        End Try
    End Sub
    'recognize the four locations starting from A4
    Public Sub VisionRecA4()
        Try
            _loc(0) = "A4"
            _loc(1) = "B4"
            _loc(2) = "A5"
            _loc(3) = "B5"

            _vExec.RecF( _
                GetConsumableFromDict(_loc(0)), _
                GetConsumableFromDict(_loc(1)), _
                GetConsumableFromDict(_loc(2)), _
                GetConsumableFromDict(_loc(3)), _
                GetDeckLocationTypes(), _
                GetDeckLocationsNumStackedConsumables(), _
                GetDeckLocationsWithLidPresent())
        Catch ex As Exception
            Me.RaiseErrorFromVSTA(5, "Error in VisionRecA4()", ex.Message)
        End Try
    End Sub
    ' Private
    'retrieve location types
    Private Function GetDeckLocationTypes() As String()
        Dim majorStep As Integer
        Dim locationType As String() = New String() {String.Empty, String.Empty, String.Empty, String.Empty}

        For idx As Integer = 0 To 3
            majorStep = GetGlobalVariableValue("g_MajorStep")
            locationType(idx) = m_DeckPopulation.DeckList(majorStep).DeckLayoutList(m_DeckPopulation.LookupDeckNumber(_loc(idx))).LocationType.ToString()
        Next
        Return locationType
    End Function
    '' Private
    Private Function GetDeckLocationsNumStackedConsumables() As Integer()
        Dim numConsumables As Integer() = New Integer() {1, 1, 1, 1}

        For idx As Integer = 0 To 3
            numConsumables(idx) = GetStackedConsumablesAtLocation(_loc(idx))
        Next
        Return numConsumables
    End Function
    ' Private
    'indicate if any of the locations has a lid
    Private Function GetDeckLocationsWithLidPresent() As Boolean()
        Dim majorStep As Integer
        Dim hasLid As Boolean() = New Boolean() {False, False, False, False}

        For idx As Integer = 0 To 3
            majorStep = GetGlobalVariableValue("g_MajorStep")
            hasLid(idx) = m_DeckPopulation.DeckList(majorStep).DeckLayoutList(m_DeckPopulation.LookupDeckNumber(_loc(idx))).Lid
        Next
        Return hasLid
    End Function
    'check tip layout on a tip box and compare the detected layout with the layout set up in maestro
    Public Sub VisionTips()
        Try
            _vExec.TipDetect()
        Catch ex As Exception
            Me.RaiseErrorFromVSTA(6, "Error in VisionTips()", ex.Message)
        End Try
    End Sub
    'check the number of 384 plate on a stack
    Public Sub VisionHeight()
        Try
            Dim deckLoc As String = Me.GetGlobalVariableValue("g_VisionDeckLocation")
            _vExec.CalcHeight(deckLoc)
        Catch ex As Exception
            Me.RaiseErrorFromVSTA(7, "Error in VisionHeight()", ex.Message)
        End Try
    End Sub
    'check the notch location on a 96 plate
    Public Sub VisionNotchOn96()
        Try
            Dim deckLoc As String = Me.GetGlobalVariableValue("g_VisionDeckLocation")
            _vExec.NotchDetect_96(_currentExpectedNotch, GetStackedConsumablesAtLocation(deckLoc))
        Catch ex As Exception
            Me.RaiseErrorFromVSTA(8, "Error in VisionNotchOn96()", ex.Message)
        End Try
    End Sub
    'check the notch location on a 96 plate
    Public Sub VisionNotchOn96DWP()
        Try
            _vExec.NotchDetect_96DWP(_currentExpectedNotch, GetConsumableFromDict(GetGlobalVariableValue("g_VisionDeckLocation")))
        Catch ex As Exception
            Me.RaiseErrorFromVSTA(9, "Error in VisionNotchOn96DWP()", ex.Message)
        End Try
    End Sub
    'check the notch location on a 384 plate
    Public Sub VisionNotchOn384()
        Try
            Dim deckLoc As String = Me.GetGlobalVariableValue("g_VisionDeckLocation")
            _vExec.NotchDetect_384(_currentExpectedNotch, GetStackedConsumablesAtLocation(deckLoc))
        Catch ex As Exception
            Me.RaiseErrorFromVSTA(10, "Error in VisionNotchOn384()", ex.Message)
        End Try
    End Sub
    'check the notch location on a 4 well reservoir
    Public Sub VisionNotchOn4Well()
        Try
            _vExec.NotchDetect_4Well(_currentExpectedNotch, GetConsumableFromDict(GetGlobalVariableValue("g_VisionDeckLocation")))
        Catch ex As Exception
            Me.RaiseErrorFromVSTA(11, "Error in VisionNotchOn4Well()", ex.Message)
        End Try
    End Sub
    'check the notch location on a 12 well reservoir
    Public Sub VisionNotchOn12Well()
        Try
            _vExec.NotchDetect_12Well(_currentExpectedNotch, GetConsumableFromDict(GetGlobalVariableValue("g_VisionDeckLocation")))
        Catch ex As Exception
            Me.RaiseErrorFromVSTA(12, "Error in VisionNotchOn12Well()", ex.Message)
        End Try
    End Sub
    'check the number of tip boxes on a stack in the sunken deck
    Public Sub VisionSunkenDeckA1()
        Try
            _vExec.SunkenDeck("A1", GetConsumableFromDict("A0"), GetConsumableFromDict("B0"), GetConsumableFromDict("A1"), GetConsumableFromDict("B1"))
        Catch ex As Exception
            Me.RaiseErrorFromVSTA(13, "Error in VisionSunkenDeckA1()", ex.Message)
        End Try
    End Sub
    'check the number of tip boxes on a stack in the sunken deck
    Public Sub VisionSunkenDeckC1()
        Try
            _vExec.SunkenDeck("C1", GetConsumableFromDict("C0"), GetConsumableFromDict("D0"), GetConsumableFromDict("C1"), GetConsumableFromDict("D1"))
        Catch ex As Exception
            Me.RaiseErrorFromVSTA(14, "Error in VisionSunkenDeckC1()", ex.Message)
        End Try
    End Sub
    'release the camera capture instance for the next run
    Public Sub VisionRelease()
        Try
            _vExec.Release()
        Catch ex As Exception
            Me.RaiseErrorFromVSTA(15, "Error in VisionRelease()", ex.Message)
        End Try
    End Sub

    Public Sub VisionCheckSimulationMode()
        Try
            If Me.QueryMaestroProperty("SimulationMode", String.Empty) = "True" Then
                Me.SetGlobalVariableValue("g_VisionInSimMode", 1)
            Else
                Me.SetGlobalVariableValue("g_VisionInSimMode", 0)
            End If
        Catch ex As Exception
            Me.RaiseErrorFromVSTA(16, "Error in VisionCheckSimulationMode()", ex.Message)
        End Try
    End Sub

    Public Sub VisionSetCurrentLayoutName()
        Try
            Dim stringArray() As String = Split(Me.MacroParameterInputs, ",")
            _currentLayoutName = stringArray(0)

            OrchestratePopulatingDict()
        Catch ex As Exception
            Me.RaiseErrorFromVSTA(17, "Error in VisionSetCurrentLayoutName()", ex.Message)
        End Try
    End Sub

    Public Sub VisionSetCurrentDeckLocationAndConsumable()
        Try
            Me.SetGlobalVariableValue("g_VisionOKToProceed_SingleLocation", True) ' default
            _currentDeckLocation = GetGlobalVariableValue("g_VisionDeckLocation")

            If _allDeckConsumablesDict.ContainsKey(_currentDeckLocation) Then
                If _allDeckConsumablesDict(_currentDeckLocation) IsNot Nothing Then
                    _currentConsumable = _allDeckConsumablesDict(_currentDeckLocation)
                    _currentConsumableName = _currentConsumable.Name
                    _currentConsumableCategory = _currentConsumable.Category
                    SetGlobalVariableValue("g_VisionConsumableName", _currentConsumable.Name)
                    SetGlobalVariableValue("g_VisionConsumableCategory", _currentConsumable.Category)
                    SetGlobalVariableValue("g_VisionConsumableDensity", _currentConsumable.Density)
                    SetGlobalVariableValue("g_VisionConsumableHeight", _currentConsumable.Height)
                    SetGlobalVariableValue("g_VisionConsumableWellShape", _currentConsumable.WellShape)
                    If _currentConsumable.NotchUL = True Then
                        _currentExpectedNotch = "ul"
                    ElseIf _currentConsumable.NotchLL = True Then
                        _currentExpectedNotch = "ll"
                    ElseIf _currentConsumable.NotchUR = True Then
                        _currentExpectedNotch = "ur"
                    ElseIf _currentConsumable.NotchLR = True Then
                        _currentExpectedNotch = "lr"
                    End If
                    _currentExpectedColor = _currentConsumable.Color
                Else
                    Me.SetGlobalVariableValue("g_VisionOKToProceed_SingleLocation", False) ' no consumable at location
                    _currentConsumable = Nothing
                End If
            Else
                Me.SetGlobalVariableValue("g_VisionOKToProceed_SingleLocation", False)
                _currentConsumable = Nothing
                Throw New Exception("Deck Consumables dictionary is not properly initialized.")
            End If
        Catch ex As Exception
            Me.RaiseErrorFromVSTA(18, "Error in VisionSetCurrentDeckLocationAndConsumable()", ex.Message)
        End Try
    End Sub

    Public Sub VisionStoreSunkenDeck()
        Try
            Dim logToDB As Boolean = Me.GetGlobalVariableValue("g_VisionLogToDatabase")
            _vExec.storeDiagnoseSunkenDeck(_currentDeckLocation, _visionFeatureResults, logToDB)
        Catch ex As Exception
            Me.RaiseErrorFromVSTA(19, "Error in VisionStoreSunkenDeck()", ex.Message)
        End Try
    End Sub

    Public Sub VisionStoreRec()
        Try
            Dim logToDB As Boolean = Me.GetGlobalVariableValue("g_VisionLogToDatabase")
            _currentDeckLocation = _loc(0)
            _vExec.storeRec(_currentDeckLocation, (_loc(0) & "|" & _loc(1) & "|" & _loc(2) & "|" & _loc(3)), _visionFeatureResults, logToDB)
        Catch ex As Exception
            Me.RaiseErrorFromVSTA(20, "Error in VisionStoreRec()", ex.Message)
        End Try
    End Sub

    Public Sub VisionStoreNotch()
        Try
            Dim logToDB As Boolean = Me.GetGlobalVariableValue("g_VisionLogToDatabase")
            _vExec.StoreNotch(_currentDeckLocation, _currentExpectedNotch, _visionFeatureResults, logToDB)
        Catch ex As Exception
            Me.RaiseErrorFromVSTA(21, "Error in VisionStoreNotch()", ex.Message)
        End Try
    End Sub

    Public Sub VisionStoreHeight()
        Try
            Dim numConsumables As Integer = Me.GetStackedConsumablesAtLocation(_currentDeckLocation)
            Dim logToDB As Boolean = Me.GetGlobalVariableValue("g_VisionLogToDatabase")
            _vExec.storeHeight(_currentDeckLocation, numConsumables.ToString(), _visionFeatureResults, logToDB)
        Catch ex As Exception
            Me.RaiseErrorFromVSTA(22, "Error in VisionStoreHeight()", ex.Message)
        End Try
    End Sub

    Public Sub VisionStoreTips()
        Try
            ' For now, ALL tips are expected...if and when a subset must be
            ' specified, use Get/SetTipsAvailable and a string variable
            Dim expectedTips As String = "ALL" ' Me.GetGlobalVariableValue("g_VisionTips")
            Dim logToDB As Boolean = Me.GetGlobalVariableValue("g_VisionLogToDatabase")
            _vExec.storeTips(_currentDeckLocation, expectedTips, _visionFeatureResults, logToDB)
        Catch ex As Exception
            Me.RaiseErrorFromVSTA(23, "Error in VisionStoreTips()", ex.Message)
        End Try
    End Sub

    ' Private
    Private Sub OrchestratePopulatingDict()
        For idx As Integer = 0 To 5
            PopulateDictEntry("A" & idx.ToString())
            PopulateDictEntry("B" & idx.ToString())
            PopulateDictEntry("C" & idx.ToString())
            PopulateDictEntry("D" & idx.ToString())
        Next
    End Sub

    ' Private
    Private Sub PopulateDictEntry(ByVal deckLocation As String)
        Dim consumableName As String = Me.GetDeckVarIDAtLocation(deckLocation, _currentLayoutName)
        If Not consumableName = String.Empty Then
            _allDeckConsumablesDict.Add(deckLocation, _consumablesManager.Consumable(consumableName))
        Else
            _allDeckConsumablesDict.Add(deckLocation, Nothing)
        End If
    End Sub

    Public Sub VisionCheckIfOkToProceed()
        Try
            Dim anyVisionErrors As Boolean = False

            ' there are 1 or more vision errors, interact with user
            For Each vfd As CVisionFeature In _visionFeatureResults
                If vfd.ErrorFlag = True Then
                    anyVisionErrors = True

                    ' only show diagnostic screens when errors were found
                    If vfd.Feature = StaticClass.FEATURE_DESIGNATION_HEIGHT384 Then
                        _vExec.DiagnoseHeight(vfd)
                    ElseIf vfd.Feature = StaticClass.FEATURE_DESIGNATION_NOTCH Then
                        If _allDeckConsumablesDict(vfd.DeckLocation).Category = "Deep Well" Then
                            _vExec.DiagnoseNotch(vfd)
                        Else
                            If _allDeckConsumablesDict(vfd.DeckLocation).Density = 96 Then
                                _vExec.DiagnoseNotch_96(vfd)
                            Else
                                _vExec.DiagnoseNotch(vfd)
                            End If
                        End If
                    ElseIf vfd.Feature = StaticClass.FEATURE_DESIGNATION_RECOGNITION Then
                        _vExec.DiagnoseRec(vfd)
                    ElseIf vfd.Feature = StaticClass.FEATURE_DESIGNATION_SUNKENDECK Then
                        _vExec.DiagnoseSunkenDeck(vfd)
                    ElseIf vfd.Feature = StaticClass.FEATURE_DESIGNATION_TIP Then
                        _vExec.DiagnoseTip(vfd)
                    End If
                End If
            Next
            Me.SetGlobalVariableValue("g_ApplicationAborted", "False")
            If anyVisionErrors = True Then
                ' ask user if they want to proceed
                Dim obj As New OKToProceedForm
                obj.ShowDialog(Me.HostWindow)
                If obj.OKToProceed = False Then
                    Me.SetGlobalVariableValue("g_ApplicationAborted", "True")
                End If
            End If
        Catch ex As Exception
            Me.RaiseErrorFromVSTA(24, "Error in VisionCheckIfOkToProceed()", ex.Message)
        End Try
    End Sub

    Public Sub VisionOKToProceed_CheckSingleLocation()
        Try
            Dim singleDeckLoc As String = Me.GetGlobalVariableValue("g_VisionDeckLocation")
            Dim recDeckLoc As String = String.Empty ' deck location that defines the recognition of interest
            If singleDeckLoc = "A2" Or singleDeckLoc = "A3" Or singleDeckLoc = "B2" Or singleDeckLoc = "B3" Then
                recDeckLoc = "A2"
            ElseIf singleDeckLoc = "C2" Or singleDeckLoc = "C3" Or singleDeckLoc = "D2" Or singleDeckLoc = "D3" Then
                recDeckLoc = "C2"
            ElseIf singleDeckLoc = "A4" Or singleDeckLoc = "A5" Or singleDeckLoc = "B4" Or singleDeckLoc = "B5" Then
                recDeckLoc = "A4"
            ElseIf singleDeckLoc = "C4" Or singleDeckLoc = "C5" Or singleDeckLoc = "D4" Or singleDeckLoc = "D5" Then
                recDeckLoc = "C4"
            End If

            Dim indexOfLocation As Integer = 0
            If singleDeckLoc = "A2" Or singleDeckLoc = "C2" Or singleDeckLoc = "A4" Or singleDeckLoc = "C4" Then
                indexOfLocation = 0
            ElseIf singleDeckLoc = "B2" Or singleDeckLoc = "D2" Or singleDeckLoc = "B4" Or singleDeckLoc = "D4" Then
                indexOfLocation = 1
            ElseIf singleDeckLoc = "A3" Or singleDeckLoc = "C3" Or singleDeckLoc = "A5" Or singleDeckLoc = "C5" Then
                indexOfLocation = 2
            ElseIf singleDeckLoc = "B3" Or singleDeckLoc = "B5" Or singleDeckLoc = "D3" Or singleDeckLoc = "D5" Then
                indexOfLocation = 3
            End If

            ' check to see if the (4 location) recognition found the proper
            ' consumable at the deck location (singleDeckLoc) of interest
            Me.SetGlobalVariableValue("g_VisionOKToProceed_SingleLocation", True) ' default
            Dim proceedFlag As Boolean = True
            For Each vfd As CVisionFeature In _visionFeatureResults
                If vfd.Feature = StaticClass.FEATURE_DESIGNATION_RECOGNITION And _
                   vfd.DeckLocation = recDeckLoc And _
                   vfd.ErrorFlag = True Then
                    If vfd.RR.errors(indexOfLocation) = True Then
                        ' this 1 of 4 location does not contain the proper consumable
                        Me.SetGlobalVariableValue("g_VisionOKToProceed_SingleLocation", False)
                        proceedFlag = False
                    End If
                End If
            Next
            If proceedFlag = True Then
                VisionSetCameraPropertiesBasedOnDeckLoc()

                ' if there's fixed height added to the deck, Z will be subsequently adjusted up
                Dim majorStep As Integer = GetGlobalVariableValue("g_MajorStep")
                If Not m_DeckPopulation.DeckList(majorStep).DeckLayoutList(m_DeckPopulation.LookupDeckNumber(singleDeckLoc)).LocationType = eLocationType.PlateLocator Then
                    Me.SetGlobalVariableValue("g_VisionSingleLocation_HasRiser", True)
                Else
                    Me.SetGlobalVariableValue("g_VisionSingleLocation_HasRiser", False)
                End If
            End If
        Catch ex As Exception
            Me.RaiseErrorFromVSTA(25, "Error in VisionOKToProceed_CheckSingleLocation()", ex.Message)
        End Try
    End Sub

    Public Sub VisionSetCameraPropertiesBasedOnDeckLoc()
        Dim singleDeckLoc As String = Me.GetGlobalVariableValue("g_VisionDeckLocation")
        If singleDeckLoc = "A2" Or singleDeckLoc = "A3" Or singleDeckLoc = "A4" Then
            _brightness = 5
        ElseIf singleDeckLoc = "B2" Or singleDeckLoc = "B3" Or singleDeckLoc = "B4" Then
            _brightness = 5
        ElseIf singleDeckLoc = "C2" Or singleDeckLoc = "C3" Or singleDeckLoc = "C4" Then
            _brightness = 5
        Else
            _brightness = 5
        End If
        _vExec.Brightness = _brightness

    End Sub

    ' Private
    Private Function GetConsumableFromDict(ByVal deckLocation As String) As CConsumable
        Dim retCons As CConsumable = Nothing
        If _allDeckConsumablesDict.ContainsKey(deckLocation) Then
            retCons = _allDeckConsumablesDict(deckLocation)
        End If
        Return retCons
    End Function

    Public Sub VisionDoLightingEval()
        Try
            Dim stringArray() As String = Split(Me.MacroParameterInputs, ",")
            Dim deckLoc As String = stringArray(0)

            Dim currVals As String = String.Empty
            Dim currQuadVals As String = String.Empty

            _vExec.VisionDoLightingEvaluation(currVals, currQuadVals)

            Dim currValsArr() As String = currVals.Split(",")
            Dim avgVals As String = Me.GetGlobalVariableValue("g_VisionLighting_" & deckLoc)
            Dim avgValsArr() As String = avgVals.Split(",")

            Dim tempValB As Integer = 0
            Dim tempValG As Integer = 0
            Dim tempValR As Integer = 0
            If Convert.ToInt32(avgValsArr(0)) = 0 Then
                tempValB = (Convert.ToInt32(currValsArr(0)))
                tempValG = (Convert.ToInt32(currValsArr(1)))
                tempValR = (Convert.ToInt32(currValsArr(2)))
            Else
                tempValB = ((Convert.ToInt32(currValsArr(0)) + Convert.ToInt32(avgValsArr(0))) / 2)
                tempValG = ((Convert.ToInt32(currValsArr(1)) + Convert.ToInt32(avgValsArr(1))) / 2)
                tempValR = ((Convert.ToInt32(currValsArr(2)) + Convert.ToInt32(avgValsArr(2))) / 2)
            End If

            Me.SetGlobalVariableValue("g_VisionLighting_" & deckLoc, _
                tempValB.ToString() & "," & tempValG.ToString() & "," & tempValR.ToString())

            Me.SetGlobalVariableValue("g_VisionLighting_" & deckLoc & "Quads", currQuadVals)
        Catch ex As Exception
            Me.RaiseErrorFromVSTA(26, "Error in VisionDoLightingEval()", ex.Message)
        End Try
    End Sub

    Public Sub VisionGetNumStackedConsumablesAtLocation()
        Try
            Dim deckLocation As String = Me.GetGlobalVariableValue("g_VisionDeckLocation")
            Dim numConsumables As Integer = Me.GetStackedConsumablesAtLocation(deckLocation)

            Me.SetGlobalVariableValue("g_VisionNumStackedConsumables", numConsumables)
        Catch ex As Exception
            Me.RaiseErrorFromVSTA(27, "Error in VisionGetNumStackedConsumablesAtLocation()", ex.Message)
        End Try
    End Sub

#End Region

#Region "TIP BOX MANAGEMENT"

    Private m_DeckInfoList As Dictionary(Of String, CDeckInfo)
    Private m_PickDeckID As String
    Private m_CurDeckInfo As CDeckInfo

    ' We are only interested in the Z height. This variable save column 1 coordinate
    Private m_TaughtCoordinateOnColumn0 As CCoordinate

    Private m_MainLayout As String

    ' This function should be called when g_numberOfReservedTipBox is 0, and should be called
    ' by Maestro method "UTIL_TIP_InitializeTipsManagementGlobalVariables"
    Public Sub ResetTipBoxObject()
        m_DeckInfoList = Nothing
        m_DeckInfoList = New Dictionary(Of String, CDeckInfo)
        m_CurDeckInfo = Nothing
        m_IsSunkenDeckNGS = Convert.ToBoolean(Me.GetGlobalVariableValue("g_IsSunkenDeckNGS"))
    End Sub

    ' Recursive function to set tip load deck location
    Public Sub GetNextAvailableTipBoxLocation()

        Dim colNum As Integer
        Dim deckAlpha As String
        Dim tipBoxDeckInfo As CDeckInfo
        Dim deckLoc As String

        Dim rightDeckInfo As CDeckInfo
        Dim rightDeckZHeight As Double

        ' With one PCR Plate, this is how low we can go down before crashing
        Dim GANTRY_SAFE_PICK_HEIGHT_LOWER_LIMIT As Integer = -137
        ' With Deep Well plate this is the highest we can stay up before crashing
        Dim GANTRY_SAFE_PICK_HEIGHT_UPPER_LIMIT As Integer = -82
        Dim INHECO_BASE_HEIGHT As Integer = 18 ' 18mm from base

        ' Get layout from Maestro
        Dim layout As String = Me.GetGlobalVariableValue("g_MainLayout")

        ' Update all decks to have latest information
        UpdateDeckProperties(layout)

        ' Send message to display that we don't have anymore pickable
        ' tip boxes to pick
        If m_DeckInfoList.Count = 0 Then
            System.Windows.Forms.MessageBox.Show("No more reserved Tip Boxes remaining. Please replenish all Tip Box locations (A0, A1, B0, B1, C0, C1, D0, and D1. Click OK to continue.")
        End If

        ' Always start picking from A1. 
        ' The variable "g_SpareTipBoxIndexHolderLocation" must be initialize to A1 in Maestro application
        If m_CurDeckInfo Is Nothing Then
            m_PickDeckID = Me.GetGlobalVariableValue("g_SpareTipBoxIndexHolderLocation")
        Else
            m_PickDeckID = m_CurDeckInfo.DeckID
        End If

        ' Check deck immediate to the right to see if  tip box is pick-able. 
        ' For example if m_PickDeckID = "A1" then we need to check A2
        tipBoxDeckInfo = m_DeckInfoList.Item(m_PickDeckID)
        tipBoxDeckInfo.NumStackedPlates = Me.GetLocalVariableValue("l_NumberOfTipBoxPerDeck")

        colNum = Convert.ToInt32(m_PickDeckID.Substring(1))
        deckAlpha = m_PickDeckID.Remove(1).Trim()
        deckLoc = deckAlpha & (colNum + 1)
        rightDeckInfo = m_DeckInfoList.Item(deckLoc)


        ' ===================================================================================
        ' HERE WE CHECK EACH DECK TO MAKE SURE WE DON'T CRASH BEFORE PICKING
        ' TIP BOX FROM RECESS DECK
        ' ===================================================================================

        ' If the current tip box deck is not empty then look to the right
        ' before pick
        If Not tipBoxDeckInfo.IsEmpty Then

            rightDeckInfo.IsSunkenDeckNGS = m_IsSunkenDeckNGS
            rightDeckZHeight = rightDeckInfo.PickZHeight()

            If rightDeckInfo.ConsumableHeight = 0 Then
                rightDeckInfo.IsSunkenDeckNGS = m_IsSunkenDeckNGS

            End If
            ' NOTE: if consumable definition is not defined for the deck immediately to
            ' right of the tip box deck we should be safe to pick the last tip box.
            If tipBoxDeckInfo.NumStackedPlates = 3 Or rightDeckInfo.ConsumableHeight = 0 Then
                ' OK to pick
                m_DeckInfoList.Item(m_PickDeckID).IsConflict = False

            ElseIf tipBoxDeckInfo.NumStackedPlates = 2 Then
                ' -82mm is the cut off height based on experiment with a 2mL deep well plate
                ' Any consumable higher than -82mm the tip box is unpickable.
                If rightDeckZHeight > GANTRY_SAFE_PICK_HEIGHT_UPPER_LIMIT Then
                    ' Unpickable. will crash the head into consumable
                    m_DeckInfoList.Item(m_PickDeckID).IsConflict = True
                ElseIf rightDeckZHeight < GANTRY_SAFE_PICK_HEIGHT_UPPER_LIMIT Then

                    'Set the current deck flag to unpickable
                    m_DeckInfoList.Item(m_PickDeckID).IsConflict = False
                End If

            ElseIf tipBoxDeckInfo.NumStackedPlates = 1 Then

                Dim isConflict As Boolean = False
                If m_PickDeckID.Equals("D0") Then

                    ' Get consumable height on D2 NOT D1
                    Dim D2Cons As CDeckInfo = m_DeckInfoList("D2")
                    Dim consHeight As Double = D2Cons.ConsumableHeight
                    Dim consZHeight As Double = D2Cons.PickZHeight + (-INHECO_BASE_HEIGHT)

                    If consZHeight < GANTRY_SAFE_PICK_HEIGHT_UPPER_LIMIT Then
                        isConflict = True
                    ElseIf consZHeight > GANTRY_SAFE_PICK_HEIGHT_UPPER_LIMIT Then
                        isConflict = False
                    End If

                Else
                    If rightDeckZHeight > GANTRY_SAFE_PICK_HEIGHT_LOWER_LIMIT Then
                        ' Unpickable. will crash the head into consumable
                        'Set the current deck flag to unpickable
                        isConflict = True
                    ElseIf rightDeckZHeight < GANTRY_SAFE_PICK_HEIGHT_LOWER_LIMIT Then
                        ' OK to pick
                        'Set the current deck flag to unpickable
                        isConflict = False
                    End If
                End If

                m_DeckInfoList.Item(m_PickDeckID).IsConflict = isConflict
            End If

            ' save this for updating l_NumberOfTipBoxPerDeck
            m_CurDeckInfo = tipBoxDeckInfo
        Else
            ' Remove empty tip box location from the list
            m_DeckInfoList.Remove(m_PickDeckID)
        End If

        ' ===================================================================================
        ' IF THE CURRENT DECK IS NOT AVAILABLE THEN CHECK THE NEXT AVAILABLE DECK
        ' A TIP BOX DECK IS AVAILABLE WHEN IT IS SAFE FOR THE GRIPPER TO PICK.
        ' ===================================================================================

        'Get the next available tip box
        If m_DeckInfoList.Item(m_PickDeckID).IsConflict = True Or m_DeckInfoList.Item(m_PickDeckID).NumStackedPlates = 0 Then 'If isSafeToPick = False Then

            ' Set the next m_PickDeckID
            Dim priorDeckID As String = m_PickDeckID
            For Each kvp As KeyValuePair(Of String, CDeckInfo) In m_DeckInfoList
                tipBoxDeckInfo = kvp.Value
                ' Do this for recess deck columns only (0 and 1)
                ' Return the first available tip box location we see
                colNum = Convert.ToInt32(tipBoxDeckInfo.DeckID.Substring(1))
                If colNum < 2 And tipBoxDeckInfo.IsConflict = False And tipBoxDeckInfo.NumStackedPlates > 0 Then
                    m_PickDeckID = tipBoxDeckInfo.DeckID

                    ' Get the z height of the deck to the right of this tip box location
                    colNum = Convert.ToInt32(m_PickDeckID.Substring(1))
                    deckAlpha = m_PickDeckID.Remove(1).Trim()
                    deckLoc = deckAlpha & (colNum + 1)
                    rightDeckInfo = m_DeckInfoList.Item(deckLoc)
                    rightDeckZHeight = rightDeckInfo.PickZHeight()

                    ' save this for updating l_NumberOfTipBoxPerDeck
                    m_CurDeckInfo = tipBoxDeckInfo

                    Exit For
                End If
            Next


            If priorDeckID.Equals(m_PickDeckID) Then
                ' Get out of this function if tip box locations are no longer available.
                ' Need to send message to display that we don't have anymore pickable
                ' tip box
                Me.SetGlobalVariableValue("g_NumberOfReservedTipBox", 0)
                m_CurDeckInfo = Nothing

                Exit Sub
            Else

                UpdateMaestroVariables(m_PickDeckID, tipBoxDeckInfo.NumStackedPlates)
                Exit Sub

            End If

            ' Call itself again if the next tip box is not available
            GetNextAvailableTipBoxLocation()
        Else

            UpdateMaestroVariables(m_PickDeckID, tipBoxDeckInfo.NumStackedPlates)

        End If


    End Sub

    Private Sub UpdateMaestroVariables(ByVal curDeckID As String, _
                                        ByVal numStacked As Integer)

        Dim ZERO_COLUMN_Y_PICK_OFFSET As Integer = 143
        Dim deckAlpha As String
        Dim colNum As Integer
        Dim deckID As String

        Select Case m_PickDeckID.Trim
            Case "A0", "B0", "C0", "D0"
                Me.SetGlobalVariableValue("g_ReservedTipBoxYOffset", ZERO_COLUMN_Y_PICK_OFFSET)
            Case Else
                Me.SetGlobalVariableValue("g_ReservedTipBoxYOffset", 0)
        End Select

        colNum = Convert.ToInt32(m_PickDeckID.Substring(1))
        deckAlpha = m_PickDeckID.Remove(1).Trim()
        If colNum = 0 Then
            colNum = colNum + 1 ' we can't pick from A0. we need to pick from A1 with y offset
        End If
        deckID = deckAlpha & colNum

        ' Save this for next time
        Me.SetGlobalVariableValue("g_SpareTipBoxIndexHolderLocation", deckID)
        Me.SetLocalVariableValue("l_NumberOfTipBoxPerDeck", numStacked)

    End Sub

    Private Sub GenerateDeckList(ByVal upperLimit As Integer, ByVal lowLimit As Integer, ByVal layout As String)

        Dim deckIDList As ArrayList = New ArrayList()

        deckIDList.Add("A")
        deckIDList.Add("B")
        deckIDList.Add("C")
        deckIDList.Add("D")

        Dim cDeck As CDeckInfo
        Dim deckLoc As String
        Dim numStacked As Integer
        Dim consumableHeight As Double
        Dim taughtCoordinate As CCoordinate = New CCoordinate
        Dim deckNum As Integer = 0

        Dim numTipBoxPerDeck As Integer = Me.GetGlobalVariableValue("g_NumberOfTipBoxPerDeck")
        Dim tmpDeckLoc As String

        Dim tipBoxHeight As Double

        For i As Integer = upperLimit To lowLimit Step -1 ' 1 through 0 columns on the Sciclone deck
            For Each deckLoc In deckIDList ' A through D rows on the Sciclone deck
                tmpDeckLoc = deckLoc & i

                cDeck = New CDeckInfo(tmpDeckLoc)
                GetDeckInfo(tmpDeckLoc, layout, numStacked, consumableHeight, taughtCoordinate)

                cDeck.NumStackedPlates = numTipBoxPerDeck

                cDeck.LayoutName = layout
                cDeck.DeckNumber = deckNum

                ' We can't get consumable information from column 0. 
                If tmpDeckLoc = "A0" Or tmpDeckLoc = "B0" Or tmpDeckLoc = "C0" Or tmpDeckLoc = "D0" Then
                    consumableHeight = tipBoxHeight
                End If

                cDeck.ConsumableHeight = consumableHeight

                ' Save consumable height for column 0
                If tmpDeckLoc = "A1" Then
                    tipBoxHeight = consumableHeight
                End If

                ' This is only if we don't have a taught coordinate, particularly column 0
                If taughtCoordinate.ZCoordinate = 0 And m_TaughtCoordinateOnColumn0 IsNot Nothing Then
                    taughtCoordinate.ZCoordinate = m_TaughtCoordinateOnColumn0.ZCoordinate
                    taughtCoordinate.YCoordinate = m_TaughtCoordinateOnColumn0.YCoordinate
                    taughtCoordinate.XCoordinate = m_TaughtCoordinateOnColumn0.XCoordinate
                End If

                cDeck.TaughtCoordinate = taughtCoordinate

                If m_DeckInfoList Is Nothing Then
                    m_DeckInfoList = New Dictionary(Of String, CDeckInfo)
                End If
                m_DeckInfoList.Add(tmpDeckLoc, cDeck)


                ' Save coordinate for column 0 since we can't teach that column. Use value from column 1.
                If tmpDeckLoc = "A1" Then
                    m_TaughtCoordinateOnColumn0 = New CCoordinate
                    m_TaughtCoordinateOnColumn0.ZCoordinate = taughtCoordinate.ZCoordinate
                    m_TaughtCoordinateOnColumn0.YCoordinate = taughtCoordinate.YCoordinate
                    m_TaughtCoordinateOnColumn0.XCoordinate = taughtCoordinate.XCoordinate
                End If

                deckNum = deckNum + 1
            Next
        Next

    End Sub

    Private Sub GetDeckInfo(ByVal deckLoc As String, _
                            ByVal deckLayout As String, _
                            ByRef numStacked As Integer, _
                            ByRef consumableHeight As Double, _
                            ByRef taughtCoordinate As CCoordinate)

        Dim consumabeName As String = String.Empty
        Dim consumable As CaliperLifeSciences.Shared.Consumables.clsConsumable

        consumabeName = Me.GetDeckVarIDAtLocation(deckLoc, deckLayout)

        If m_MasterConsumablesList Is Nothing Then
            consumablesAPI = New CaliperLifeSciences.Shared.Consumables.clsConsumablesAPI
            consumablesAPI.Get_All_Consumables(m_MasterConsumablesList)
        End If

        consumable = GetMasterConsumable(consumabeName)

        If Not consumable Is Nothing Then
            consumableHeight = consumable.Height
        Else
            consumableHeight = 0 ' No consumable defined for this deck
        End If


        ' Taugh Location
        Dim x As Double = 0.0F
        Dim y As Double = 0.0F
        Dim z As Double = 0.0F
        ' Some decks are not taught...such as A0, B0, C0, D0 so set coordinate to 0. 
        Try
            Me.GetTaughtPosition(deckLoc, "MainArray", x, y, z)
            taughtCoordinate.XCoordinate = x
            taughtCoordinate.YCoordinate = y
            taughtCoordinate.ZCoordinate = z
        Catch ex As Exception
            taughtCoordinate.XCoordinate = 0.0F
            taughtCoordinate.YCoordinate = 0.0F
            taughtCoordinate.ZCoordinate = 0.0F
        End Try


    End Sub

    Private Sub UpdateDeckProperties(ByVal layout As String)

        Dim deckInfo As CDeckInfo
        Dim numStacked As Integer
        Dim consumableHeight As Double
        Dim taughtCoordinate As CCoordinate = New CCoordinate
        Dim deckNum As Integer = 0
        Dim colNum As Integer

        If m_DeckInfoList Is Nothing Then

            ' Generate information about column 1 and 0
            GenerateDeckList(1, 0, layout)

            ' Generate information about column 5 through 2
            GenerateDeckList(5, 2, layout)
        End If

        If m_DeckInfoList.Count = 0 Then

            ' Generate information about column 1 and 0
            GenerateDeckList(1, 0, layout)

            ' Generate information about column 5 through 2
            GenerateDeckList(5, 2, layout)
        End If

        For Each kvp As KeyValuePair(Of String, CDeckInfo) In m_DeckInfoList

            deckInfo = kvp.Value

            ' Get column number
            colNum = Convert.ToInt32(deckInfo.DeckID.Substring(1))

            ' Only update if not Sciclone deck column 0 and 1, 
            If colNum > 1 Then
                GetDeckInfo(deckInfo.DeckID, deckInfo.LayoutName, numStacked, consumableHeight, taughtCoordinate)
                deckInfo.NumStackedPlates = numStacked
                deckInfo.ConsumableHeight = consumableHeight

            Else ' Update tip box deck locations (column 0 through 1)

                If m_CurDeckInfo IsNot Nothing Then
                    If m_CurDeckInfo.DeckID = deckInfo.DeckID Then
                        deckInfo.NumStackedPlates = Me.GetLocalVariableValue("l_NumberOfTipBoxPerDeck")
                    End If
                End If
            End If
        Next
    End Sub

#End Region

End Class

